@@ -26,6 +26,8 @@
 #define ALSA_PCM_NEW_SW_PARAMS_API
 
 #include <alsa/asoundlib.h>
+#include <alsa/control.h>
+#include <alsa/error.h>
 #include <gst/gst.h>
 
 
@@ -100,7 +102,7 @@ enum {
   GST_ALSA_OPEN = GST_ELEMENT_FLAG_LAST,
   GST_ALSA_RUNNING,
   GST_ALSA_CAPS_NEGO,
-  GST_ALSA_FLAG_LAST = GST_ELEMENT_FLAG_LAST + 3,
+  GST_ALSA_FLAG_LAST = GST_ELEMENT_FLAG_LAST + 3
 };
 
 typedef enum {
@@ -154,11 +156,12 @@ struct _GstAlsa {
 
   /* clocking */
   GstAlsaClock *		clock;		/* our provided clock */
-  snd_pcm_uframes_t		transmitted; 	/* samples transmitted since last sync 
+  snd_pcm_uframes_t		played; 	/* samples transmitted since last sync 
 						   This thing actually is our master clock.
 						   We will event insert silent samples or
 						   drop some to sync to incoming timestamps.
 						 */
+  snd_pcm_uframes_t		captured;
   GstClockTime			max_discont;	/* max difference between current
   						   playback timestamp and buffers timestamps
 						 */
@@ -209,6 +212,13 @@ inline GstClockTime		gst_alsa_bytes_to_timestamp 	(GstAlsa *		this,
 inline guint			gst_alsa_timestamp_to_bytes 	(GstAlsa *		this,
 								 GstClockTime	 	time);
 
+/* debugging functions (useful in gdb) - require running with --gst-debug=alsa:4 or better */
+void 				gst_alsa_sw_params_dump		(GstAlsa *		this, 
+								 snd_pcm_sw_params_t *	sw_params);
+void				gst_alsa_hw_params_dump		(GstAlsa *		this, 
+								 snd_pcm_hw_params_t *	hw_params);
+
+
 G_END_DECLS
 
 #endif /* __GST_ALSA_H__ */
