@@ -43,7 +43,6 @@ static int gst_alsa_src_read (GstAlsa * this, snd_pcm_sframes_t * avail);
 static void gst_alsa_src_loop (GstElement * element);
 static void gst_alsa_src_flush (GstAlsaSrc * src);
 static GstElementStateReturn gst_alsa_src_change_state (GstElement * element);
-
 static GstClockTime gst_alsa_src_get_time (GstAlsa * this);
 
 static GstAlsa *src_parent_class = NULL;
@@ -79,7 +78,8 @@ gst_alsa_src_get_type (void)
     };
 
     alsa_src_type =
-        g_type_register_static (GST_TYPE_ALSA, "GstAlsaSrc", &alsa_src_info, 0);
+        g_type_register_static (GST_TYPE_ALSA_MIXER, "GstAlsaSrc",
+        &alsa_src_info, 0);
   }
   return alsa_src_type;
 }
@@ -109,7 +109,7 @@ gst_alsa_src_class_init (gpointer g_class, gpointer class_data)
   alsa_class = (GstAlsaClass *) klass;
 
   if (src_parent_class == NULL)
-    src_parent_class = g_type_class_ref (GST_TYPE_ALSA);
+    src_parent_class = g_type_class_ref (GST_TYPE_ALSA_MIXER);
 
   alsa_class->stream = SND_PCM_STREAM_CAPTURE;
   alsa_class->transmit_mmap = gst_alsa_src_mmap;
@@ -117,6 +117,7 @@ gst_alsa_src_class_init (gpointer g_class, gpointer class_data)
 
   element_class->change_state = gst_alsa_src_change_state;
 }
+
 static void
 gst_alsa_src_init (GstAlsaSrc * src)
 {
@@ -133,8 +134,53 @@ gst_alsa_src_init (GstAlsaSrc * src)
   gst_object_ref (GST_OBJECT (this->clock));
   gst_object_sink (GST_OBJECT (this->clock));
 
+  src->status = NULL;
   gst_element_set_loop_function (GST_ELEMENT (this), gst_alsa_src_loop);
 }
+
+/* alsasrc provides a clock starting from the trigger tstamp
+ * (last play/pause/stop), and added to that the time for the currently
+ * processed samples, and the current fill state of the buffer */
+static GstClockTime
+gst_alsa_src_get_time (GstAlsa * this)
+{
+  struct timeval trigger;
+  snd_pcm_sframes_t delay;
+  GstClockTime gct_trigger, gct_captured, gct_delay, retval =
+      GST_CLOCK_TIME_NONE;
+  int err;
+  GstAlsaSrc *src = GST_ALSA_SRC (this);
+
+  GTimeVal now;
+
+  g_get_current_time (&now);
+  return GST_TIMEVAL_TO_TIME (now);
+
+  if (src->status == NULL)
+    return GST_CLOCK_TIME_NONE;
+
+  if ((err = snd_pcm_status (this->handle, src->status)) < 0) {
+    GST_ERROR_OBJECT (this, "status error: %s", snd_strerror (err));
+    return GST_CLOCK_TIME_NONE;
+  }
+
+  /* trigger tstamp is the last time the device got started/stopped/paused */
+  snd_pcm_status_get_trigger_tstamp (src->status, &trigger);
+  gct_trigger = GST_TIMEVAL_TO_TIME (trigger);
+
+  /* captured is the number of samples already sent out as buffers */
+  gct_captured = gst_alsa_samples_to_timestamp (this, this->captured);
+
+  /* delay is the number of samples in the buffer not yet processed */
+  delay = snd_pcm_status_get_delay (src->status);
+  gct_delay = gst_alsa_samples_to_timestamp (this, delay);
+
+  retval = gct_trigger + gct_captured + gct_delay;
+  GST_LOG_OBJECT (src, "returning clock time of %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (retval));
+  return retval;
+}
+
 static int
 gst_alsa_src_mmap (GstAlsa * this, snd_pcm_sframes_t * avail)
 {
@@ -310,6 +356,30 @@ gst_alsa_src_set_caps (GstAlsaSrc * src, gboolean aggressive)
   return FALSE;
 }
 
+inline snd_pcm_sframes_t
+gst_alsa_src_update_avail (GstAlsa * this)
+{
+  snd_pcm_sframes_t avail = -1;
+
+  while (avail < 0) {
+    avail = snd_pcm_avail_update (this->handle);
+    if (avail < 0) {
+      if (avail == -EPIPE) {
+        gst_alsa_xrun_recovery (this);
+      } else {
+        GST_WARNING_OBJECT (this, "unknown ALSA avail_update return value (%d)",
+            (int) avail);
+      }
+    }
+    if (snd_pcm_state (this->handle) != SND_PCM_STATE_RUNNING) {
+      if (!gst_alsa_start (this)) {
+        return 0;
+      }
+    }
+  }
+  return avail;
+}
+
 /* we transmit buffers of period_size frames */
 static void
 gst_alsa_src_loop (GstElement * element)
@@ -328,16 +398,9 @@ gst_alsa_src_loop (GstElement * element)
     }
   }
 
-  while ((avail = gst_alsa_update_avail (this)) < this->period_size) {
-    if (avail == -EPIPE)
-      continue;
-    if (avail < 0)
-      return;
-    if (snd_pcm_state (this->handle) != SND_PCM_STATE_RUNNING) {
-      if (!gst_alsa_start (this))
-        return;
-      continue;
-    };
+  /* the cast to long is explicitly needed;
+   * with avail = -32 and period_size = 100, avail < period_size is false */
+  while ((avail = gst_alsa_src_update_avail (this)) < this->period_size) {
     /* wait */
     if (gst_alsa_pcm_wait (this) == FALSE)
       return;
@@ -345,27 +408,78 @@ gst_alsa_src_loop (GstElement * element)
   g_assert (avail >= this->period_size);
   /* make sure every pad has a buffer */
   for (i = 0; i < element->numpads; i++) {
-    if (!src->buf[i]) {
-      src->buf[i] = gst_buffer_new_and_alloc (4096);
-    }
+    if (src->buf[i])
+      gst_data_unref (GST_DATA (src->buf[i]));
+    src->buf[i] =
+        gst_buffer_new_and_alloc (gst_alsa_samples_to_bytes (this, avail));
   }
+
   /* fill buffer with data */
   if ((copied = this->transmit (this, &avail)) <= 0)
     return;
-  /* push the buffers out and let them have fun */
-  for (i = 0; i < element->numpads; i++) {
-    if (!src->buf[i])
-      return;
-    if (copied != this->period_size)
-      GST_BUFFER_SIZE (src->buf[i]) = gst_alsa_samples_to_bytes (this, copied);
-    GST_BUFFER_TIMESTAMP (src->buf[i]) =
-        gst_alsa_samples_to_timestamp (this, this->transmitted);
-    GST_BUFFER_DURATION (src->buf[i]) =
-        gst_alsa_samples_to_timestamp (this, copied);
-    gst_pad_push (this->pad[i], GST_DATA (src->buf[i]));
-    src->buf[i] = NULL;
+
+  {
+    gint outsize;
+    GstClockTime outtime, outdur, outreal, outideal, startalsa, outalsa;
+    gint64 diff, offset;
+    struct timeval tstamp;
+    int err;
+
+
+    if ((err = snd_pcm_status (this->handle, src->status)) < 0)
+      GST_ERROR_OBJECT (this, "status error: %s", snd_strerror (err));
+
+    offset = this->captured;
+
+    /* duration of buffer is just the time of the samples */
+    outdur = gst_alsa_samples_to_timestamp (this, copied);
+
+    /* The real capture time is the time of the clock minus the duration and
+     * what is now in the buffer */
+    outreal = gst_element_get_time (GST_ELEMENT (this)) - outdur;
+    /* ideal time is counting samples */
+    outideal = gst_alsa_samples_to_timestamp (this, offset);
+
+    snd_pcm_status_get_trigger_tstamp (src->status, &tstamp);
+    startalsa = GST_TIMEVAL_TO_TIME (tstamp) - element->base_time;
+    outalsa = startalsa + outideal;
+
+    outsize = gst_alsa_samples_to_bytes (this, copied);
+    outtime = GST_CLOCK_TIME_NONE;
+
+    if (GST_ELEMENT_CLOCK (this)) {
+      if (GST_CLOCK (GST_ALSA (this)->clock) == GST_ELEMENT_CLOCK (this)) {
+        outtime = outalsa;
+        diff = outideal - outreal;
+        GST_DEBUG_OBJECT (this, "ideal %lld, real %lld, diff %lld\n", outideal,
+            outreal, diff);
+        offset = gst_alsa_timestamp_to_samples (this, outtime);
+      } else {
+        outtime = outreal;
+        offset = gst_alsa_timestamp_to_samples (this, outtime);
+      }
+    }
+
+    /* push the buffers out and let them have fun */
+    for (i = 0; i < element->numpads; i++) {
+      GstBuffer *buf;
+
+      if (!src->buf[i])
+        return;
+      if (copied != this->period_size)
+        GST_BUFFER_SIZE (src->buf[i]) = outsize;
+
+      GST_BUFFER_TIMESTAMP (src->buf[i]) = outtime;
+      GST_BUFFER_DURATION (src->buf[i]) = outdur;
+      GST_BUFFER_OFFSET (src->buf[i]) = offset;
+      GST_BUFFER_OFFSET_END (src->buf[i]) = offset + copied;
+
+      buf = src->buf[i];
+      src->buf[i] = NULL;
+      gst_pad_push (this->pad[i], GST_DATA (buf));
+    }
+    this->captured += copied;
   }
-  this->transmitted += copied;
 }
 
 static void
@@ -380,6 +494,7 @@ gst_alsa_src_flush (GstAlsaSrc * src)
     }
   }
 }
+
 static GstElementStateReturn
 gst_alsa_src_change_state (GstElement * element)
 {
@@ -390,17 +505,23 @@ gst_alsa_src_change_state (GstElement * element)
 
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_NULL_TO_READY:
+      break;
     case GST_STATE_READY_TO_PAUSED:
+      snd_pcm_status_malloc (&src->status);
+      break;
     case GST_STATE_PAUSED_TO_PLAYING:
+      break;
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
+      snd_pcm_status_free (src->status);
+      src->status = NULL;
       gst_alsa_src_flush (src);
       break;
     case GST_STATE_READY_TO_NULL:
       break;
     default:
-      g_assert_not_reached ();
+      break;
   }
 
   if (GST_ELEMENT_CLASS (src_parent_class)->change_state)
@@ -408,15 +529,3 @@ gst_alsa_src_change_state (GstElement * element)
 
   return GST_STATE_SUCCESS;
 }
-
-static GstClockTime
-gst_alsa_src_get_time (GstAlsa * this)
-{
-  snd_pcm_sframes_t delay;
-
-  if (snd_pcm_delay (this->handle, &delay) == 0) {
-    return GST_SECOND * (this->transmitted + delay) / this->format->rate;
-  } else {
-    return 0;
-  }
-}
