@@ -45,7 +45,7 @@ enum
 {
   ARG_0,
   ARG_MUTE,
-  ARG_NAME,
+  ARG_NAME
 };
 
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -91,6 +91,7 @@ gst_artsdsink_get_type (void)
       0,
       (GInstanceInitFunc) gst_artsdsink_init,
     };
+
     artsdsink_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstArtsdsink",
         &artsdsink_info, 0);
