@@ -32,7 +32,7 @@
 /* elementfactory information */
 static GstElementDetails afparse_details = {
   "Audiofile Parse",
-  "Codec/Parser/Audio",
+  "Codec/Demuxer/Audio",
   "Audiofile parser for audio/raw",
   "Steve Baker <stevebaker_org@yahoo.co.uk>",
 };
@@ -48,7 +48,7 @@ enum
 
 enum
 {
-  ARG_0,
+  ARG_0
 };
 
 /* added a src factory function to force audio/raw MIME type */
@@ -112,6 +112,7 @@ gst_afparse_get_type (void)
       0,
       (GInstanceInitFunc) gst_afparse_init,
     };
+
     afparse_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstAFParse", &afparse_info,
         0);
