@@ -29,7 +29,7 @@
 /* elementfactory information */
 GstElementDetails gst_divxenc_details = {
   "Divx4linux encoder",
-  "Codec/Video/Encoder",
+  "Codec/Encoder/Video",
   "Divx encoder based on divxencore",
   "Ronald Bultje <rbultje@ronald.bitfreak.net>"
 };
@@ -137,6 +137,7 @@ gst_divxenc_get_type (void)
       0,
       (GInstanceInitFunc) gst_divxenc_init,
     };
+
     divxenc_type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstDivxEnc", &divxenc_info, 0);
   }
@@ -176,7 +177,7 @@ gst_divxenc_class_init (GstDivxEncClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAXKEYINTERVAL,
       g_param_spec_int ("max_key_interval", "Max. Key Interval",
           "Maximum number of frames between two keyframes",
-          0, G_MAXINT, 0, G_PARAM_READWRITE));
+          -1, G_MAXINT, -1, G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BUFSIZE,
       g_param_spec_ulong ("buffer_size", "Buffer Size",
@@ -516,7 +517,7 @@ plugin_init (GstPlugin * plugin)
 
   /* create an elementfactory for the v4lmjpegsrcparse element */
   return gst_element_register (plugin, "divxenc",
-      GST_RANK_PRIMARY, GST_TYPE_DIVXENC);
+      GST_RANK_NONE, GST_TYPE_DIVXENC);
 }
 
 
