@@ -63,7 +63,10 @@ struct _GstDVDec {
   gint     channels;
   
   gint 		 length;
+  gint		 framecount;
+  gint		 drop_factor;
   guint64	 next_ts;
+  guint64	 audio_offset;
   guint64	 end_position;
   gboolean	 need_discont;
   gboolean	 new_media;
