@@ -61,7 +61,7 @@ enum
   ARG_EXHAUSTIVE_MODEL_SEARCH,
   ARG_MIN_RESIDUAL_PARTITION_ORDER,
   ARG_MAX_RESIDUAL_PARTITION_ORDER,
-  ARG_RICE_PARAMETER_SEARCH_DIST,
+  ARG_RICE_PARAMETER_SEARCH_DIST
 };
 
 static void gst_flacenc_base_init (gpointer g_class);
@@ -137,7 +137,8 @@ typedef struct
   guint rice_parameter_search_dist;
   guint max_lpc_order;
   guint blocksize;
-} FlacEncParams;
+}
+FlacEncParams;
 
 static const FlacEncParams flacenc_params[] = {
   {FALSE, FALSE, FALSE, FALSE, 0, FALSE, 2, 2, 0, 0, 1152},
@@ -174,6 +175,7 @@ gst_flacenc_quality_get_type (void)
       {9, "9", "9 - Insane"},
       {0, NULL, NULL}
     };
+
     qtype = g_enum_register_static ("FlacEncQuality", values);
   }
   return qtype;
@@ -362,6 +364,7 @@ gst_flacenc_sinkconnect (GstPad * pad, const GstCaps * caps)
   caps = gst_caps_new_simple ("audio/x-flac",
       "channels", G_TYPE_INT, flacenc->channels,
       "rate", G_TYPE_INT, flacenc->sample_rate, NULL);
+
   ret = gst_pad_try_set_caps (flacenc->srcpad, caps);
   if (ret <= 0) {
     return ret;
@@ -396,8 +399,11 @@ gst_flacenc_update_quality (FlacEnc * flacenc, gint quality)
 
   g_object_freeze_notify (G_OBJECT (flacenc));
 
-  DO_UPDATE (do_mid_side_stereo, mid_side, "mid_side_stereo");
-  DO_UPDATE (loose_mid_side_stereo, loose_mid_side, "loose_mid_side");
+  if (flacenc->channels == 2) {
+    DO_UPDATE (do_mid_side_stereo, mid_side, "mid_side_stereo");
+    DO_UPDATE (loose_mid_side_stereo, loose_mid_side, "loose_mid_side");
+  }
+
   DO_UPDATE (blocksize, blocksize, "blocksize");
   DO_UPDATE (max_lpc_order, max_lpc_order, "max_lpc_order");
   DO_UPDATE (qlp_coeff_precision, qlp_coeff_precision, "qlp_coeff_precision");
