@@ -23,6 +23,13 @@
 #include "string.h"
 #include "gstlame.h"
 
+#ifdef lame_set_preset
+#define GST_LAME_PRESET
+#endif
+
+GST_DEBUG_CATEGORY_STATIC (debug);
+#define GST_CAT_DEFAULT debug
+
 /* elementfactory information */
 static GstElementDetails gst_lame_details = {
   "L.A.M.E. mp3 encoder",
@@ -69,7 +76,7 @@ gst_lame_mode_get_type (void)
     {2, "2", "Dual channel"},
     {3, "3", "Mono"},
     {4, "4", "Auto"},
-    {0, NULL, NULL},
+    {0, NULL, NULL}
   };
 
   if (!lame_mode_type) {
@@ -94,7 +101,7 @@ gst_lame_quality_get_type (void)
     {7, "7", "7"},
     {8, "8", "8"},
     {9, "9", "9 - Worst"},
-    {0, NULL, NULL},
+    {0, NULL, NULL}
   };
 
   if (!lame_quality_type) {
@@ -112,7 +119,7 @@ gst_lame_padding_get_type (void)
     {0, "0", "No Padding"},
     {1, "1", "Always Pad"},
     {2, "2", "Adjust Padding"},
-    {0, NULL, NULL},
+    {0, NULL, NULL}
   };
 
   if (!lame_padding_type) {
@@ -121,6 +128,50 @@ gst_lame_padding_get_type (void)
   return lame_padding_type;
 }
 
+#define GST_TYPE_LAME_VBRMODE (gst_lame_vbrmode_get_type())
+static GType
+gst_lame_vbrmode_get_type (void)
+{
+  static GType lame_vbrmode_type = 0;
+  static GEnumValue lame_vbrmode[] = {
+    {vbr_off, "cbr", "No VBR (Constant Bitrate)"},
+    {vbr_rh, "old", "Lame's old VBR algorithm"},
+    {vbr_abr, "abr", "VBR Average Bitrate"},
+    {vbr_mtrh, "new", "Lame's new VBR algorithm"},
+    {0, NULL, NULL}
+  };
+
+  if (!lame_vbrmode_type) {
+    lame_vbrmode_type = g_enum_register_static ("GstLameVbrmode", lame_vbrmode);
+  }
+
+  return lame_vbrmode_type;
+}
+
+#ifdef GSTLAME_PRESET
+#define GST_TYPE_LAME_PRESET (gst_lame_preset_get_type())
+static GType
+gst_lame_preset_get_type (void)
+{
+  static GType gst_lame_preset = 0;
+  static GEnumValue gst_lame_presets[] = {
+    {0, "none", "None"},
+    {MEDIUM, "medium", "Medium"},
+    {STANDARD, "standard", "Standard"},
+    {EXTREME, "extreme", "Extreme"},
+    {INSANE, "insane", "Insane"},
+    {0, NULL, NULL}
+  };
+
+  if (!gst_lame_preset) {
+    gst_lame_preset =
+        g_enum_register_static ("GstLamePreset", gst_lame_presets);
+  }
+
+  return gst_lame_preset;
+}
+#endif
+
 /********** Standard stuff for signals and arguments **********/
 /* GstLame signals and args */
 enum
@@ -161,7 +212,14 @@ enum
   ARG_CWLIMIT,
   ARG_ALLOW_DIFF_SHORT,
   ARG_NO_SHORT_BLOCKS,
-  ARG_EMPHASIS
+  ARG_EMPHASIS,
+  ARG_VBR_QUALITY,
+#ifdef GSTLAME_PRESET
+  ARG_XINGHEADER,
+  ARG_PRESET
+#else
+  ARG_XINGHEADER
+#endif
 };
 
 static void gst_lame_base_init (gpointer g_class);
@@ -239,14 +297,16 @@ gst_lame_class_init (GstLameClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BITRATE,
       g_param_spec_int ("bitrate", "Bitrate (kb/s)", "Bitrate in kbit/sec",
           8, 320, 128, G_PARAM_READWRITE));
+  /* compression ratio set to 0.0 by default otherwise it overrides the bitrate setting */
   g_object_class_install_property (G_OBJECT_CLASS (klass),
       ARG_COMPRESSION_RATIO, g_param_spec_float ("compression_ratio",
           "Compression Ratio",
-          "choose bitrate to achive selected compression ratio", 1.0, 200.0,
-          11.0, G_PARAM_READWRITE));
+          "let lame choose bitrate to achieve selected compression ratio", 0.0,
+          200.0, 0.0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_QUALITY,
-      g_param_spec_enum ("quality", "Quality", "Encoding Quality",
-          GST_TYPE_LAME_QUALITY, 5, G_PARAM_READWRITE));
+      g_param_spec_enum ("quality", "Quality",
+          "Quality of algorithm used for encoding", GST_TYPE_LAME_QUALITY, 5,
+          G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MODE,
       g_param_spec_enum ("mode", "Mode", "Encoding mode", GST_TYPE_LAME_MODE, 0,
           G_PARAM_READWRITE));
@@ -280,8 +340,11 @@ gst_lame_class_init (GstLameClass * klass)
           "Disable reservoir", "Disable the bit reservoir", TRUE,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR,
-      g_param_spec_boolean ("vbr", "VBR", "Use variable bitrate", TRUE,
-          G_PARAM_READWRITE));
+      g_param_spec_enum ("vbr", "VBR", "Specify bitrate mode",
+          GST_TYPE_LAME_VBRMODE, vbr_off, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR_QUALITY,
+      g_param_spec_enum ("vbr_quality", "VBR Quality", "VBR Quality",
+          GST_TYPE_LAME_QUALITY, 5, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR_MEAN_BITRATE,
       g_param_spec_int ("vbr_mean_bitrate", "VBR mean bitrate",
           "Specify mean bitrate", 0, G_MAXINT, 0, G_PARAM_READWRITE));
@@ -336,13 +399,89 @@ gst_lame_class_init (GstLameClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_EMPHASIS,
       g_param_spec_boolean ("emphasis", "Emphasis", "Emphasis", TRUE,
           G_PARAM_READWRITE));
-
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_XINGHEADER,
+      g_param_spec_boolean ("xingheader", "Output Xing Header",
+          "Output Xing Header", FALSE, G_PARAM_READWRITE));
+#ifdef GSTLAME_PRESET
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PRESET,
+      g_param_spec_enum ("preset", "Lame Preset", "Lame Preset",
+          GST_TYPE_LAME_PRESET, 0, G_PARAM_READWRITE));
+#endif
   gobject_class->set_property = gst_lame_set_property;
   gobject_class->get_property = gst_lame_get_property;
 
   gstelement_class->change_state = gst_lame_change_state;
 }
 
+static GstCaps *
+gst_lame_src_getcaps (GstPad * pad)
+{
+  GstLame *lame;
+  GstCaps *caps;
+
+  lame = GST_LAME (gst_pad_get_parent (pad));
+
+  if (!gst_lame_setup (lame)) {
+    GST_DEBUG_OBJECT (lame, "problem doing lame setup");
+    return
+        gst_caps_copy (gst_pad_template_get_caps (gst_static_pad_template_get
+            (&gst_lame_src_template)));
+  }
+
+  caps = gst_caps_new_simple ("audio/mpeg",
+      "mpegversion", G_TYPE_INT, 1,
+      "layer", G_TYPE_INT, 3,
+      "rate", G_TYPE_INT, lame_get_out_samplerate (lame->lgf),
+      "channels", G_TYPE_INT, lame->num_channels, NULL);
+
+  return caps;
+}
+
+static GstPadLinkReturn
+gst_lame_src_link (GstPad * pad, const GstCaps * caps)
+{
+  GstLame *lame;
+  gint out_samplerate;
+  GstStructure *structure;
+  GstCaps *othercaps, *channelcaps;
+  GstPadLinkReturn result;
+
+  lame = GST_LAME (gst_pad_get_parent (pad));
+  structure = gst_caps_get_structure (caps, 0);
+
+  if (!gst_structure_get_int (structure, "rate", &out_samplerate) ||
+      !gst_structure_get_int (structure, "channels", &lame->num_channels))
+    g_return_val_if_reached (GST_PAD_LINK_REFUSED);
+
+  if (lame_set_out_samplerate (lame->lgf, out_samplerate) != 0)
+    return GST_PAD_LINK_REFUSED;
+
+  /* we don't do channel conversion */
+  channelcaps = gst_caps_new_simple ("audio/x-raw-int", "channels", G_TYPE_INT,
+      lame->num_channels, NULL);
+  othercaps = gst_caps_intersect (gst_pad_get_pad_template_caps (lame->sinkpad),
+      channelcaps);
+  gst_caps_free (channelcaps);
+
+  result = gst_pad_try_set_caps_nonfixed (lame->sinkpad, othercaps);
+
+  if (GST_PAD_LINK_FAILED (result))
+    return result;
+
+  caps = gst_pad_get_negotiated_caps (lame->sinkpad);
+  structure = gst_caps_get_structure (caps, 0);
+  if (!gst_structure_get_int (structure, "rate", &lame->samplerate))
+    g_return_val_if_reached (GST_PAD_LINK_REFUSED);
+
+  if (!gst_lame_setup (lame)) {
+    GST_ELEMENT_ERROR (lame, CORE, NEGOTIATION, (NULL),
+        ("could not initialize encoder (wrong parameters?)"));
+    return GST_PAD_LINK_REFUSED;
+  }
+
+  return result;
+}
+
 static GstPadLinkReturn
 gst_lame_sink_link (GstPad * pad, const GstCaps * caps)
 {
@@ -354,9 +493,11 @@ gst_lame_sink_link (GstPad * pad, const GstCaps * caps)
   lame = GST_LAME (gst_pad_get_parent (pad));
   structure = gst_caps_get_structure (caps, 0);
 
-  gst_structure_get_int (structure, "rate", &lame->samplerate);
-  gst_structure_get_int (structure, "channels", &lame->num_channels);
+  if (!gst_structure_get_int (structure, "rate", &lame->samplerate) ||
+      !gst_structure_get_int (structure, "channels", &lame->num_channels))
+    g_return_val_if_reached (GST_PAD_LINK_REFUSED);
 
+  lame_set_out_samplerate (lame->lgf, 0);
   if (!gst_lame_setup (lame)) {
     GST_ELEMENT_ERROR (lame, CORE, NEGOTIATION, (NULL),
         ("could not initialize encoder (wrong parameters?)"));
@@ -390,7 +531,8 @@ gst_lame_init (GstLame * lame)
       gst_pad_new_from_template (gst_static_pad_template_get
       (&gst_lame_src_template), "src");
   gst_element_add_pad (GST_ELEMENT (lame), lame->srcpad);
-
+  gst_pad_set_link_function (lame->srcpad, gst_lame_src_link);
+  gst_pad_set_getcaps_function (lame->srcpad, gst_lame_src_getcaps);
   GST_FLAG_SET (lame, GST_ELEMENT_EVENT_AWARE);
 
   GST_DEBUG ("setting up lame encoder");
@@ -401,7 +543,7 @@ gst_lame_init (GstLame * lame)
   lame->initialized = FALSE;
 
   lame->bitrate = 128;          /* lame_get_brate (lame->lgf); => 0/out of range */
-  lame->compression_ratio = 5;  /* lame_get_compression_ratio (lame->lgf); => 0/out of range */
+  lame->compression_ratio = 0.0;        /* lame_get_compression_ratio (lame->lgf); => 0/out of range ... NOTE: 0.0 makes bitrate take precedence */
   lame->quality = 5;            /* lame_get_quality (lame->lgf); => -1/out of range */
   lame->mode = lame_get_mode (lame->lgf);
   lame->force_ms = lame_get_force_ms (lame->lgf);
@@ -413,12 +555,14 @@ gst_lame_init (GstLame * lame)
   lame->extension = lame_get_extension (lame->lgf);
   lame->strict_iso = lame_get_strict_ISO (lame->lgf);
   lame->disable_reservoir = lame_get_disable_reservoir (lame->lgf);
-  lame->vbr = lame_get_VBR_q (lame->lgf);
+  lame->vbr = vbr_off;          /* lame_get_VBR (lame->lgf); */
+  lame->vbr_quality = 5;
   lame->vbr_mean_bitrate = lame_get_VBR_mean_bitrate_kbps (lame->lgf);
   lame->vbr_min_bitrate = lame_get_VBR_min_bitrate_kbps (lame->lgf);
   lame->vbr_max_bitrate = 320;  /* lame_get_VBR_max_bitrate_kbps (lame->lgf); => 0/no vbr possible */
   lame->vbr_hard_min = lame_get_VBR_hard_min (lame->lgf);
-  lame->lowpass_freq = 50000;   /* lame_get_lowpassfreq (lame->lgf); => 0/lowpass on everything ? */
+  //lame->lowpass_freq = 50000;   /* lame_get_lowpassfreq (lame->lgf); => 0/lowpass on everything ? */
+  lame->lowpass_freq = 0;
   lame->lowpass_width = 0;      /* lame_get_lowpasswidth (lame->lgf); => -1/out of range */
   lame->highpass_freq = lame_get_highpassfreq (lame->lgf);
   lame->highpass_width = 0;     /* lame_get_highpasswidth (lame->lgf); => -1/out of range */
@@ -429,12 +573,15 @@ gst_lame_init (GstLame * lame)
   lame->ath_lower = lame_get_ATHlower (lame->lgf);
   lame->cwlimit = 8.8717;       /* lame_get_cwlimit (lame->lgf); => 0 */
   lame->allow_diff_short = lame_get_allow_diff_short (lame->lgf);
-  lame->no_short_blocks = lame_get_no_short_blocks (lame->lgf);
+  lame->no_short_blocks = TRUE; /* lame_get_no_short_blocks (lame->lgf); */
   lame->emphasis = lame_get_emphasis (lame->lgf);
+  lame->xingheader = FALSE;
+  lame->preset = 0;
   lame->tags = gst_tag_list_new ();
 
   id3tag_init (lame->lgf);
 
+  lame->newmediacount = 0;
   GST_DEBUG_OBJECT (lame, "done initializing");
 }
 
@@ -581,7 +728,10 @@ gst_lame_set_property (GObject * object, guint prop_id, const GValue * value,
       lame->disable_reservoir = g_value_get_boolean (value);
       break;
     case ARG_VBR:
-      lame->vbr = g_value_get_boolean (value);
+      lame->vbr = g_value_get_enum (value);
+      break;
+    case ARG_VBR_QUALITY:
+      lame->vbr_quality = g_value_get_enum (value);
       break;
     case ARG_VBR_MEAN_BITRATE:
       lame->vbr_mean_bitrate = g_value_get_int (value);
@@ -631,6 +781,14 @@ gst_lame_set_property (GObject * object, guint prop_id, const GValue * value,
     case ARG_EMPHASIS:
       lame->emphasis = g_value_get_boolean (value);
       break;
+    case ARG_XINGHEADER:
+      lame->xingheader = g_value_get_boolean (value);
+      break;
+#ifdef GSTLAME_PRESET
+    case ARG_PRESET:
+      lame->preset = g_value_get_enum (value);
+      break;
+#endif
     default:
       break;
   }
@@ -689,7 +847,10 @@ gst_lame_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_boolean (value, lame->disable_reservoir);
       break;
     case ARG_VBR:
-      g_value_set_boolean (value, lame->vbr);
+      g_value_set_enum (value, lame->vbr);
+      break;
+    case ARG_VBR_QUALITY:
+      g_value_set_enum (value, lame->vbr_quality);
       break;
     case ARG_VBR_MEAN_BITRATE:
       g_value_set_int (value, lame->vbr_mean_bitrate);
@@ -739,6 +900,14 @@ gst_lame_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_EMPHASIS:
       g_value_set_boolean (value, lame->emphasis);
       break;
+    case ARG_XINGHEADER:
+      g_value_set_boolean (value, lame->xingheader);
+      break;
+#ifdef GSTLAME_PRESET
+    case ARG_PRESET:
+      g_value_set_enum (value, lame->preset);
+      break;
+#endif
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -757,13 +926,15 @@ gst_lame_chain (GstPad * pad, GstData * _data)
 
   lame = GST_LAME (gst_pad_get_parent (pad));
 
-  GST_DEBUG ("entered chain");
+  GST_LOG_OBJECT (lame, "entered chain");
 
   if (GST_IS_EVENT (buf)) {
     switch (GST_EVENT_TYPE (buf)) {
       case GST_EVENT_EOS:
+        GST_DEBUG_OBJECT (lame, "handling EOS event");
         eos = TRUE;
       case GST_EVENT_FLUSH:
+        GST_DEBUG_OBJECT (lame, "handling FLUSH event");
         mp3_buffer_size = 7200;
         mp3_data = g_malloc (mp3_buffer_size);
 
@@ -771,6 +942,7 @@ gst_lame_chain (GstPad * pad, GstData * _data)
         gst_event_unref (GST_EVENT (buf));
         break;
       case GST_EVENT_TAG:
+        GST_DEBUG_OBJECT (lame, "handling TAG event");
         if (lame->tags) {
           gst_tag_list_insert (lame->tags,
               gst_event_tag_get_list (GST_EVENT (buf)),
@@ -778,7 +950,22 @@ gst_lame_chain (GstPad * pad, GstData * _data)
         } else {
           g_assert_not_reached ();
         }
-        //      gst_pad_event_default (pad, GST_EVENT (buf));
+        gst_pad_event_default (pad, GST_EVENT (buf));
+        break;
+      case GST_EVENT_DISCONTINUOUS:
+        if (GST_EVENT_DISCONT_NEW_MEDIA (GST_EVENT (buf))) {
+          /* do not re-initialise if it is first new media discont */
+          if (lame->newmediacount++ > 0) {
+            lame_close (lame->lgf);
+            lame->lgf = lame_init ();
+            lame->initialized = FALSE;
+            lame->last_ts = GST_CLOCK_TIME_NONE;
+
+            gst_lame_setup (lame);
+          }
+        }
+        gst_pad_event_default (pad, GST_EVENT (buf));
+
         break;
       default:
         gst_pad_event_default (pad, GST_EVENT (buf));
@@ -799,12 +986,20 @@ gst_lame_chain (GstPad * pad, GstData * _data)
         ((GST_BUFFER_SIZE (buf) / (2 + lame->num_channels)) * 1.25) + 7200;
     mp3_data = g_malloc (mp3_buffer_size);
 
-    mp3_size = lame_encode_buffer_interleaved (lame->lgf,
-        (short int *) (GST_BUFFER_DATA (buf)),
-        GST_BUFFER_SIZE (buf) / 2 / lame->num_channels,
-        mp3_data, mp3_buffer_size);
+    /* lame seems to be too stupid to get mono interleaved going */
+    if (lame->num_channels == 1) {
+      mp3_size = lame_encode_buffer (lame->lgf,
+          (short int *) (GST_BUFFER_DATA (buf)),
+          (short int *) (GST_BUFFER_DATA (buf)),
+          GST_BUFFER_SIZE (buf) / 2, mp3_data, mp3_buffer_size);
+    } else {
+      mp3_size = lame_encode_buffer_interleaved (lame->lgf,
+          (short int *) (GST_BUFFER_DATA (buf)),
+          GST_BUFFER_SIZE (buf) / 2 / lame->num_channels,
+          mp3_data, mp3_buffer_size);
+    }
 
-    GST_DEBUG ("encoded %d bytes of audio to %d bytes of mp3",
+    GST_LOG_OBJECT (lame, "encoded %d bytes of audio to %d bytes of mp3",
         GST_BUFFER_SIZE (buf), mp3_size);
 
     duration = (GST_SECOND * GST_BUFFER_SIZE (buf) /
@@ -812,9 +1007,10 @@ gst_lame_chain (GstPad * pad, GstData * _data)
 
     if (GST_BUFFER_DURATION (buf) != GST_CLOCK_TIME_NONE &&
         GST_BUFFER_DURATION (buf) != duration)
-      GST_DEBUG ("mad: incoming buffer had incorrect duration %lld, "
-          "outgoing buffer will have correct duration %lld",
-          GST_BUFFER_DURATION (buf), duration);
+      GST_DEBUG_OBJECT (lame, "incoming buffer had incorrect duration "
+          GST_TIME_FORMAT "outgoing buffer will have correct duration "
+          GST_TIME_FORMAT,
+          GST_TIME_ARGS (GST_BUFFER_DURATION (buf)), GST_TIME_ARGS (duration));
 
     if (lame->last_ts == GST_CLOCK_TIME_NONE) {
       lame->last_ts = GST_BUFFER_TIMESTAMP (buf);
@@ -852,13 +1048,23 @@ gst_lame_chain (GstPad * pad, GstData * _data)
 static gboolean
 gst_lame_setup (GstLame * lame)
 {
+#define CHECK_ERROR(command) G_STMT_START {\
+  if ((command) < 0) { \
+    GST_ERROR_OBJECT (lame, "setup failed: " G_STRINGIFY (command)); \
+    return FALSE; \
+  } \
+}G_STMT_END
+  int retval;
+
   GST_DEBUG_OBJECT (lame, "starting setup");
 
   /* check if we're already initialized; if we are, we might want to check
    * if this initialization is compatible with the previous one */
   /* FIXME: do this */
-  if (lame->initialized)
-    g_warning ("already initialized");
+  if (lame->initialized) {
+    GST_WARNING_OBJECT (lame, "already initialized");
+    lame->initialized = FALSE;
+  }
 
   /* copy the parameters over */
   lame_set_in_samplerate (lame->lgf, lame->samplerate);
@@ -867,52 +1073,63 @@ gst_lame_setup (GstLame * lame)
   if (lame->num_channels == 1)
     lame->mode = 3;
 
-  lame_set_brate (lame->lgf, lame->bitrate);
-  lame_set_compression_ratio (lame->lgf, lame->compression_ratio);
-  lame_set_quality (lame->lgf, lame->quality);
-  lame_set_mode (lame->lgf, lame->mode);
-  lame_set_force_ms (lame->lgf, lame->force_ms);
-  lame_set_free_format (lame->lgf, lame->free_format);
-  lame_set_copyright (lame->lgf, lame->copyright);
-  lame_set_original (lame->lgf, lame->original);
-  lame_set_error_protection (lame->lgf, lame->error_protection);
-  lame_set_padding_type (lame->lgf, lame->padding_type);
-  lame_set_extension (lame->lgf, lame->extension);
-  lame_set_strict_ISO (lame->lgf, lame->strict_iso);
-  lame_set_disable_reservoir (lame->lgf, lame->disable_reservoir);
-  lame_set_VBR_q (lame->lgf, lame->vbr);
-  lame_set_VBR_mean_bitrate_kbps (lame->lgf, lame->vbr_mean_bitrate);
-  lame_set_VBR_min_bitrate_kbps (lame->lgf, lame->vbr_min_bitrate);
-  lame_set_VBR_max_bitrate_kbps (lame->lgf, lame->vbr_max_bitrate);
-  lame_set_VBR_hard_min (lame->lgf, lame->vbr_hard_min);
-  lame_set_lowpassfreq (lame->lgf, lame->lowpass_freq);
-  lame_set_lowpasswidth (lame->lgf, lame->lowpass_width);
-  lame_set_highpassfreq (lame->lgf, lame->highpass_freq);
-  lame_set_highpasswidth (lame->lgf, lame->highpass_width);
-  lame_set_ATHonly (lame->lgf, lame->ath_only);
-  lame_set_ATHshort (lame->lgf, lame->ath_short);
-  lame_set_noATH (lame->lgf, lame->no_ath);
-  lame_set_ATHlower (lame->lgf, lame->ath_lower);
-  lame_set_cwlimit (lame->lgf, lame->cwlimit);
-  lame_set_allow_diff_short (lame->lgf, lame->allow_diff_short);
-  lame_set_no_short_blocks (lame->lgf, lame->no_short_blocks);
-  lame_set_emphasis (lame->lgf, lame->emphasis);
-
+  CHECK_ERROR (lame_set_num_channels (lame->lgf, lame->num_channels));
+  CHECK_ERROR (lame_set_brate (lame->lgf, lame->bitrate));
+  CHECK_ERROR (lame_set_compression_ratio (lame->lgf, lame->compression_ratio));
+  CHECK_ERROR (lame_set_quality (lame->lgf, lame->quality));
+  CHECK_ERROR (lame_set_mode (lame->lgf, lame->mode));
+  CHECK_ERROR (lame_set_force_ms (lame->lgf, lame->force_ms));
+  CHECK_ERROR (lame_set_free_format (lame->lgf, lame->free_format));
+  CHECK_ERROR (lame_set_copyright (lame->lgf, lame->copyright));
+  CHECK_ERROR (lame_set_original (lame->lgf, lame->original));
+  CHECK_ERROR (lame_set_error_protection (lame->lgf, lame->error_protection));
+  CHECK_ERROR (lame_set_padding_type (lame->lgf, lame->padding_type));
+  CHECK_ERROR (lame_set_extension (lame->lgf, lame->extension));
+  CHECK_ERROR (lame_set_strict_ISO (lame->lgf, lame->strict_iso));
+  CHECK_ERROR (lame_set_disable_reservoir (lame->lgf, lame->disable_reservoir));
+  CHECK_ERROR (lame_set_VBR (lame->lgf, lame->vbr));
+  CHECK_ERROR (lame_set_VBR_q (lame->lgf, lame->vbr_quality));
+  CHECK_ERROR (lame_set_VBR_mean_bitrate_kbps (lame->lgf,
+          lame->vbr_mean_bitrate));
+  CHECK_ERROR (lame_set_VBR_min_bitrate_kbps (lame->lgf,
+          lame->vbr_min_bitrate));
+  CHECK_ERROR (lame_set_VBR_max_bitrate_kbps (lame->lgf,
+          lame->vbr_max_bitrate));
+  CHECK_ERROR (lame_set_VBR_hard_min (lame->lgf, lame->vbr_hard_min));
+  CHECK_ERROR (lame_set_lowpassfreq (lame->lgf, lame->lowpass_freq));
+  CHECK_ERROR (lame_set_lowpasswidth (lame->lgf, lame->lowpass_width));
+  CHECK_ERROR (lame_set_highpassfreq (lame->lgf, lame->highpass_freq));
+  CHECK_ERROR (lame_set_highpasswidth (lame->lgf, lame->highpass_width));
+  CHECK_ERROR (lame_set_ATHonly (lame->lgf, lame->ath_only));
+  CHECK_ERROR (lame_set_ATHshort (lame->lgf, lame->ath_short));
+  CHECK_ERROR (lame_set_noATH (lame->lgf, lame->no_ath));
+  CHECK_ERROR (lame_set_ATHlower (lame->lgf, lame->ath_lower));
+  CHECK_ERROR (lame_set_cwlimit (lame->lgf, lame->cwlimit));
+  CHECK_ERROR (lame_set_allow_diff_short (lame->lgf, lame->allow_diff_short));
+  CHECK_ERROR (lame_set_no_short_blocks (lame->lgf, lame->no_short_blocks));
+  CHECK_ERROR (lame_set_emphasis (lame->lgf, lame->emphasis));
+  CHECK_ERROR (lame_set_bWriteVbrTag (lame->lgf, lame->xingheader ? 1 : 0));
+#ifdef GSTLAME_PRESET
+  if (lame->preset > 0) {
+    CHECK_ERROR (lame_set_preset (lame->lgf, lame->preset));
+  }
+#endif
   gst_lame_set_metadata (lame);
 
   /* initialize the lame encoder */
-  if (lame_init_params (lame->lgf) < 0) {
-    lame->initialized = FALSE;
-  } else {
+  if ((retval = lame_init_params (lame->lgf)) >= 0) {
     lame->initialized = TRUE;
     /* FIXME: it would be nice to print out the mode here */
     GST_INFO ("lame encoder initialized (%d kbit/s, %d Hz, %d channels)",
         lame->bitrate, lame->samplerate, lame->num_channels);
+  } else {
+    GST_ERROR_OBJECT (lame, "lame_init_params returned %d", retval);
   }
 
   GST_DEBUG_OBJECT (lame, "done with setup");
 
   return lame->initialized;
+#undef CHECK_ERROR
 }
 
 static GstElementStateReturn
@@ -933,6 +1150,7 @@ gst_lame_change_state (GstElement * element)
     case GST_STATE_READY_TO_NULL:
       if (lame->initialized) {
         lame_close (lame->lgf);
+        lame->lgf = lame_init ();
         lame->initialized = FALSE;
       }
       break;
@@ -953,6 +1171,7 @@ plugin_init (GstPlugin * plugin)
   if (!gst_element_register (plugin, "lame", GST_RANK_NONE, GST_TYPE_LAME))
     return FALSE;
 
+  GST_DEBUG_CATEGORY_INIT (debug, "lame", 0, "lame mp3 encoder");
   return TRUE;
 }
 
