@@ -23,11 +23,16 @@
 #include <string.h>
 #include <gst/gst.h>
 
+#include "gstpngdec.h"
 #include "gstpngenc.h"
 
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  if (!gst_element_register (plugin, "pngdec", GST_RANK_PRIMARY,
+          GST_TYPE_PNGDEC))
+    return FALSE;
+
   if (!gst_element_register (plugin, "pngenc", GST_RANK_NONE, GST_TYPE_PNGENC))
     return FALSE;
 
