@@ -102,6 +102,7 @@ gst_musicbrainz_get_type (void)
       0,
       (GInstanceInitFunc) gst_musicbrainz_init,
     };
+
     musicbrainz_type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstMusicBrainz", &musicbrainz_info, 0);
   }
@@ -114,7 +115,7 @@ gst_musicbrainz_base_init (GstMusicBrainzClass * klass)
 {
   GstElementDetails gst_musicbrainz_details = {
     "Compute TRM Id",
-    "Codec/Audio/Decoder",
+    "Filter/Analyzer/Audio",
     "Compute TRM Id from muscibrainz",
     "Jeremy Simon <jsimon13@yahoo.fr>",
   };
@@ -267,7 +268,6 @@ gst_musicbrainz_chain (GstPad * pad, GstData * data)
     trm_FinalizeSignature (musicbrainz->trm, musicbrainz->signature, NULL);
     trm_ConvertSigToASCII (musicbrainz->trm, musicbrainz->signature,
         musicbrainz->ascii_signature);
-    g_print ("Signature : %s\n", musicbrainz->ascii_signature);
     musicbrainz->signature_available = TRUE;
     g_signal_emit (G_OBJECT (musicbrainz),
         gst_musicbrainz_signals[SIGNAL_SIGNATURE_AVAILABLE], 0);
