@@ -396,7 +396,7 @@ gst_ogg_demux_src_query (GstPad * pad, GstQueryType type,
   gboolean res = FALSE;
   GstOggDemux *ogg;
   GstOggPad *cur;
-  guint64 granulepos;
+  guint64 granulepos = 0;
 
   ogg = GST_OGG_DEMUX (gst_pad_get_parent (pad));
 
