@@ -58,6 +58,10 @@ typedef struct
   guint state;                  /* state of the pad */
 
   GList *headers;
+
+  gboolean new_page;            /* starting a new page */
+  gboolean first_delta;         /* was the first packet in the page a delta */
+  gboolean prev_delta;          /* was the previous buffer a delta frame */
 }
 GstOggPad;
 
@@ -92,6 +96,11 @@ struct _GstOggMux
 
   guint64 max_delay;
   guint64 max_page_delay;
+
+  GstOggPad *delta_pad;         /* when a delta frame is detected on a stream, we mark
+                                   pages as delta frames up to the page that has the
+                                   keyframe */
+
 };
 
 typedef enum
@@ -230,6 +239,7 @@ gst_ogg_mux_get_sink_event_masks (GstPad * pad)
 {
   static const GstEventMask gst_ogg_mux_sink_event_masks[] = {
     {GST_EVENT_EOS, 0},
+    {GST_EVENT_DISCONTINUOUS, 0},
     {0,}
   };
 
@@ -259,6 +269,8 @@ gst_ogg_mux_init (GstOggMux * ogg_mux)
   ogg_mux->max_delay = DEFAULT_MAX_DELAY;
   ogg_mux->max_page_delay = DEFAULT_MAX_PAGE_DELAY;
 
+  ogg_mux->delta_pad = NULL;
+
   gst_element_set_loop_function (GST_ELEMENT (ogg_mux), gst_ogg_mux_loop);
 }
 
@@ -284,18 +296,16 @@ static void
 gst_ogg_mux_pad_link (GstPad * pad, GstPad * peer, gpointer data)
 {
   GstOggMux *ogg_mux = ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
-  const gchar *padname = gst_pad_get_name (pad);
 
-  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' connected", padname);
+  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' connected", gst_pad_get_name (pad));
 }
 
 static void
 gst_ogg_mux_pad_unlink (GstPad * pad, GstPad * peer, gpointer data)
 {
   GstOggMux *ogg_mux = ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
-  const gchar *padname = gst_pad_get_name (pad);
 
-  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' unlinked", padname);
+  GST_DEBUG_OBJECT (ogg_mux, "pad '%s' unlinked", gst_pad_get_name (pad));
 }
 
 static GstPad *
@@ -346,6 +356,9 @@ gst_ogg_mux_request_new_pad (GstElement * element,
       oggpad->eos = FALSE;
       /* we assume there will be some control data first for this pad */
       oggpad->state = GST_OGG_PAD_STATE_CONTROL;
+      oggpad->new_page = TRUE;
+      oggpad->first_delta = FALSE;
+      oggpad->prev_delta = FALSE;
 
       /* save a pointer to our data in the pad */
       gst_pad_set_element_private (newpad, oggpad);
@@ -412,12 +425,17 @@ gst_ogg_mux_next_buffer (GstOggPad * pad)
 
       switch (type) {
         case GST_EVENT_EOS:
+          pad->eos = TRUE;
           gst_event_unref (event);
           return NULL;
         case GST_EVENT_DISCONTINUOUS:
         {
-          gint64 value = 0;
+          guint64 value;
 
+          if (GST_EVENT_DISCONT_NEW_MEDIA (event)) {
+            gst_pad_event_default (pad->pad, event);
+            break;
+          }
           if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &value)) {
             GST_DEBUG_OBJECT (ogg_mux,
                 "got discont of %" G_GUINT64_FORMAT " on pad %s:%s",
@@ -425,8 +443,9 @@ gst_ogg_mux_next_buffer (GstOggPad * pad)
           }
           pad->offset = value;
           gst_event_unref (event);
-          break;
+
         }
+          break;
         default:
           gst_pad_event_default (pad->pad, event);
           break;
@@ -444,6 +463,7 @@ gst_ogg_mux_next_buffer (GstOggPad * pad)
           GST_DEBUG ("muxer: got incaps buffer in control state, ignoring");
           /* just ignore */
           gst_buffer_unref (buf);
+          /* pull next one in next iteration */
           data = NULL;
         } else {
           GST_DEBUG
@@ -458,7 +478,7 @@ gst_ogg_mux_next_buffer (GstOggPad * pad)
 }
 
 static GstBuffer *
-gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page)
+gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page, gboolean delta)
 {
   GstBuffer *buffer;
 
@@ -474,15 +494,17 @@ gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page)
   GST_BUFFER_OFFSET (buffer) = mux->offset;
   mux->offset += GST_BUFFER_SIZE (buffer);
   GST_BUFFER_OFFSET_END (buffer) = mux->offset;
+  if (delta)
+    GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_DELTA_UNIT);
 
   return buffer;
 }
 
 static void
-gst_ogg_mux_push_page (GstOggMux * mux, ogg_page * page)
+gst_ogg_mux_push_page (GstOggMux * mux, ogg_page * page, gboolean delta)
 {
   if (GST_PAD_IS_USABLE (mux->srcpad)) {
-    GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page);
+    GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page, delta);
 
     gst_pad_push (mux->srcpad, GST_DATA (buffer));
   }
@@ -548,6 +570,9 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
 
     walk = walk->next;
 
+    if (pad->eos)
+      continue;
+
     /* try to get a new buffer for this pad if needed and possible */
     if (pad->buffer == NULL && GST_PAD_IS_USABLE (pad->pad)) {
       pad->buffer = gst_ogg_mux_next_buffer (pad);
@@ -692,14 +717,16 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
       buf = GST_BUFFER (pad->headers->data);
       pad->headers = g_list_remove (pad->headers, buf);
     } else {
-      /* create empty buffer for the stream */
-      buf = gst_buffer_new_and_alloc (0);
+      buf = pad->buffer;
+      gst_buffer_ref (buf);
     }
 
     /* create a packet from the buffer */
     packet.packet = GST_BUFFER_DATA (buf);
     packet.bytes = GST_BUFFER_SIZE (buf);
     packet.granulepos = GST_BUFFER_OFFSET_END (buf);
+    if (packet.granulepos == -1)
+      packet.granulepos = 0;
     /* mark BOS and packet number */
     packet.b_o_s = (pad->packetno == 0);
     packet.packetno = pad->packetno++;
@@ -712,7 +739,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
 
     GST_LOG ("flushing page with first packet");
     while (ogg_stream_flush (&pad->stream, &page)) {
-      GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page);
+      GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
 
       GST_LOG ("swapped out page");
       hbufs = g_list_append (hbufs, hbuf);
@@ -741,6 +768,8 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
       packet.packet = GST_BUFFER_DATA (buf);
       packet.bytes = GST_BUFFER_SIZE (buf);
       packet.granulepos = GST_BUFFER_OFFSET_END (buf);
+      if (packet.granulepos == -1)
+        packet.granulepos = 0;
       /* mark BOS and packet number */
       packet.b_o_s = (pad->packetno == 0);
       packet.packetno = pad->packetno++;
@@ -756,7 +785,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
         GST_LOG ("flushing page as packet %d is first or last packet",
             pad->packetno);
         while (ogg_stream_flush (&pad->stream, &page)) {
-          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page);
+          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
 
           GST_LOG ("swapped out page");
           hbufs = g_list_append (hbufs, hbuf);
@@ -765,7 +794,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
         GST_LOG ("try to swap out page");
         /* just try to swap out a page then */
         while (ogg_stream_pageout (&pad->stream, &page) > 0) {
-          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page);
+          GstBuffer *hbuf = gst_ogg_mux_buffer_from_page (mux, &page, FALSE);
 
           GST_LOG ("swapped out page");
           hbufs = g_list_append (hbufs, hbuf);
@@ -804,39 +833,48 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
  * 1) find a pad to pull on, this is done by pulling on all pads and
  *    looking at the buffers to decide which one should be muxed first.
  * 2) store the selected pad and keep on pulling until we fill a 
- *    complete ogg page. This is needed because the ogg spec says that
+ *    complete ogg page or the ogg page is filled above the max-delay 
+ *    threshold. This is needed because the ogg spec says that
  *    you should fill a complete page with data from the same logical
  *    stream. When the page is filled, go back to 1).
  * 3) before filling a packet, read ahead one more buffer to see if this
- *    packet is the last of the stream.
+ *    packet is the last of the stream. We need to do this because the ogg
+ *    spec mandates that the last packet should have the EOS flag set before
+ *    sending it to ogg.
  */
 static void
 gst_ogg_mux_loop (GstElement * element)
 {
   GstOggMux *ogg_mux;
   GstOggPad *best;
+  gboolean delta_unit;
 
   ogg_mux = GST_OGG_MUX (element);
 
   best = gst_ogg_mux_queue_pads (ogg_mux);
 
+
   /* we're pulling a pad and there is a better one, see if we need
    * to flush the current page */
   if (ogg_mux->pulling && best &&
       ogg_mux->pulling != best && ogg_mux->pulling->buffer) {
+    GstOggPad *pad = ogg_mux->pulling;
+
     GstClockTime last_ts =
-        GST_BUFFER_TIMESTAMP (ogg_mux->pulling->buffer) +
-        GST_BUFFER_DURATION (ogg_mux->pulling->buffer);
+        GST_BUFFER_TIMESTAMP (pad->buffer) + GST_BUFFER_DURATION (pad->buffer);
     /* if the next packet in the current page is going to make the page 
      * too long, we need to flush */
     if (last_ts > ogg_mux->next_ts + ogg_mux->max_delay) {
       ogg_page page;
 
-      while (ogg_stream_flush (&ogg_mux->pulling->stream, &page)) {
-        gst_ogg_mux_push_page (ogg_mux, &page);
+      while (ogg_stream_flush (&pad->stream, &page)) {
+        gst_ogg_mux_push_page (ogg_mux, &page, pad->first_delta);
         /* increment the page number counter */
-        ogg_mux->pulling->pageno++;
+        pad->pageno++;
+        /* mark other pages as delta */
+        pad->first_delta = TRUE;
       }
+      pad->new_page = TRUE;
       ogg_mux->pulling = NULL;
     }
   }
@@ -856,6 +894,7 @@ gst_ogg_mux_loop (GstElement * element)
       return;
     }
   }
+
   if (ogg_mux->need_headers) {
     gst_ogg_mux_send_headers (ogg_mux);
     ogg_mux->need_headers = FALSE;
@@ -869,6 +908,7 @@ gst_ogg_mux_loop (GstElement * element)
     GstBuffer *buf, *tmpbuf;
     GstOggPad *pad = ogg_mux->pulling;
     gint64 duration;
+    gboolean force_flush;
 
     /* now see if we have a buffer */
     buf = pad->buffer;
@@ -883,12 +923,15 @@ gst_ogg_mux_loop (GstElement * element)
       }
     }
 
+    delta_unit = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_DELTA_UNIT);
     duration = GST_BUFFER_DURATION (buf);
 
     /* create a packet from the buffer */
     packet.packet = GST_BUFFER_DATA (buf);
     packet.bytes = GST_BUFFER_SIZE (buf);
     packet.granulepos = GST_BUFFER_OFFSET_END (buf);
+    if (packet.granulepos == -1)
+      packet.granulepos = 0;
     /* mark BOS and packet number */
     packet.b_o_s = (pad->packetno == 0);
     packet.packetno = pad->packetno++;
@@ -903,6 +946,59 @@ gst_ogg_mux_loop (GstElement * element)
     /* mark EOS */
     packet.e_o_s = (tmpbuf == NULL ? 1 : 0);
 
+    /* we flush when we see a new keyframe */
+    force_flush = (pad->prev_delta && !delta_unit);
+    if (duration != -1) {
+      pad->duration += duration;
+      /* if page duration exceeds max, flush page */
+      if (pad->duration > ogg_mux->max_page_delay) {
+        force_flush = TRUE;
+        pad->duration = 0;
+      }
+    }
+
+    /* force flush */
+    if (force_flush) {
+      while (ogg_stream_flush (&pad->stream, &page)) {
+        gst_ogg_mux_push_page (ogg_mux, &page, pad->first_delta);
+        /* increment the page number counter */
+        pad->pageno++;
+        /* mark other pages as delta */
+        pad->first_delta = TRUE;
+      }
+      pad->new_page = TRUE;
+    }
+
+    /* if this is the first packet of a new page figure out the delta flag */
+    if (pad->new_page) {
+      if (delta_unit) {
+        /* This page is a delta frame */
+        if (ogg_mux->delta_pad == NULL) {
+          /* we got a delta unit on this pad */
+          ogg_mux->delta_pad = pad;
+        }
+        /* mark the page as delta */
+        pad->first_delta = TRUE;
+      } else {
+        /* got a keyframe */
+        if (ogg_mux->delta_pad == pad) {
+          /* if we get it on the pad with deltaunits,
+           * we mark the page as non delta */
+          pad->first_delta = FALSE;
+        } else if (ogg_mux->delta_pad != NULL) {
+          /* if there are pads with delta frames, we
+           * must mark this one as delta */
+          pad->first_delta = TRUE;
+        } else {
+          pad->first_delta = FALSE;
+        }
+      }
+      pad->new_page = FALSE;
+    }
+
+    /* save key unit to track delta->key unit transitions */
+    pad->prev_delta = delta_unit;
+
     /* swap the packet in */
     ogg_stream_packetin (&pad->stream, &packet);
 
@@ -911,32 +1007,30 @@ gst_ogg_mux_loop (GstElement * element)
     /* store new readahead buffer */
     pad->buffer = tmpbuf;
 
-    if (duration != -1) {
-      pad->duration += duration;
-      /* if page duration exceeds max, flush page */
-      if (pad->duration > ogg_mux->max_page_delay) {
-        while (ogg_stream_flush (&pad->stream, &page)) {
-          gst_ogg_mux_push_page (ogg_mux, &page);
-          /* increment the page number counter */
-          pad->pageno++;
-        }
-        pad->duration = 0;
-        return;
-      }
-    }
-
-    /* flush out the pages now. The packet we got could end up in
-     * more than one page so we need to flush them all */
-    while (ogg_stream_pageout (&pad->stream, &page) > 0) {
-      /* we have a complete page now, we can push the page 
-       * and make sure to pull on a new pad the next time around */
-      gst_ogg_mux_push_page (ogg_mux, &page);
-      /* increment the page number counter */
+    /* let ogg write out the pages now. The packet we got could end 
+     * up in more than one page so we need to write them all */
+    if (ogg_stream_pageout (&pad->stream, &page) > 0) {
+      /* push the page */
+      gst_ogg_mux_push_page (ogg_mux, &page, pad->first_delta);
       pad->pageno++;
+      /* mark next pages as delta */
+      pad->first_delta = TRUE;
+
+      /* use an inner loop her to flush the remaining pages and
+       * mark them as delta frames as well */
+      while (ogg_stream_pageout (&pad->stream, &page) > 0) {
+        /* we have a complete page now, we can push the page 
+         * and make sure to pull on a new pad the next time around */
+        gst_ogg_mux_push_page (ogg_mux, &page, pad->first_delta);
+        /* increment the page number counter */
+        pad->pageno++;
+      }
+      /* need a new page as well */
+      pad->new_page = TRUE;
+      pad->duration = 0;
       /* we're done pulling on this pad, make sure to choose a new 
        * pad for pulling in the next iteration */
       ogg_mux->pulling = NULL;
-      pad->duration = 0;
     }
   }
 }
