@@ -78,6 +78,9 @@ static const GstFormat *vorbis_dec_get_formats (GstPad * pad);
 static gboolean vorbis_dec_src_event (GstPad * pad, GstEvent * event);
 static gboolean vorbis_dec_src_query (GstPad * pad,
     GstQueryType query, GstFormat * format, gint64 * value);
+static gboolean vorbis_dec_convert (GstPad * pad,
+    GstFormat src_format, gint64 src_value,
+    GstFormat * dest_format, gint64 * dest_value);
 
 
 static void
@@ -103,48 +106,43 @@ gst_vorbis_dec_class_init (GstVorbisDecClass * klass)
 static const GstFormat *
 vorbis_dec_get_formats (GstPad * pad)
 {
-  static const GstFormat src_formats[] = {
+  static GstFormat src_formats[] = {
     GST_FORMAT_BYTES,
-    GST_FORMAT_DEFAULT,
+    GST_FORMAT_DEFAULT,         /* samples in the audio case */
     GST_FORMAT_TIME,
     0
   };
-  static const GstFormat sink_formats[] = {
+  static GstFormat sink_formats[] = {
     GST_FORMAT_BYTES,
     GST_FORMAT_TIME,
+    GST_FORMAT_DEFAULT,         /* granulepos or samples */
     0
   };
 
   return (GST_PAD_IS_SRC (pad) ? src_formats : sink_formats);
 }
 
-static GstPadLinkReturn
-vorbis_dec_link (GstPad * pad, const GstCaps * caps)
+static const GstEventMask *
+vorbis_get_event_masks (GstPad * pad)
 {
-  GstVorbisDec *dec = GST_VORBIS_DEC (gst_pad_get_parent (pad));
+  static const GstEventMask vorbis_dec_src_event_masks[] = {
+    {GST_EVENT_SEEK, GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH},
+    {0,}
+  };
 
-  if (dec->packetno < 3)
-    return GST_PAD_LINK_DELAYED;
-
-  return GST_PAD_LINK_OK;
+  return vorbis_dec_src_event_masks;
 }
 
-static GstCaps *
-vorbis_dec_getcaps (GstPad * pad)
+static const GstQueryType *
+vorbis_get_query_types (GstPad * pad)
 {
-  GstVorbisDec *dec = GST_VORBIS_DEC (gst_pad_get_parent (pad));
+  static const GstQueryType vorbis_dec_src_query_types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
+    0
+  };
 
-  if (dec->packetno < 3)
-    return gst_caps_copy (gst_pad_get_pad_template_caps (pad));
-
-  return gst_caps_new_simple ("audio/x-raw-float",
-      "rate", G_TYPE_INT, dec->vi.rate,
-      "channels", G_TYPE_INT, dec->vi.channels,
-      "endianness", G_TYPE_INT, G_BYTE_ORDER, "width", G_TYPE_INT, 32,
-#ifdef GST_VORBIS_DEC_SEQUENTIAL
-      "layout", G_TYPE_STRING, "sequential",
-#endif
-      "buffer-frames", G_TYPE_INT, 0, NULL);
+  return vorbis_dec_src_query_types;
 }
 
 static void
@@ -160,75 +158,95 @@ gst_vorbis_dec_init (GstVorbisDec * dec)
   dec->srcpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&vorbis_dec_src_factory), "src");
-  gst_pad_set_link_function (dec->srcpad, vorbis_dec_link);
-  gst_pad_set_getcaps_function (dec->srcpad, vorbis_dec_getcaps);
+  gst_pad_use_explicit_caps (dec->srcpad);
+  gst_pad_set_event_mask_function (dec->srcpad, vorbis_get_event_masks);
   gst_pad_set_event_function (dec->srcpad, vorbis_dec_src_event);
+  gst_pad_set_query_type_function (dec->srcpad, vorbis_get_query_types);
   gst_pad_set_query_function (dec->srcpad, vorbis_dec_src_query);
   gst_pad_set_formats_function (dec->srcpad, vorbis_dec_get_formats);
+  gst_pad_set_convert_function (dec->srcpad, vorbis_dec_convert);
   gst_element_add_pad (GST_ELEMENT (dec), dec->srcpad);
 
   GST_FLAG_SET (dec, GST_ELEMENT_EVENT_AWARE);
 }
 
-/* FIXME: plug this to the pad convert function */
 static gboolean
-vorbis_dec_to_granulepos (GstVorbisDec * dec, GstFormat format, guint64 from,
-    guint64 * to)
+vorbis_dec_convert (GstPad * pad,
+    GstFormat src_format, gint64 src_value,
+    GstFormat * dest_format, gint64 * dest_value)
 {
+  gboolean res = TRUE;
+  GstVorbisDec *dec;
+  guint64 scale = 1;
+
+  dec = GST_VORBIS_DEC (gst_pad_get_parent (pad));
+
   if (dec->packetno < 1)
     return FALSE;
 
-  switch (format) {
+  switch (src_format) {
     case GST_FORMAT_TIME:
-      *to = from * dec->vi.rate / GST_SECOND;
-      return TRUE;
+      switch (*dest_format) {
+        case GST_FORMAT_BYTES:
+          scale = sizeof (float) * dec->vi.channels;
+        case GST_FORMAT_DEFAULT:
+          *dest_value = scale * (src_value * dec->vi.rate / GST_SECOND);
+          break;
+        default:
+          res = FALSE;
+      }
+      break;
     case GST_FORMAT_DEFAULT:
-      *to = from;
-      return TRUE;
+      switch (*dest_format) {
+        case GST_FORMAT_BYTES:
+          *dest_value = src_value * sizeof (float) * dec->vi.channels;
+          break;
+        case GST_FORMAT_TIME:
+          *dest_value = src_value * GST_SECOND / dec->vi.rate;
+          break;
+        default:
+          res = FALSE;
+      }
+      break;
     case GST_FORMAT_BYTES:
-      *to = from / sizeof (float) / dec->vi.channels;
-      return TRUE;
+      switch (*dest_format) {
+        case GST_FORMAT_DEFAULT:
+          *dest_value = src_value / (sizeof (float) * dec->vi.channels);
+          break;
+        case GST_FORMAT_TIME:
+          *dest_value = src_value * GST_SECOND /
+              (dec->vi.rate * sizeof (float) * dec->vi.channels);
+          break;
+        default:
+          res = FALSE;
+      }
+      break;
     default:
-      return FALSE;
+      res = FALSE;
   }
+
+  return res;
 }
 
-static gboolean
-vorbis_dec_from_granulepos (GstVorbisDec * dec, GstFormat format, guint64 from,
-    guint64 * to)
-{
-  if (dec->packetno < 1)
-    return FALSE;
-
-  switch (format) {
-    case GST_FORMAT_TIME:
-      *to = from * GST_SECOND / dec->vi.rate;
-      return TRUE;
-    case GST_FORMAT_DEFAULT:
-      *to = from;
-      return TRUE;
-    case GST_FORMAT_BYTES:
-      *to = from * sizeof (float) * dec->vi.channels;
-      return TRUE;
-    default:
-      return FALSE;
-  }
-}
-
-
 static gboolean
 vorbis_dec_src_query (GstPad * pad, GstQueryType query, GstFormat * format,
     gint64 * value)
 {
-  gint64 granulepos;
+  gint64 granulepos = 0;
   GstVorbisDec *dec = GST_VORBIS_DEC (gst_pad_get_parent (pad));
   GstFormat my_format = GST_FORMAT_DEFAULT;
 
-  if (!gst_pad_query (GST_PAD_PEER (dec->sinkpad), query, &my_format,
-          &granulepos))
-    return FALSE;
+  if (query == GST_QUERY_POSITION) {
+    granulepos = dec->granulepos;
+  } else {
+    /* query peer in default format */
+    if (!gst_pad_query (GST_PAD_PEER (dec->sinkpad), query, &my_format,
+            &granulepos))
+      return FALSE;
+  }
 
-  if (!vorbis_dec_from_granulepos (dec, *format, granulepos, value))
+  /* and convert to the final format */
+  if (!gst_pad_convert (pad, GST_FORMAT_DEFAULT, granulepos, format, value))
     return FALSE;
 
   GST_LOG_OBJECT (dec,
@@ -246,9 +264,11 @@ vorbis_dec_src_event (GstPad * pad, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:{
       guint64 value;
+      GstFormat my_format = GST_FORMAT_DEFAULT;
 
-      res = vorbis_dec_to_granulepos (dec, GST_EVENT_SEEK_FORMAT (event),
-          GST_EVENT_SEEK_OFFSET (event), &value);
+      /* convert to granulepos */
+      res = vorbis_dec_convert (pad, GST_EVENT_SEEK_FORMAT (event),
+          GST_EVENT_SEEK_OFFSET (event), &my_format, &value);
       if (res) {
         GstEvent *real_seek = gst_event_new_seek (
             (GST_EVENT_SEEK_TYPE (event) & ~GST_SEEK_FORMAT_MASK) |
@@ -267,6 +287,7 @@ vorbis_dec_src_event (GstPad * pad, GstEvent * event)
 
   return res;
 }
+
 static void
 vorbis_dec_event (GstVorbisDec * dec, GstEvent * event)
 {
@@ -275,7 +296,8 @@ vorbis_dec_event (GstVorbisDec * dec, GstEvent * event)
   GST_LOG_OBJECT (dec, "handling event");
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_DISCONTINUOUS:
-      if (gst_event_discont_get_value (event, GST_FORMAT_DEFAULT, &value)) {
+      if (gst_event_discont_get_value (event, GST_FORMAT_DEFAULT,
+              (gint64 *) & value)) {
         dec->granulepos = value;
         GST_DEBUG_OBJECT (dec,
             "setting granuleposition to %" G_GUINT64_FORMAT " after discont",
@@ -293,27 +315,31 @@ vorbis_dec_event (GstVorbisDec * dec, GstEvent * event)
                     GST_FORMAT_TIME, (guint64) 0, GST_FORMAT_DEFAULT,
                     (guint64) 0, GST_FORMAT_BYTES, (guint64) 0, 0)));
       } else {
+        GstFormat time_format, default_format, bytes_format;
+
+        time_format = GST_FORMAT_TIME;
+        default_format = GST_FORMAT_DEFAULT;
+        bytes_format = GST_FORMAT_BYTES;
+
         dec->packetno = 3;
         /* if one of them works, all of them work */
-        if (vorbis_dec_from_granulepos (dec, GST_FORMAT_TIME, dec->granulepos,
-                &time)
-            && vorbis_dec_from_granulepos (dec, GST_FORMAT_DEFAULT,
-                dec->granulepos, &value)
-            && vorbis_dec_from_granulepos (dec, GST_FORMAT_BYTES,
-                dec->granulepos, &bytes)) {
+        if (vorbis_dec_convert (dec->srcpad, GST_FORMAT_DEFAULT,
+                dec->granulepos, &time_format, &time)
+            && vorbis_dec_convert (dec->srcpad, GST_FORMAT_DEFAULT,
+                dec->granulepos, &bytes_format, &bytes)) {
           gst_pad_push (dec->srcpad,
               GST_DATA (gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
-                      time, GST_FORMAT_DEFAULT, value, GST_FORMAT_BYTES, bytes,
-                      0)));
+                      time, GST_FORMAT_DEFAULT, dec->granulepos,
+                      GST_FORMAT_BYTES, bytes, 0)));
         } else {
           GST_ERROR_OBJECT (dec,
               "failed to parse data for DISCONT event, not sending any");
         }
-        gst_data_unref (GST_DATA (event));
 #ifdef HAVE_VORBIS_SYNTHESIS_RESTART
         vorbis_synthesis_restart (&dec->vd);
 #endif
       }
+      gst_data_unref (GST_DATA (event));
       break;
     default:
       gst_pad_event_default (dec->sinkpad, event);
@@ -340,6 +366,13 @@ vorbis_dec_chain (GstPad * pad, GstData * data)
   packet.bytes = GST_BUFFER_SIZE (buf);
   packet.granulepos = GST_BUFFER_OFFSET_END (buf);
   packet.packetno = vd->packetno++;
+
+  /* 
+   * FIXME. Is there anyway to know that this is the last packet and
+   * set e_o_s??
+   */
+  packet.e_o_s = 0;
+
   /* switch depending on packet type */
   if (packet.packet[0] & 1) {
     /* header packet */
@@ -351,6 +384,8 @@ vorbis_dec_chain (GstPad * pad, GstData * data)
       gst_data_unref (data);
       return;
     }
+    /* Packetno = 0 if the first byte is exactly 0x01 */
+    packet.b_o_s = (packet.packet[0] == 0x1) ? 1 : 0;
     if (vorbis_synthesis_headerin (&vd->vi, &vd->vc, &packet)) {
       GST_ELEMENT_ERROR (GST_ELEMENT (vd), STREAM, DECODE,
           (NULL), ("couldn't read header packet"));
@@ -385,16 +420,32 @@ vorbis_dec_chain (GstPad * pad, GstData * data)
             GST_TAG_MINIMUM_BITRATE, (guint) vd->vi.bitrate_lower, NULL);
       gst_element_found_tags_for_pad (GST_ELEMENT (vd), vd->srcpad, 0, list);
     } else if (packet.packetno == 2) {
+      GstCaps *caps;
+
       /* done */
       vorbis_synthesis_init (&vd->vd, &vd->vi);
       vorbis_block_init (&vd->vd, &vd->vb);
-      if (gst_pad_is_linked (vd->srcpad))
-        gst_pad_renegotiate (vd->srcpad);
+      caps = gst_caps_new_simple ("audio/x-raw-float",
+          "rate", G_TYPE_INT, vd->vi.rate,
+          "channels", G_TYPE_INT, vd->vi.channels,
+          "endianness", G_TYPE_INT, G_BYTE_ORDER,
+          "width", G_TYPE_INT, 32, "buffer-frames", G_TYPE_INT, 0, NULL);
+      if (!gst_pad_set_explicit_caps (vd->srcpad, caps)) {
+        gst_caps_free (caps);
+        return;
+      }
+      gst_caps_free (caps);
     }
   } else {
     float **pcm;
     guint sample_count;
 
+    if (packet.packetno < 3) {
+      GST_ELEMENT_ERROR (GST_ELEMENT (vd), STREAM, DECODE,
+          (NULL), ("no header sent yet (packet no is %d)", packet.packetno));
+      gst_data_unref (data);
+      return;
+    }
     /* normal data packet */
     if (vorbis_synthesis (&vd->vb, &packet)) {
       GST_ELEMENT_ERROR (GST_ELEMENT (vd), STREAM, DECODE,
@@ -467,7 +518,6 @@ vorbis_dec_change_state (GstElement * element)
     case GST_STATE_READY_TO_NULL:
       break;
     default:
-      g_assert_not_reached ();
       break;
   }
 
