@@ -22,6 +22,9 @@
 
 #include <gst/audio/audioclock.h>
 
+#ifndef __GST_AUDIO_AUDIO_H__
+#define __GST_AUDIO_AUDIO_H__
+
 G_BEGIN_DECLS
 
 /* For people that are looking at this source: the purpose of these defines is
@@ -50,69 +53,41 @@ G_BEGIN_DECLS
 
 #define GST_AUDIO_DEF_RATE 44100
 
-#define GST_AUDIO_INT_PAD_TEMPLATE_PROPS \
-  gst_props_new (\
-    "rate",       GST_PROPS_INT_RANGE (1, G_MAXINT),\
-    "channels",   GST_PROPS_INT_RANGE (1, G_MAXINT),\
-    "endianness", GST_PROPS_LIST (\
-                    GST_PROPS_INT (G_LITTLE_ENDIAN),\
-                    GST_PROPS_INT (G_BIG_ENDIAN)\
-                  ),\
-    "width",      GST_PROPS_LIST (\
-                    GST_PROPS_INT (8),\
-                    GST_PROPS_INT (16),\
-                    GST_PROPS_INT (32)\
-                  ),\
-    "depth",      GST_PROPS_INT_RANGE (1, 32),\
-    "signed",     GST_PROPS_LIST (\
-                    GST_PROPS_BOOLEAN (TRUE),\
-                    GST_PROPS_BOOLEAN (FALSE)\
-                  ),\
-    NULL)
+#define GST_AUDIO_INT_PAD_TEMPLATE_CAPS \
+  "audio/x-raw-int, " \
+  "rate = (int) [ 1, MAX ], " \
+  "channels = (int) [ 1, MAX ], " \
+  "endianness = (int) { LITTLE_ENDIAN, BIG_ENDIAN }, " \
+  "width = (int) { 8, 16, 24, 32 }, " \
+  "depth = (int) [ 1, 32 ], " \
+  "signed = (boolean) { true, false }" 
 
-#define GST_AUDIO_INT_MONO_PAD_TEMPLATE_PROPS \
-  gst_props_new (\
-    "rate",       GST_PROPS_INT_RANGE (1, G_MAXINT),\
-    "channels",   GST_PROPS_INT (1),\
-    "endianness", GST_PROPS_LIST (\
-                    GST_PROPS_INT (G_LITTLE_ENDIAN),\
-                    GST_PROPS_INT (G_BIG_ENDIAN)\
-                  ),\
-    "width",      GST_PROPS_LIST (\
-                    GST_PROPS_INT (8),\
-                    GST_PROPS_INT (16),\
-                    GST_PROPS_INT (32)\
-                  ),\
-    "depth",      GST_PROPS_INT_RANGE (1, 32),\
-    "signed",     GST_PROPS_LIST (\
-                    GST_PROPS_BOOLEAN (TRUE),\
-                    GST_PROPS_BOOLEAN (FALSE)\
-                  ),\
-    NULL)
 
-#define GST_AUDIO_FLOAT_PAD_TEMPLATE_PROPS \
-  gst_props_new (\
-    "rate",          GST_PROPS_INT_RANGE (1, G_MAXINT),\
-    "channels",      GST_PROPS_INT_RANGE (1, G_MAXINT),\
-    "endianness",    GST_PROPS_LIST (\
-                       GST_PROPS_INT (G_LITTLE_ENDIAN),\
-                       GST_PROPS_INT (G_BIG_ENDIAN)\
-                     ),\
-    "width",         GST_PROPS_LIST (\
-                       GST_PROPS_INT (32),\
-                       GST_PROPS_INT (64)\
-                     ),\
-    "buffer-frames", GST_PROPS_INT_RANGE (1, G_MAXINT),\
-    NULL)
+/* "standard" int audio is native order, 16 bit stereo. */
+#define GST_AUDIO_INT_STANDARD_PAD_TEMPLATE_CAPS \
+  "audio/x-raw-int, " \
+  "rate = (int) [ 1, MAX ], " \
+  "channels = (int) 2, " \
+  "endianness = (int) BYTE_ORDER, " \
+  "width = (int) 16, " \
+  "depth = (int) 16, " \
+  "signed = (boolean) true" 
 
-#define GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_PROPS \
-  gst_props_new (\
-    "rate",          GST_PROPS_INT_RANGE (1, G_MAXINT),\
-    "channels",      GST_PROPS_INT (1),\
-    "endianness",    GST_PROPS_INT (G_BYTE_ORDER),\
-    "width",         GST_PROPS_INT (32),\
-    "buffer-frames", GST_PROPS_INT_RANGE (1, G_MAXINT),\
-    NULL)
+#define GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS \
+  "audio/x-raw-float, " \
+  "rate = (int) [ 1, MAX ], " \
+  "channels = (int) [ 1, MAX ], " \
+  "endianness = (int) { LITTLE_ENDIAN , BIG_ENDIAN }, " \
+  "width = (int) { 32, 64 }, " \
+  "buffer-frames = (int) [ 1, MAX]"
+
+/* "standard" float audio is native order, 32 bit mono. */
+#define GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS \
+  "audio/x-raw-float, " \
+  "rate = (int) [ 1, MAX ], " \
+  "channels = (int) 1, " \
+  "endianness = (int) BYTE_ORDER, " \
+  "buffer-frames = (int) [ 1, MAX]"
 
 /*
  * this library defines and implements some helper functions for audio
@@ -137,5 +112,19 @@ long     gst_audio_highest_sample_value (GstPad* pad);
 /* check if the buffer size is a whole multiple of the frame size */
 gboolean gst_audio_is_buffer_framed     (GstPad* pad, GstBuffer* buf);
 
+/* functions useful for _getcaps functions */
+typedef enum {
+  GST_AUDIO_FIELD_RATE          = (1 << 0),
+  GST_AUDIO_FIELD_CHANNELS      = (1 << 1),
+  GST_AUDIO_FIELD_ENDIANNESS    = (1 << 2),
+  GST_AUDIO_FIELD_WIDTH         = (1 << 3),
+  GST_AUDIO_FIELD_DEPTH         = (1 << 4),
+  GST_AUDIO_FIELD_SIGNED        = (1 << 5),
+  GST_AUDIO_FIELD_BUFFER_FRAMES = (1 << 6)
+} GstAudioFieldFlag;
+
+void gst_audio_structure_set_int (GstStructure *structure, GstAudioFieldFlag flag);
+
 G_END_DECLS
 
+#endif /* __GST_AUDIO_AUDIO_H__ */
