@@ -46,8 +46,19 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
     GstBuffer * strf_data, GstBuffer * strd_data, char **codec_name)
 {
   GstCaps *caps = NULL;
+  GstBuffer *palette = NULL;
 
   switch (codec_fcc) {
+    case GST_MAKE_FOURCC ('D', 'I', 'B', ' '):
+      caps = gst_caps_new_simple ("video/x-raw-rgb",
+          "bpp", G_TYPE_INT, 8,
+          "depth", G_TYPE_INT, 8, "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
+      palette = strf_data;
+      strf_data = NULL;
+      if (codec_name)
+        *codec_name = g_strdup ("Palettized 8-bit RGB");
+      break;
+
     case GST_MAKE_FOURCC ('I', '4', '2', '0'):
       caps = gst_caps_new_simple ("video/x-raw-yuv",
           "format", GST_TYPE_FOURCC, codec_fcc, NULL);
@@ -185,6 +196,7 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
       break;
 
     case GST_MAKE_FOURCC ('M', 'P', 'G', '4'):
+    case GST_MAKE_FOURCC ('M', 'P', '4', 'S'):
       caps = gst_caps_new_simple ("video/x-msmpeg",
           "msmpegversion", G_TYPE_INT, 41, NULL);
       if (codec_name)
@@ -266,25 +278,8 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
     case GST_MAKE_FOURCC (0x1, 0x0, 0x0, 0x0): /* why, why, why? */
       caps = gst_caps_new_simple ("video/x-rle",
           "layout", G_TYPE_STRING, "microsoft", NULL);
-      if (strf_data && GST_BUFFER_SIZE (strf_data) >= 256 * 4) {
-        GstBuffer *copy = gst_buffer_copy (strf_data);
-        GValue value = { 0 };
-
-#if (G_BYTE_ORDER == G_BIG_ENDIAN)
-        gint n;
-        guint32 *data = (guint32 *) GST_BUFFER_DATA (copy);
-
-        /* own endianness */
-        for (n = 0; n < 256; n++)
-          data[n] = GUINT32_FROM_LE (data[n]);
-#endif
-        g_value_init (&value, GST_TYPE_BUFFER);
-        g_value_set_boxed (&value, copy);
-        gst_structure_set_value (gst_caps_get_structure (caps, 0),
-            "palette_data", &value);
-        g_value_unset (&value);
-        gst_buffer_unref (copy);
-      }
+      palette = strf_data;
+      strf_data = NULL;
       if (strf) {
         gst_caps_set_simple (caps,
             "depth", G_TYPE_INT, (gint) strf->bit_cnt, NULL);
@@ -292,7 +287,14 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
         gst_caps_set_simple (caps, "depth", GST_TYPE_INT_RANGE, 1, 64, NULL);
       }
       if (codec_name)
-        *codec_name = g_strdup ("Mcrosoft RLE");
+        *codec_name = g_strdup ("Microsoft RLE");
+      break;
+
+    case GST_MAKE_FOURCC ('X', 'x', 'a', 'n'):
+      caps = gst_caps_new_simple ("video/x-xan",
+          "wcversion", G_TYPE_INT, 4, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Xan Wing Commander 4");
       break;
 
     default:
@@ -320,6 +322,28 @@ gst_riff_create_video_caps_with_data (guint32 codec_fcc,
         "height", GST_TYPE_INT_RANGE, 16, 4096, NULL);
   }
 
+  /* extradata */
+  if (strf_data || strd_data) {
+    gst_caps_set_simple (caps, "codec_data", GST_TYPE_BUFFER,
+        strf_data ? strf_data : strd_data, NULL);
+  }
+
+  /* palette */
+  if (palette && GST_BUFFER_SIZE (palette) >= 256 * 4) {
+    GstBuffer *copy = gst_buffer_copy (palette);
+
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+    gint n;
+    guint32 *data = (guint32 *) GST_BUFFER_DATA (copy);
+
+    /* own endianness */
+    for (n = 0; n < 256; n++)
+      data[n] = GUINT32_FROM_LE (data[n]);
+#endif
+    gst_caps_set_simple (caps, "palette_data", GST_TYPE_BUFFER, copy, NULL);
+    gst_buffer_unref (copy);
+  }
+
   return caps;
 }
 
@@ -431,8 +455,9 @@ gst_riff_create_audio_caps_with_data (guint16 codec_id,
       break;
     case GST_RIFF_WAVE_FORMAT_WMAV1:
     case GST_RIFF_WAVE_FORMAT_WMAV2:
+    case GST_RIFF_WAVE_FORMAT_WMAV3:
     {
-      gint version = codec_id == GST_RIFF_WAVE_FORMAT_WMAV1 ? 1 : 2;
+      gint version = (codec_id - GST_RIFF_WAVE_FORMAT_WMAV1) + 1;
 
       block_align = TRUE;
 
@@ -449,12 +474,14 @@ gst_riff_create_audio_caps_with_data (guint16 codec_id,
         gst_caps_set_simple (caps,
             "bitrate", GST_TYPE_INT_RANGE, 0, G_MAXINT, NULL);
       }
-      if (strf_data) {
-        gst_caps_set_simple (caps,
-            "codec_data", GST_TYPE_BUFFER, strf_data, NULL);
-      }
       break;
     }
+    case GST_RIFF_WAVE_FORMAT_SONY_ATRAC3:
+      caps = gst_caps_new_simple ("audio/x-vnd.sony.atrac3", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Sony ATRAC3");
+      break;
+
     default:
       GST_WARNING ("Unknown audio tag 0x%04x", codec_id);
       return NULL;
@@ -482,6 +509,12 @@ gst_riff_create_audio_caps_with_data (guint16 codec_id,
     }
   }
 
+  /* extradata */
+  if (strf_data || strd_data) {
+    gst_caps_set_simple (caps, "codec_data", GST_TYPE_BUFFER,
+        strf_data ? strf_data : strd_data, NULL);
+  }
+
   return caps;
 }
 
@@ -546,6 +579,8 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('c', 'v', 'i', 'd'),
     GST_MAKE_FOURCC ('m', 's', 'v', 'c'),
     GST_MAKE_FOURCC ('R', 'L', 'E', ' '),
+    GST_MAKE_FOURCC ('D', 'I', 'B', ' '),
+    GST_MAKE_FOURCC ('X', 'x', 'a', 'n'),
     /* FILL ME */
     0
   };
