@@ -31,20 +31,6 @@
 GST_DEBUG_CATEGORY_STATIC (avidemux_debug);
 #define GST_CAT_DEFAULT avidemux_debug
 
-/* AviDemux signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
-enum
-{
-  ARG_0,
-  ARG_STREAMINFO
-      /* FILL ME */
-};
-
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -73,13 +59,8 @@ static gboolean gst_avi_demux_src_convert (GstPad * pad,
 
 static GstElementStateReturn gst_avi_demux_change_state (GstElement * element);
 
-static void gst_avi_demux_get_property (GObject * object,
-    guint prop_id, GValue * value, GParamSpec * pspec);
-
 static GstRiffReadClass *parent_class = NULL;
 
-/*static guint gst_avi_demux_signals[LAST_SIGNAL] = { 0 }; */
-
 GType
 gst_avi_demux_get_type (void)
 {
@@ -145,17 +126,11 @@ gst_avi_demux_class_init (GstAviDemuxClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  g_object_class_install_property (gobject_class, ARG_STREAMINFO,
-      g_param_spec_boxed ("streaminfo", "Streaminfo", "Streaminfo",
-          GST_TYPE_CAPS, G_PARAM_READABLE));
-
   GST_DEBUG_CATEGORY_INIT (avidemux_debug, "avidemux",
       0, "Demuxer for AVI streams");
 
   parent_class = g_type_class_ref (GST_TYPE_RIFF_READ);
 
-  gobject_class->get_property = gst_avi_demux_get_property;
-
   gstelement_class->change_state = gst_avi_demux_change_state;
   gstelement_class->send_event = gst_avi_demux_send_event;
 }
@@ -174,7 +149,6 @@ gst_avi_demux_init (GstAviDemux * avi)
   gst_element_set_loop_function (GST_ELEMENT (avi), gst_avi_demux_loop);
   gst_avi_demux_reset (avi);
 
-  avi->streaminfo = NULL;
   avi->index_entries = NULL;
   memset (&avi->stream, 0, sizeof (avi->stream));
 }
@@ -203,24 +177,12 @@ gst_avi_demux_reset (GstAviDemux * avi)
     avi->index_entries = NULL;
   }
   avi->index_size = 0;
+  avi->current_entry = 0;
 
   avi->num_frames = 0;
   avi->us_per_frame = 0;
 
   avi->seek_offset = (guint64) - 1;
-
-  gst_caps_replace (&avi->streaminfo, NULL);
-}
-
-static void
-gst_avi_demux_streaminfo (GstAviDemux * avi)
-{
-  /* compression formats are added later - a bit hacky */
-
-  gst_caps_replace (&avi->streaminfo,
-      gst_caps_new_simple ("application/x-gst-streaminfo", NULL));
-
-  /*g_object_notify(G_OBJECT(avi), "streaminfo"); */
 }
 
 static gst_avi_index_entry *
@@ -341,7 +303,7 @@ gst_avi_demux_src_convert (GstPad * pad,
   /*GstAviDemux *avi = GST_AVI_DEMUX (gst_pad_get_parent (pad)); */
   avi_stream_context *stream = gst_pad_get_element_private (pad);
 
-  if (stream->strh->type != GST_RIFF_FCC_auds &&
+  if (stream->strh->type == GST_RIFF_FCC_vids &&
       (src_format == GST_FORMAT_BYTES || *dest_format == GST_FORMAT_BYTES))
     return FALSE;
 
@@ -349,8 +311,7 @@ gst_avi_demux_src_convert (GstPad * pad,
     case GST_FORMAT_TIME:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
-          *dest_value = src_value * stream->strh->rate /
-              (stream->strh->scale * GST_SECOND);
+          *dest_value = src_value * stream->bitrate / GST_SECOND;
           break;
         case GST_FORMAT_DEFAULT:
           *dest_value = src_value * stream->strh->rate /
@@ -364,7 +325,7 @@ gst_avi_demux_src_convert (GstPad * pad,
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
         case GST_FORMAT_TIME:
-          *dest_value = ((gfloat) src_value) * GST_SECOND / stream->strh->rate;
+          *dest_value = ((gfloat) src_value) * GST_SECOND / stream->bitrate;
           break;
         default:
           res = FALSE;
@@ -406,6 +367,7 @@ gst_avi_demux_handle_src_query (GstPad * pad,
     GstQueryType type, GstFormat * format, gint64 * value)
 {
   gboolean res = TRUE;
+  GstAviDemux *demux = GST_AVI_DEMUX (gst_pad_get_parent (pad));
 
   /*GstAviDemux *avi = GST_AVI_DEMUX (gst_pad_get_parent (pad)); */
   avi_stream_context *stream = gst_pad_get_element_private (pad);
@@ -424,9 +386,12 @@ gst_avi_demux_handle_src_query (GstPad * pad,
             res = FALSE;
           break;
         case GST_FORMAT_DEFAULT:
-          if (stream->strh->type == GST_RIFF_FCC_auds)
-            *value = stream->strh->length * stream->strh->samplesize;
-          else if (stream->strh->type == GST_RIFF_FCC_vids)
+          if (stream->strh->type == GST_RIFF_FCC_auds) {
+            if (!stream->strh->samplesize)
+              *value = stream->total_frames;
+            else
+              *value = stream->total_bytes / stream->strh->samplesize;
+          } else if (stream->strh->type == GST_RIFF_FCC_vids)
             *value = stream->strh->length;
           else
             res = FALSE;
@@ -439,12 +404,33 @@ gst_avi_demux_handle_src_query (GstPad * pad,
     case GST_QUERY_POSITION:
       switch (*format) {
         case GST_FORMAT_TIME:
-          if (stream->strh->rate && stream->strh->type == GST_RIFF_FCC_auds) {
-            *value = ((gfloat) stream->current_byte) * GST_SECOND /
-                stream->strh->rate;
+          if (stream->strh->type == GST_RIFF_FCC_auds) {
+            if (!stream->strh->samplesize) {
+              *value = GST_SECOND * stream->current_frame *
+                  stream->strh->scale / stream->strh->rate;
+            } else if (stream->bitrate != 0) {
+              *value = ((gfloat) stream->current_byte) * GST_SECOND /
+                  stream->bitrate;
+            } else if (stream->total_frames != 0 && stream->total_bytes != 0) {
+              /* calculate timestamps based on video size */
+              guint64 len = demux->us_per_frame * demux->num_frames *
+                  GST_USECOND;
+
+              if (!stream->strh->samplesize)
+                *value = len * stream->current_frame / stream->total_frames;
+              else
+                *value = len * stream->current_byte / stream->total_bytes;
+            } else {
+              res = FALSE;
+            }
           } else {
-            *value = (((gfloat) stream->current_frame) * stream->strh->scale /
-                stream->strh->rate) * GST_SECOND;
+            if (stream->strh->rate != 0) {
+              *value = ((gfloat) stream->current_frame * stream->strh->scale *
+                  GST_SECOND / stream->strh->rate);
+            } else {
+              *value = stream->current_frame * demux->us_per_frame *
+                  GST_USECOND;
+            }
           }
           break;
         case GST_FORMAT_BYTES:
@@ -453,7 +439,7 @@ gst_avi_demux_handle_src_query (GstPad * pad,
         case GST_FORMAT_DEFAULT:
           if (stream->strh->samplesize &&
               stream->strh->type == GST_RIFF_FCC_auds)
-            *value = stream->current_byte * stream->strh->samplesize;
+            *value = stream->current_byte / stream->strh->samplesize;
           else
             *value = stream->current_frame;
           break;
@@ -478,52 +464,6 @@ gst_avi_demux_src_getcaps (GstPad * pad)
   return gst_caps_copy (stream->caps);
 }
 
-static gint32
-gst_avi_demux_sync_streams (GstAviDemux * avi, guint64 time)
-{
-  gint i;
-  guint32 min_index = G_MAXUINT;
-  avi_stream_context *stream;
-  gst_avi_index_entry *entry;
-
-  for (i = 0; i < avi->num_streams; i++) {
-    stream = &avi->stream[i];
-
-    GST_DEBUG ("finding %d for time %" G_GINT64_FORMAT, i, time);
-
-    entry = gst_avi_demux_index_entry_for_time (avi, stream->num, time,
-        GST_RIFF_IF_KEYFRAME);
-    if (entry) {
-      min_index = MIN (entry->index_nr, min_index);
-    }
-  }
-  GST_DEBUG ("first index at %d", min_index);
-
-  /* now we know the entry we need to sync on. calculate number of frames to
-   * skip fro there on and the stream stats */
-  for (i = 0; i < avi->num_streams; i++) {
-    gst_avi_index_entry *next_entry;
-
-    stream = &avi->stream[i];
-
-    /* next entry */
-    next_entry = gst_avi_demux_index_next (avi, stream->num, min_index, 0);
-    /* next entry with keyframe */
-    entry = gst_avi_demux_index_next (avi, stream->num, min_index,
-        GST_RIFF_IF_KEYFRAME);
-
-    stream->current_byte = next_entry->bytes_before;
-    stream->current_frame = next_entry->frames_before;
-    stream->skip = entry->frames_before - next_entry->frames_before;
-
-    GST_DEBUG ("%d skip %d", stream->num, stream->skip);
-  }
-
-  GST_DEBUG ("final index at %d", min_index);
-
-  return min_index;
-}
-
 static gboolean
 gst_avi_demux_send_event (GstElement * element, GstEvent * event)
 {
@@ -571,6 +511,9 @@ gst_avi_demux_handle_src_event (GstPad * pad, GstEvent * event)
   GstAviDemux *avi = GST_AVI_DEMUX (gst_pad_get_parent (pad));
   avi_stream_context *stream;
 
+  if (!avi->index_entries)
+    return FALSE;
+
   stream = gst_pad_get_element_private (pad);
 
   switch (GST_EVENT_TYPE (event)) {
@@ -582,10 +525,9 @@ gst_avi_demux_handle_src_event (GstPad * pad, GstEvent * event)
         case GST_FORMAT_BYTES:
         case GST_FORMAT_DEFAULT:
         case GST_FORMAT_TIME:{
-          gst_avi_index_entry *seek_entry, *entry = NULL;
+          gst_avi_index_entry *entry = NULL;
           gint64 desired_offset = GST_EVENT_SEEK_OFFSET (event);
           guint32 flags;
-          guint64 min_index;
 
           /* no seek on audio yet */
           if (stream->strh->type == GST_RIFF_FCC_auds) {
@@ -611,16 +553,17 @@ gst_avi_demux_handle_src_event (GstPad * pad, GstEvent * event)
           }
 
           if (entry) {
-            min_index = gst_avi_demux_sync_streams (avi, entry->ts);
-            seek_entry = &avi->index_entries[min_index];
-
-            avi->seek_offset = seek_entry->offset + avi->index_offset;
+            avi->seek_offset = entry->offset + avi->index_offset;
             avi->last_seek = entry->ts;
+            avi->seek_flush =
+                (GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH);
+            avi->seek_entry = entry->index_nr;
           } else {
             GST_DEBUG ("no index entry found for format=%d value=%"
                 G_GINT64_FORMAT, GST_EVENT_SEEK_FORMAT (event), desired_offset);
             res = FALSE;
           }
+          GST_LOG ("seek done\n");
           break;
         }
         default:
@@ -643,7 +586,7 @@ done:
  * "Open" a RIFF file.
  */
 
-gboolean
+static gboolean
 gst_avi_demux_stream_init (GstAviDemux * avi)
 {
   GstRiffRead *riff = GST_RIFF_READ (avi);
@@ -663,7 +606,7 @@ gst_avi_demux_stream_init (GstAviDemux * avi)
  * Read 'avih' header.
  */
 
-gboolean
+static gboolean
 gst_avi_demux_stream_avih (GstAviDemux * avi,
     guint32 * flags, guint32 * streams)
 {
@@ -747,6 +690,7 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
   GstPadTemplate *templ = NULL;
   GstPad *pad;
   avi_stream_context *stream;
+  gint blockalign = 0, bitrate = 0;
   union
   {
     gst_riff_strf_vids *vids;
@@ -779,7 +723,7 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
         return FALSE;
       break;
     case GST_RIFF_FCC_auds:
-      if (!gst_riff_read_strf_auds (riff, &strf.auds))
+      if (!gst_riff_read_strf_auds_with_data (riff, &strf.auds, &extradata))
         return FALSE;
       break;
     case GST_RIFF_FCC_iavs:
@@ -839,10 +783,15 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
     {
       char *codec_name = NULL;
       GstTagList *list = gst_tag_list_new ();
+      guint32 tag;
 
       padname = g_strdup_printf ("video_%02d", avi->num_v_streams);
       templ = gst_element_class_get_pad_template (klass, "video_%02d");
-      caps = gst_riff_create_video_caps_with_data (strf.vids->compression,
+      if (strf.vids->compression)
+        tag = strf.vids->compression;
+      else
+        tag = strh->fcc_handler;
+      caps = gst_riff_create_video_caps_with_data (tag,
           strh, strf.vids, extradata, initdata, &codec_name);
       gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_VIDEO_CODEC,
           codec_name, NULL);
@@ -861,14 +810,17 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
 
       padname = g_strdup_printf ("audio_%02d", avi->num_a_streams);
       templ = gst_element_class_get_pad_template (klass, "audio_%02d");
-      caps = gst_riff_create_audio_caps (strf.auds->format, strh, strf.auds,
-          &codec_name);
+      caps =
+          gst_riff_create_audio_caps_with_data (strf.auds->format, strh,
+          strf.auds, extradata, initdata, &codec_name);
       gst_tag_list_add (list, GST_TAG_MERGE_APPEND, GST_TAG_AUDIO_CODEC,
           codec_name, NULL);
       gst_element_found_tags (GST_ELEMENT (avi), list);
       gst_tag_list_free (list);
       if (codec_name)
         g_free (codec_name);
+      blockalign = strf.auds->blockalign;
+      bitrate = strf.auds->av_bps;
       g_free (strf.auds);
       avi->num_a_streams++;
       break;
@@ -920,6 +872,8 @@ gst_avi_demux_add_stream (GstAviDemux * avi)
   stream->current_byte = 0;
   stream->current_entry = -1;
   stream->skip = 0;
+  stream->blockalign = blockalign;
+  stream->bitrate = bitrate;
   gst_pad_set_element_private (pad, stream);
   avi->num_streams++;
 
@@ -1017,24 +971,34 @@ gst_avi_demux_stream_odml (GstAviDemux * avi)
 
 /*
  * Seek to index, read it, seek back.
+ * Return value indicates if we can continue processing. It
+ * does not indicate if index-reading succeeded.
  */
 
-gboolean
-gst_avi_demux_stream_index (GstAviDemux * avi)
+static gboolean
+gst_avi_demux_stream_index (GstAviDemux * avi,
+    GList ** index, GList ** alloc_list)
 {
+  GList *list = NULL;
   GstBuffer *buf = NULL;
   guint i;
   GstEvent *event;
   GstRiffRead *riff = GST_RIFF_READ (avi);
   guint64 pos_before, pos_after, length;
   guint32 tag;
+  guint index_size;
+  gst_avi_index_entry *index_entries = NULL;
 
   /* first, we need to know the current position (to seek back
    * when we're done) and the total length of the file. */
   length = gst_bytestream_length (riff->bs);
   pos_before = gst_bytestream_tell (riff->bs);
 
-  /* skip movi */
+  /* skip movi
+   *
+   * FIXME:
+   * - we want to add error handling here so we can recover.
+   */
   if (!gst_riff_read_skip (riff))
     return FALSE;
 
@@ -1059,12 +1023,11 @@ gst_avi_demux_stream_index (GstAviDemux * avi)
     return FALSE;
 
   /* parse all entries */
-  avi->index_size = GST_BUFFER_SIZE (buf) / sizeof (gst_riff_index_entry);
-  avi->index_entries =
-      g_malloc (avi->index_size * sizeof (gst_avi_index_entry));
+  index_size = GST_BUFFER_SIZE (buf) / sizeof (gst_riff_index_entry);
+  index_entries = g_malloc (index_size * sizeof (gst_avi_index_entry));
   GST_INFO ("%u index entries", avi->index_size);
 
-  for (i = 0; i < avi->index_size; i++) {
+  for (i = 0; i < index_size; i++) {
     gst_riff_index_entry entry, *_entry;
     avi_stream_context *stream;
     gint stream_nr;
@@ -1076,14 +1039,14 @@ gst_avi_demux_stream_index (GstAviDemux * avi)
     entry.offset = GUINT32_FROM_LE (_entry->offset);
     entry.flags = GUINT32_FROM_LE (_entry->flags);
     entry.size = GUINT32_FROM_LE (_entry->size);
-    target = &avi->index_entries[i];
+    target = &index_entries[i];
 
     if (entry.id == GST_RIFF_rec)
       continue;
 
     stream_nr = CHUNKID_TO_STREAMNR (entry.id);
     if (stream_nr >= avi->num_streams || stream_nr < 0) {
-      g_warning ("Index entry %d has invalid stream nr %d", i, stream_nr);
+      GST_WARNING ("Index entry %d has invalid stream nr %d", i, stream_nr);
       target->stream_nr = -1;
       continue;
     }
@@ -1093,7 +1056,7 @@ gst_avi_demux_stream_index (GstAviDemux * avi)
     target->index_nr = i;
     target->flags = entry.flags;
     target->size = entry.size;
-    target->offset = entry.offset;
+    target->offset = entry.offset + 8;
 
     /* figure out if the index is 0 based or relative to the MOVI start */
     if (i == 0) {
@@ -1116,14 +1079,21 @@ gst_avi_demux_stream_index (GstAviDemux * avi)
       /* constant rate stream */
       gst_pad_convert (stream->pad, GST_FORMAT_BYTES,
           stream->total_bytes, &format, &target->ts);
+      gst_pad_convert (stream->pad, GST_FORMAT_BYTES,
+          stream->total_bytes + target->size, &format, &target->dur);
     } else {
       /* VBR stream */
       gst_pad_convert (stream->pad, GST_FORMAT_DEFAULT,
           stream->total_frames, &format, &target->ts);
+      gst_pad_convert (stream->pad, GST_FORMAT_DEFAULT,
+          stream->total_frames + 1, &format, &target->dur);
     }
+    target->dur -= target->ts;
 
     stream->total_bytes += target->size;
     stream->total_frames++;
+
+    list = g_list_prepend (list, target);
   }
 
   /* debug our indexes */
@@ -1140,27 +1110,364 @@ end:
     gst_buffer_unref (buf);
 
   /* seek back to the data */
-  if (!(event = gst_riff_read_seek (riff, pos_before)))
+  if (!(event = gst_riff_read_seek (riff, pos_before))) {
+    g_free (index_entries);
+    g_list_free (list);
     return FALSE;
+  }
   gst_event_unref (event);
 
+  if (list)
+    *index = g_list_reverse (list);
+  if (index_entries)
+    *alloc_list = g_list_prepend (*alloc_list, index_entries);
+
+  return TRUE;
+}
+
+/*
+ * Sync to next data chunk.
+ */
+
+static gboolean
+gst_avi_demux_sync (GstAviDemux * avi, guint32 * ret_tag, gboolean prevent_eos)
+{
+  GstRiffRead *riff = GST_RIFF_READ (avi);
+  guint32 tag;
+  guint64 length = gst_bytestream_length (riff->bs);
+
+  if (gst_bytestream_tell (riff->bs) + 12 >= length)
+    return FALSE;
+
+  /* peek first (for the end of this 'list/movi' section) */
+  if (!(tag = gst_riff_peek_tag (riff, &avi->level_up)))
+    return FALSE;
+
+  /* if we're at top-level, we didn't read the 'movi'
+   * list tag yet. This can also be 'AVIX' in case of
+   * openDML-2.0 AVI files. Lastly, it might be idx1,
+   * in which case we skip it so we come at EOS. */
+  while (g_list_length (riff->level) < 2) {
+    if (gst_bytestream_tell (riff->bs) + 12 >= length)
+      return FALSE;
+
+    if (!(tag = gst_riff_peek_tag (riff, NULL)))
+      return FALSE;
+
+    switch (tag) {
+      case GST_RIFF_TAG_LIST:
+        if (!(tag = gst_riff_peek_list (riff)))
+          return FALSE;
+
+        switch (tag) {
+          case GST_RIFF_LIST_AVIX:
+          case GST_RIFF_LIST_movi:
+            if (!gst_riff_read_list (riff, &tag))
+              return FALSE;
+            /* we're now going to read buffers! */
+            break;
+
+          default:
+            GST_WARNING ("Unknown list " GST_FOURCC_FORMAT " before AVI data",
+                GST_FOURCC_ARGS (tag));
+            /* fall-through */
+
+          case GST_RIFF_TAG_JUNK:
+            if (!gst_riff_read_skip (riff))
+              return FALSE;
+            break;
+        }
+
+        break;
+
+      default:
+        GST_WARNING ("Unknown tag " GST_FOURCC_FORMAT " before AVI data",
+            GST_FOURCC_ARGS (tag));
+        /* fall-through */
+
+      case GST_RIFF_TAG_idx1:
+      case GST_RIFF_TAG_JUNK:
+        if (!gst_riff_read_skip (riff))
+          return FALSE;
+        break;
+    }
+  }
+
+  /* And then, we get the data */
+  if (gst_bytestream_tell (riff->bs) + 12 >= length)
+    return FALSE;
+
+  if (!(tag = gst_riff_peek_tag (riff, NULL)))
+    return FALSE;
+
+  /* Support for rec-list files */
+  switch (tag) {
+    case GST_RIFF_TAG_LIST:
+      if (!(tag = gst_riff_peek_list (riff)))
+        return FALSE;
+      if (tag == GST_RIFF_rec) {
+        /* Simply skip the list */
+        if (!gst_riff_read_list (riff, &tag))
+          return FALSE;
+        if (!(tag = gst_riff_peek_tag (riff, NULL)))
+          return FALSE;
+      }
+      break;
+
+    case GST_RIFF_TAG_JUNK:
+      gst_riff_read_skip (riff);
+      return FALSE;
+  }
+
+  if (ret_tag)
+    *ret_tag = tag;
+
   return TRUE;
 }
 
 /*
  * Scan the file for all chunks to "create" a new index.
+ * Return value indicates if we can continue reading the stream. It
+ * does not say anything about whether we created an index.
  */
 
-gboolean
-gst_avi_demux_stream_scan (GstAviDemux * avi)
+static gboolean
+gst_avi_demux_stream_scan (GstAviDemux * avi,
+    GList ** index, GList ** alloc_list)
 {
-  //GstRiffRead *riff = GST_RIFF_READ (avi);
+  GstRiffRead *riff = GST_RIFF_READ (avi);
+  gst_avi_index_entry *entry, *entries = NULL;
+  avi_stream_context *stream;
+  guint64 pos = gst_bytestream_tell (riff->bs);
+  guint32 tag;
+  GstEvent *event;
+  GList *list = NULL;
+  guint index_size = 0;
 
-  /* FIXME */
+  /* FIXME:
+   * - implement non-seekable source support.
+   */
+
+  GST_LOG_OBJECT (avi, "Creating index");
+
+  while (gst_avi_demux_sync (avi, &tag, TRUE)) {
+    gint stream_nr = CHUNKID_TO_STREAMNR (tag);
+    guint8 *data;
+    GstFormat format = GST_FORMAT_TIME;
+
+    if (stream_nr < 0 || stream_nr >= avi->num_streams)
+      goto next;
+    stream = &avi->stream[stream_nr];
+
+    /* get chunk size */
+    if (gst_bytestream_peek_bytes (riff->bs, &data, 8) != 8)
+      goto next;
+
+    /* pre-allocate */
+    if (index_size % 1024 == 0) {
+      entries = g_new (gst_avi_index_entry, 1024);
+      *alloc_list = g_list_prepend (*alloc_list, entries);
+    }
+    entry = &entries[index_size % 1024];
+
+    /* fill in */
+    entry->index_nr = index_size++;
+    entry->stream_nr = stream_nr;
+    entry->flags = 0;
+    entry->offset = gst_bytestream_tell (riff->bs) + 8;
+    entry->size = GST_READ_UINT32_LE (&data[4]);
+
+    /* timestamps */
+    if (stream->strh->samplesize && stream->strh->type == GST_RIFF_FCC_auds) {
+      /* constant rate stream */
+      gst_pad_convert (stream->pad, GST_FORMAT_BYTES,
+          stream->total_bytes, &format, &entry->ts);
+      gst_pad_convert (stream->pad, GST_FORMAT_BYTES,
+          stream->total_bytes + entry->size, &format, &entry->dur);
+    } else {
+      /* VBR stream */
+      gst_pad_convert (stream->pad, GST_FORMAT_DEFAULT,
+          stream->total_frames, &format, &entry->ts);
+      gst_pad_convert (stream->pad, GST_FORMAT_DEFAULT,
+          stream->total_frames + 1, &format, &entry->dur);
+    }
+    entry->dur -= entry->ts;
+
+    /* stream position */
+    entry->bytes_before = stream->total_bytes;
+    stream->total_bytes += entry->size;
+    entry->frames_before = stream->total_frames;
+    stream->total_frames++;
+
+    list = g_list_prepend (list, entry);
+
+  next:
+    if (!gst_riff_read_skip (riff))
+      break;
+  }
+
+  avi->index_offset = 0;
+
+  /* seek back */
+  if (!(event = gst_riff_read_seek (riff, pos))) {
+    g_list_free (list);
+    return FALSE;
+  }
+  gst_event_unref (event);
+
+  GST_LOG_OBJECT (avi, "index created, %d items", index_size);
+
+  *index = g_list_reverse (list);
 
   return TRUE;
 }
 
+/*
+ * Massage index.
+ * We're going to go over each entry in the index and finetune
+ * some things we don't like about AVI. For example, a single
+ * chunk might be too long. Also, individual streams might be
+ * out-of-sync. In the first case, we cut the chunk in several
+ * smaller pieces. In the second case, we re-order chunk reading
+ * order. The end result should be a smoother playing AVI.
+ */
+
+static gint G_GNUC_UNUSED
+sort (gst_avi_index_entry * a, gst_avi_index_entry * b)
+{
+  if (a->ts > b->ts)
+    return 1;
+  else if (a->ts < b->ts)
+    return -1;
+  else
+    return a->stream_nr - b->stream_nr;
+}
+
+static void
+gst_avi_demux_massage_index (GstAviDemux * avi,
+    GList * list, GList * alloc_list)
+{
+  gst_avi_index_entry *entry;
+  avi_stream_context *stream;
+  gint i;
+  GList *one;
+
+  GST_LOG ("Starting index massage");
+
+  /* init frames */
+  for (i = 0; i < avi->num_streams; i++) {
+    stream = &avi->stream[i];
+    if (stream->strh->type == GST_RIFF_FCC_vids) {
+      if (stream->strh->rate != 0)
+        stream->delay = stream->strh->init_frames * GST_SECOND *
+            stream->strh->scale / stream->strh->rate;
+    } else {
+      if (stream->total_frames * stream->bitrate != 0)
+        stream->delay = GST_SECOND * stream->strh->init_frames *
+            stream->strh->length / (stream->total_frames * stream->bitrate);
+    }
+  }
+  for (one = list; one != NULL; one = one->next) {
+    entry = one->data;
+
+    if (entry->stream_nr >= avi->num_streams)
+      continue;
+
+    stream = &avi->stream[entry->stream_nr];
+    entry->ts += stream->delay;
+  }
+
+  GST_LOG ("I'm now going to cut large chunks into smaller pieces");
+
+  /* cut chunks in small (seekable) pieces */
+  for (one = list; one != NULL; one = one->next) {
+    entry = one->data;
+
+    if (entry->stream_nr >= avi->num_streams)
+      continue;
+
+#define MAX_DURATION (GST_SECOND / 2)
+
+    /* check for max duration of a single buffer. I suppose that
+     * the allocation of index entries could be improved. */
+    stream = &avi->stream[entry->stream_nr];
+    if (entry->dur > MAX_DURATION && stream->strh->type == GST_RIFF_FCC_auds) {
+      guint32 ideal_size = stream->bitrate / 10;
+      gst_avi_index_entry *entries;
+      gint old_size, num_added;
+      GList *one2;
+
+      /* copy index */
+      old_size = entry->size;
+      num_added = (entry->size - 1) / ideal_size;
+      avi->index_size += num_added;
+      entries = g_malloc (sizeof (gst_avi_index_entry) * num_added);
+      alloc_list = g_list_prepend (alloc_list, entries);
+      for (one2 = one->next; one2 != NULL; one2 = one2->next) {
+        gst_avi_index_entry *entry2 = one2->data;
+
+        entry2->index_nr += num_added;
+        if (entry2->stream_nr == entry->stream_nr)
+          entry2->frames_before += num_added;
+      }
+
+      /* new sized index chunks */
+      for (i = 0; i < num_added + 1; i++) {
+        gst_avi_index_entry *entry2;
+
+        if (i == 0) {
+          entry2 = entry;
+        } else {
+          entry2 = &entries[i - 1];
+          list = g_list_insert_before (list, one->next, entry2);
+          entry = one->data;
+          one = one->next;
+          memcpy (entry2, entry, sizeof (gst_avi_index_entry));
+        }
+
+        if (old_size >= ideal_size) {
+          entry2->size = ideal_size;
+          old_size -= ideal_size;
+        } else {
+          entry2->size = old_size;
+        }
+
+        entry2->dur = GST_SECOND * entry2->size / stream->bitrate;
+        if (i != 0) {
+          entry2->index_nr++;
+          entry2->ts += entry->dur;
+          entry2->offset += entry->size;
+          entry2->bytes_before += entry->size;
+          entry2->frames_before++;
+        }
+      }
+    }
+  }
+
+  GST_LOG ("I'm now going to reorder the index entries for time");
+
+  /* re-order for time */
+  list = g_list_sort (list, (GCompareFunc) sort);
+
+  GST_LOG ("Filling in index array");
+
+  avi->index_size = g_list_length (list);
+  avi->index_entries = g_new (gst_avi_index_entry, avi->index_size);
+  for (i = 0, one = list; one != NULL; one = one->next, i++) {
+    entry = one->data;
+    memcpy (&avi->index_entries[i], entry, sizeof (gst_avi_index_entry));
+    avi->index_entries[i].index_nr = i;
+  }
+
+  GST_LOG ("Freeing original index list");
+
+  g_list_foreach (alloc_list, (GFunc) g_free, NULL);
+  g_list_free (alloc_list);
+  g_list_free (list);
+
+  GST_LOG ("Index massaging done");
+}
+
 /*
  * Read full AVI headers.
  */
@@ -1170,6 +1477,7 @@ gst_avi_demux_stream_header (GstAviDemux * avi)
 {
   GstRiffRead *riff = GST_RIFF_READ (avi);
   guint32 tag, flags, streams;
+  GList *index = NULL, *alloc = NULL;
 
   /* the header consists of a 'hdrl' LIST tag */
   if (!(tag = gst_riff_peek_tag (riff, NULL)))
@@ -1262,9 +1570,10 @@ gst_avi_demux_stream_header (GstAviDemux * avi)
     g_warning ("Stream header mentioned %d streams, but %d available",
         streams, avi->num_streams);
   }
-
-  /* we've got streaminfo now */
-  g_object_notify (G_OBJECT (avi), "streaminfo");
+  /* at this point we know all the streams and we can signal the no more
+   * pads signal */
+  GST_DEBUG ("signaling no more pads");
+  gst_element_no_more_pads (GST_ELEMENT (avi));
 
   /* Now, find the data (i.e. skip all junk between header and data) */
   while (1) {
@@ -1291,11 +1600,25 @@ gst_avi_demux_stream_header (GstAviDemux * avi)
 
   /* create or read stream index (for seeking) */
   if (flags & GST_RIFF_AVIH_HASINDEX) {
-    if (!gst_avi_demux_stream_index (avi))
+    if (!gst_avi_demux_stream_index (avi, &index, &alloc)) {
+      g_list_foreach (alloc, (GFunc) g_free, NULL);
+      g_list_free (alloc);
       return FALSE;
-  } else {
-    if (!gst_avi_demux_stream_scan (avi))
+    }
+  }
+  if (!index) {
+    if (!gst_avi_demux_stream_scan (avi, &index, &alloc)) {
+      g_list_foreach (alloc, (GFunc) g_free, NULL);
+      g_list_free (alloc);
       return FALSE;
+    }
+  }
+  if (index)
+    gst_avi_demux_massage_index (avi, index, alloc);
+  else {
+    g_list_free (index);
+    g_list_foreach (alloc, (GFunc) g_free, NULL);
+    g_list_free (alloc);
   }
 
   return TRUE;
@@ -1308,23 +1631,24 @@ gst_avi_demux_stream_header (GstAviDemux * avi)
 static gboolean
 gst_avi_demux_handle_seek (GstAviDemux * avi)
 {
-  GstRiffRead *riff = GST_RIFF_READ (avi);
   guint i;
   GstEvent *event;
 
   /* FIXME: if we seek in an openDML file, we will have multiple
    * primary levels. Seeking in between those will cause havoc. */
 
-  if (!(event = gst_riff_read_seek (riff, avi->seek_offset)))
-    return FALSE;
-  gst_event_unref (event);
+  avi->current_entry = avi->seek_entry;
 
   for (i = 0; i < avi->num_streams; i++) {
     avi_stream_context *stream = &avi->stream[i];
 
     if (GST_PAD_IS_USABLE (stream->pad)) {
+      if (avi->seek_flush) {
+        event = gst_event_new (GST_EVENT_FLUSH);
+        gst_pad_push (stream->pad, GST_DATA (event));
+      }
       event = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
-          avi->last_seek + stream->delay, NULL);
+          avi->last_seek, NULL);
       gst_pad_push (stream->pad, GST_DATA (event));
     }
   }
@@ -1332,6 +1656,74 @@ gst_avi_demux_handle_seek (GstAviDemux * avi)
   return TRUE;
 }
 
+static gboolean
+gst_avi_demux_process_next_entry (GstAviDemux * avi)
+{
+  GstRiffRead *riff = GST_RIFF_READ (avi);
+  gboolean processed;
+
+  for (processed = FALSE; !processed;) {
+    if (avi->current_entry >= avi->index_size) {
+      gst_bytestream_seek (riff->bs, 0, GST_SEEK_METHOD_END);
+
+      /* get eos */
+      gst_riff_peek_tag (GST_RIFF_READ (avi), &avi->level_up);
+      gst_pad_event_default (avi->sinkpad, gst_event_new (GST_EVENT_EOS));
+      processed = TRUE;
+    } else {
+      GstBuffer *buf;
+      guint got;
+      gst_avi_index_entry *entry = &avi->index_entries[avi->current_entry++];
+      avi_stream_context *stream;
+
+      if (entry->stream_nr >= avi->num_streams) {
+        continue;
+      }
+
+      stream = &avi->stream[entry->stream_nr];
+
+      if (GST_PAD_IS_USABLE (stream->pad) && entry->size > 0) {
+        guint64 needed_off = entry->offset + avi->index_offset, pos;
+        guint32 remain;
+
+        pos = gst_bytestream_tell (riff->bs);
+        gst_bytestream_get_status (riff->bs, &remain, NULL);
+        if (pos <= needed_off && needed_off - pos <= remain) {
+          gst_bytestream_flush_fast (riff->bs, needed_off - pos);
+        } else {
+          GstEvent *event;
+
+          event = gst_riff_read_seek (riff, needed_off);
+          if (event)
+            gst_event_unref (event);
+          else {
+            GST_ELEMENT_ERROR (avi, RESOURCE, READ, (NULL), (NULL));
+            return FALSE;
+          }
+        }
+        if (!(buf = gst_riff_read_element_data (riff, entry->size, &got))) {
+          return FALSE;
+        }
+        if (entry->flags & GST_RIFF_IF_KEYFRAME) {
+          GST_BUFFER_FLAG_SET (buf, GST_BUFFER_KEY_UNIT);
+        }
+        GST_BUFFER_TIMESTAMP (buf) = entry->ts;
+        GST_BUFFER_DURATION (buf) = entry->dur;
+        GST_DEBUG_OBJECT (avi, "Processing buffer of size %d and time %"
+            GST_TIME_FORMAT " on pad %s",
+            GST_BUFFER_SIZE (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+            gst_pad_get_name (stream->pad));
+        gst_pad_push (stream->pad, GST_DATA (buf));
+        processed = TRUE;
+      }
+      stream->current_frame++;
+      stream->current_byte += entry->size;
+    }
+  }
+
+  return TRUE;
+}
+
 /*
  * Read data.
  */
@@ -1342,7 +1734,6 @@ gst_avi_demux_stream_data (GstAviDemux * avi)
   GstRiffRead *riff = GST_RIFF_READ (avi);
   guint32 tag;
   guint stream_nr;
-  gst_avi_index_entry *entry;
 
   if (avi->seek_offset != (guint64) - 1) {
     if (!gst_avi_demux_handle_seek (avi))
@@ -1350,83 +1741,19 @@ gst_avi_demux_stream_data (GstAviDemux * avi)
     avi->seek_offset = (guint64) - 1;
   }
 
-  /* peek first (for the end of this 'list/movi' section) */
-  if (!(tag = gst_riff_peek_tag (riff, &avi->level_up)))
-    return FALSE;
-
-  /* if we're at top-level, we didn't read the 'movi'
-   * list tag yet. This can also be 'AVIX' in case of
-   * openDML-2.0 AVI files. Lastly, it might be idx1,
-   * in which case we skip it so we come at EOS. */
-  while (g_list_length (riff->level) < 2) {
-    if (!(tag = gst_riff_peek_tag (riff, NULL)))
-      return FALSE;
-
-    switch (tag) {
-      case GST_RIFF_TAG_LIST:
-        if (!(tag = gst_riff_peek_list (riff)))
-          return FALSE;
-
-        switch (tag) {
-          case GST_RIFF_LIST_AVIX:
-          case GST_RIFF_LIST_movi:
-            if (!gst_riff_read_list (riff, &tag))
-              return FALSE;
-            /* we're now going to read buffers! */
-            break;
-
-          default:
-            GST_WARNING ("Unknown list " GST_FOURCC_FORMAT " before AVI data",
-                GST_FOURCC_ARGS (tag));
-            /* fall-through */
-
-          case GST_RIFF_TAG_JUNK:
-            if (!gst_riff_read_skip (riff))
-              return FALSE;
-            break;
-        }
-
-        break;
-
-      default:
-        GST_WARNING ("Unknown tag " GST_FOURCC_FORMAT " before AVI data",
-            GST_FOURCC_ARGS (tag));
-        /* fall-through */
-
-      case GST_RIFF_TAG_idx1:
-      case GST_RIFF_TAG_JUNK:
-        if (!gst_riff_read_skip (riff))
-          return FALSE;
-        break;
-    }
+  /* if we have a avi->index_entries[], we don't want to read
+   * the stream linearly, but seek to the next ts/index_entry. */
+  if (avi->index_entries != NULL) {
+    return gst_avi_demux_process_next_entry (avi);
   }
 
-  /* And then, we get the data */
-  if (!(tag = gst_riff_peek_tag (riff, NULL)))
+  if (!gst_avi_demux_sync (avi, &tag, FALSE))
     return FALSE;
-
-  /* Support for rec-list files */
-  switch (tag) {
-    case GST_RIFF_TAG_LIST:
-      if (!(tag = gst_riff_peek_list (riff)))
-        return FALSE;
-      if (tag == GST_RIFF_rec) {
-        /* Simply skip the list */
-        if (!gst_riff_read_list (riff, &tag))
-          return FALSE;
-        if (!(tag = gst_riff_peek_tag (riff, NULL)))
-          return FALSE;
-      }
-      break;
-
-    case GST_RIFF_TAG_JUNK:
-      return gst_riff_read_skip (riff);
-  }
-
   stream_nr = CHUNKID_TO_STREAMNR (tag);
+
   if (stream_nr < 0 || stream_nr >= avi->num_streams) {
     /* recoverable */
-    g_warning ("Invalid stream ID %d (" GST_FOURCC_FORMAT ")",
+    GST_WARNING ("Invalid stream ID %d (" GST_FOURCC_FORMAT ")",
         stream_nr, GST_FOURCC_ARGS (tag));
     if (!gst_riff_read_skip (riff))
       return FALSE;
@@ -1442,14 +1769,6 @@ gst_avi_demux_stream_data (GstAviDemux * avi)
 
     /* get time of this buffer */
     stream = &avi->stream[stream_nr];
-    entry = gst_avi_demux_index_next (avi, stream_nr,
-        stream->current_entry + 1, 0);
-    if (entry) {
-      stream->current_entry = entry->index_nr;
-      if (entry->flags & GST_RIFF_IF_KEYFRAME) {
-        GST_BUFFER_FLAG_SET (buf, GST_BUFFER_KEY_UNIT);
-      }
-    }
     format = GST_FORMAT_TIME;
     gst_pad_query (stream->pad, GST_QUERY_POSITION, &format, &next_ts);
 
@@ -1474,7 +1793,8 @@ gst_avi_demux_stream_data (GstAviDemux * avi)
         GST_BUFFER_TIMESTAMP (buf) = next_ts;
         gst_pad_query (stream->pad, GST_QUERY_POSITION, &format, &dur_ts);
         GST_BUFFER_DURATION (buf) = dur_ts - next_ts;
-
+        GST_DEBUG ("Pushing buffer with time=%" GST_TIME_FORMAT " over pad %s",
+            GST_TIME_ARGS (next_ts), gst_pad_get_name (stream->pad));
         gst_pad_push (stream->pad, GST_DATA (buf));
       }
     }
@@ -1499,7 +1819,7 @@ gst_avi_demux_loop (GstElement * element)
       if (!gst_avi_demux_stream_header (avi))
         return;
       avi->state = GST_AVI_DEMUX_MOVI;
-      /* fall-through */
+      break;
 
     case GST_AVI_DEMUX_MOVI:
       if (!gst_avi_demux_stream_data (avi))
@@ -1517,9 +1837,6 @@ gst_avi_demux_change_state (GstElement * element)
   GstAviDemux *avi = GST_AVI_DEMUX (element);
 
   switch (GST_STATE_TRANSITION (element)) {
-    case GST_STATE_READY_TO_PAUSED:
-      gst_avi_demux_streaminfo (avi);
-      break;
     case GST_STATE_PAUSED_TO_READY:
       gst_avi_demux_reset (avi);
       break;
@@ -1532,19 +1849,3 @@ gst_avi_demux_change_state (GstElement * element)
 
   return GST_STATE_SUCCESS;
 }
-
-static void
-gst_avi_demux_get_property (GObject * object,
-    guint prop_id, GValue * value, GParamSpec * pspec)
-{
-  GstAviDemux *avi = GST_AVI_DEMUX (object);
-
-  switch (prop_id) {
-    case ARG_STREAMINFO:
-      g_value_set_boxed (value, avi->streaminfo);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
