@@ -19,62 +19,68 @@
  */
 
 
-#ifndef __GST_CDXA_PARSE_H__
-#define __GST_CDXA_PARSE_H__
+#ifndef __GST_CDXAPARSE_H__
+#define __GST_CDXAPARSE_H__
 
 #include <gst/gst.h>
-#include <gst/bytestream/bytestream.h>
+#include "gst/riff/riff-ids.h"
+#include "gst/riff/riff-read.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
-#define GST_TYPE_CDXA_PARSE \
-  (gst_cdxa_parse_get_type())
-#define GST_CDXA_PARSE(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_CDXA_PARSE,GstCDXAParse))
-#define GST_CDXA_PARSE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_CDXA_PARSE,GstCDXAParse))
-#define GST_IS_CDXA_PARSE(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_CDXA_PARSE))
-#define GST_IS_CDXA_PARSE_CLASS(obj) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_CDXA_PARSE))
+#define GST_TYPE_CDXAPARSE \
+  (gst_cdxaparse_get_type())
+#define GST_CDXAPARSE(obj) \
+  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_CDXAPARSE,GstCDXAParse))
+#define GST_CDXAPARSE_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_CDXAPARSE,GstCDXAParse))
+#define GST_IS_CDXAPARSE(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_CDXAPARSE))
+#define GST_IS_CDXAPARSE_CLASS(obj) \
+  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_CDXAPARSE))
 
-#define CDXA_SECTOR_SIZE  	2352
-#define CDXA_DATA_SIZE  	2324
+#define GST_CDXA_SECTOR_SIZE  	2352
+#define GST_CDXA_DATA_SIZE  	2324
+#define GST_CDXA_HEADER_SIZE	24
 
-typedef enum
-{
-  CDXA_PARSE_HEADER,
-  CDXA_PARSE_DATA,
+typedef enum {
+  GST_CDXAPARSE_START,
+  GST_CDXAPARSE_FMT,
+  GST_CDXAPARSE_OTHER,
+  GST_CDXAPARSE_DATA,
 } GstCDXAParseState;
 
 typedef struct _GstCDXAParse GstCDXAParse;
 typedef struct _GstCDXAParseClass GstCDXAParseClass;
 
 struct _GstCDXAParse {
-  GstElement element;
+  GstRiffRead parent;
 
   /* pads */
-  GstPad *sinkpad, *srcpad;
-
-  GstByteStream *bs;
+  GstPad *sinkpad,*srcpad;
 
+  /* CDXA decoding state */
   GstCDXAParseState state;
 
-  guint32 riff_size;
-  guint32 data_size;
-  guint32 sectors;
+  guint64 dataleft, datasize, datastart;
+  int byteoffset;
+  
+  gboolean seek_pending;
+  guint64 seek_offset;
 };
 
 struct _GstCDXAParseClass {
   GstElementClass parent_class;
 };
 
-GType 		gst_cdxa_parse_get_type		(void);
+GType 		gst_cdxaparse_get_type		(void);
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* __GST_CDXA_PARSE_H__ */
+
+#endif /* __GST_CDXAPARSE_H__ */
+
