@@ -51,6 +51,7 @@ struct _InterleaveInputChannel
 {
   GstPad *sinkpad;
   GstBuffer *buffer;
+  gboolean new_media;
 };
 
 struct _Interleave
@@ -412,6 +413,22 @@ interleave_link (GstPad * pad, const GstCaps * caps)
   return GST_PAD_LINK_OK;
 }
 
+static gboolean
+all_channels_new_media (GList * channels)
+{
+  GList *p;
+
+  for (p = channels; p; p = p->next) {
+    InterleaveInputChannel *c = INTERLEAVE_CHANNEL (p);
+
+    if (c->new_media == FALSE) {
+      return FALSE;
+    }
+  }
+
+  return TRUE;
+}
+
 static void
 interleave_buffered_loop (GstElement * element)
 {
@@ -438,6 +455,10 @@ interleave_buffered_loop (GstElement * element)
   for (l = this->channels, i = 0; l; l = l->next, i++) {
     channel = INTERLEAVE_CHANNEL (l);
 
+    if (channel->new_media) {
+      continue;
+    }
+
   get_buffer:
     channel->buffer = GST_BUFFER (gst_pad_pull (channel->sinkpad));
 
@@ -454,6 +475,33 @@ interleave_buffered_loop (GstElement * element)
           gst_pad_push (this->srcpad, GST_DATA (e));
           gst_element_set_eos (GST_ELEMENT (this));
           return;
+
+        case GST_EVENT_DISCONTINUOUS:
+          GST_DEBUG ("its a discont");
+
+          if (GST_EVENT_DISCONT_NEW_MEDIA (e)) {
+            channel->new_media = TRUE;
+            if (all_channels_new_media (this->channels)) {
+              GList *t;
+
+              /* Reset new media */
+              for (t = this->channels; t; t = t->next) {
+                InterleaveInputChannel *c = INTERLEAVE_CHANNEL (t);
+
+                c->new_media = FALSE;
+              }
+
+              /* Push event */
+              gst_pad_event_default (channel->sinkpad, e);
+              goto get_buffer;
+            } else {
+              GST_DEBUG ("Not all channels have received a new-media yet");
+              gst_event_unref (e);
+              goto get_buffer;
+            }
+          }
+          break;
+
         default:
           GST_DEBUG ("doing default action for event");
           gst_pad_event_default (channel->sinkpad, e);
