@@ -114,6 +114,64 @@ gst_ebml_read_change_state (GstElement * element)
   return GST_STATE_SUCCESS;
 }
 
+/*
+ * Event handler. Basic:
+ * - EOS: end-of-file, stop processing, forward EOS.
+ * - Interrupt: stop processing.
+ * - Discont: shouldn't be handled here but in the seek handler. Error.
+ * - Flush: ignore, since we check for flush flags manually. Don't forward.
+ * - Others: warn, ignore.
+ * Return value indicates whether to continue processing.
+ */
+
+static gboolean
+gst_ebml_read_use_event (GstEbmlRead * ebml, GstEvent * event)
+{
+  if (!event) {
+    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL), (NULL));
+    return FALSE;
+  }
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_EOS:
+      gst_pad_event_default (ebml->sinkpad, event);
+      return FALSE;
+
+    case GST_EVENT_INTERRUPT:
+      gst_event_unref (event);
+      return FALSE;
+
+    case GST_EVENT_DISCONTINUOUS:
+      GST_WARNING_OBJECT (ebml, "Unexpected discont - might lose sync");
+      gst_pad_event_default (ebml->sinkpad, event);
+      return TRUE;
+
+    case GST_EVENT_FLUSH:
+      gst_event_unref (event);
+      return TRUE;
+
+    default:
+      GST_WARNING ("don't know how to handle event %d", GST_EVENT_TYPE (event));
+      gst_pad_event_default (ebml->sinkpad, event);
+      return FALSE;
+  }
+
+  /* happy */
+  g_assert_not_reached ();
+  return FALSE;
+}
+
+static gboolean
+gst_ebml_read_handle_event (GstEbmlRead * ebml)
+{
+  GstEvent *event = NULL;
+  guint32 remaining;
+
+  gst_bytestream_get_status (ebml->bs, &remaining, &event);
+
+  return gst_ebml_read_use_event (ebml, event);
+}
+
 /*
  * Return: the amount of levels in the hierarchy that the
  * current element lies higher than the previous one.
@@ -155,21 +213,8 @@ gst_ebml_read_element_id (GstEbmlRead * ebml, guint32 * id, guint * level_up)
   guint32 total;
 
   while (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
-    GstEvent *event = NULL;
-    guint32 remaining;
-
-    /* Here, we might encounter EOS */
-    gst_bytestream_get_status (ebml->bs, &remaining, &event);
-    if (event) {
-      gst_pad_event_default (ebml->sinkpad, event);
-    } else {
-      guint64 pos = gst_bytestream_tell (ebml->bs);
-
-      gst_event_unref (event);
-      GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-          ("Read error at position %llu (0x%llx)", pos, pos));
+    if (!gst_ebml_read_handle_event (ebml))
       return -1;
-    }
   }
   total = data[0];
   while (read <= 4 && !(total & len_mask)) {
@@ -185,12 +230,9 @@ gst_ebml_read_element_id (GstEbmlRead * ebml, guint32 * id, guint * level_up)
     return -1;
   }
 
-  if (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
-    guint64 pos = gst_bytestream_tell (ebml->bs);
-
-    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-        ("Read error at position %llu (0x%llx)", pos, pos));
-    return -1;
+  while (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
+    if (!gst_ebml_read_handle_event (ebml))
+      return -1;
   }
   while (n < read)
     total = (total << 8) | data[n++];
@@ -216,12 +258,9 @@ gst_ebml_read_element_length (GstEbmlRead * ebml, guint64 * length)
   gint len_mask = 0x80, read = 1, n = 1, num_ffs = 0;
   guint64 total;
 
-  if (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
-    guint64 pos = gst_bytestream_tell (ebml->bs);
-
-    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-        ("Read error at position %llu (0x%llx)", pos, pos));
-    return -1;
+  while (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
+    if (!gst_ebml_read_handle_event (ebml))
+      return -1;
   }
   total = data[0];
   while (read <= 8 && !(total & len_mask)) {
@@ -239,12 +278,9 @@ gst_ebml_read_element_length (GstEbmlRead * ebml, guint64 * length)
 
   if ((total &= (len_mask - 1)) == len_mask - 1)
     num_ffs++;
-  if (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
-    guint64 pos = gst_bytestream_tell (ebml->bs);
-
-    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-        ("Read error at position %llu (0x%llx)", pos, pos));
-    return -1;
+  while (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
+    if (!gst_ebml_read_handle_event (ebml))
+      return -1;
   }
   while (n < read) {
     if (data[n] == 0xff)
@@ -272,13 +308,8 @@ gst_ebml_read_element_data (GstEbmlRead * ebml, guint64 length)
   GstBuffer *buf = NULL;
 
   if (gst_bytestream_peek (ebml->bs, &buf, length) != length) {
-    guint64 pos = gst_bytestream_tell (ebml->bs);
-
-    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-        ("Read error at position %llu (0x%llx)", pos, pos));
-    if (buf)
-      gst_buffer_unref (buf);
-    return NULL;
+    if (!gst_ebml_read_handle_event (ebml))
+      return NULL;
   }
 
   gst_bytestream_flush_fast (ebml->bs, length);
@@ -299,7 +330,7 @@ gst_ebml_peek_id (GstEbmlRead * ebml, guint * level_up)
 
   g_assert (level_up);
 
-  if (gst_ebml_read_element_id (ebml, &id, level_up) < 0)
+  if (gst_ebml_read_element_id (ebml, &id, level_up) <= 0)
     return 0;
 
   return id;
@@ -319,8 +350,10 @@ gst_ebml_read_seek (GstEbmlRead * ebml, guint64 offset)
   /* first, flush remaining buffers */
   gst_bytestream_get_status (ebml->bs, &remaining, &event);
   if (event) {
-    g_warning ("Unexpected event before seek");
-    gst_event_unref (event);
+    GST_WARNING ("Unexpected event before seek");
+    if (!gst_ebml_read_use_event (ebml, event))
+      return NULL;
+    event = NULL;
   }
   if (remaining)
     gst_bytestream_flush_fast (ebml->bs, remaining);
@@ -346,7 +379,8 @@ gst_ebml_read_seek (GstEbmlRead * ebml, guint64 offset)
       GST_WARNING ("No discontinuity event after seek - seek failed");
       break;
     } else if (GST_EVENT_TYPE (event) != GST_EVENT_DISCONTINUOUS) {
-      gst_pad_event_default (ebml->sinkpad, event);
+      if (!gst_ebml_read_use_event (ebml, event))
+        return NULL;
       event = NULL;
     }
   }
@@ -378,15 +412,17 @@ gst_ebml_read_skip (GstEbmlRead * ebml)
   gst_bytestream_get_status (ebml->bs, &remaining, &event);
   if (event) {
     g_warning ("Unexpected event before skip");
-    gst_event_unref (event);
+    if (!gst_ebml_read_use_event (ebml, event))
+      return FALSE;
   }
 
   if (remaining >= length)
     return gst_bytestream_flush (ebml->bs, length);
 
   if (!(event = gst_ebml_read_seek (ebml,
-              gst_bytestream_tell (ebml->bs) + length)))
+              gst_bytestream_tell (ebml->bs) + length))) {
     return FALSE;
+  }
 
   gst_event_unref (event);
 
@@ -587,13 +623,18 @@ gst_ebml_read_utf8 (GstEbmlRead * ebml, guint32 * id, gchar ** str)
 }
 
 /*
- * Read the next element as a date (nanoseconds since 1/1/2000).
+ * Read the next element as a date.
+ * Returns the seconds since the unix epoch.
  */
 
 gboolean
 gst_ebml_read_date (GstEbmlRead * ebml, guint32 * id, gint64 * date)
 {
-  return gst_ebml_read_sint (ebml, id, date);
+  gint64 ebml_date;
+  gboolean res = gst_ebml_read_sint (ebml, id, &ebml_date);
+
+  *date = (ebml_date / GST_SECOND) + GST_EBML_DATE_OFFSET;
+  return res;
 }
 
 /*
