@@ -101,7 +101,11 @@ static GstStaticPadTemplate audiosink_templ =
         "width = (int) { 8, 16, 24 }, "
         "depth = (int) { 8, 16, 24 }, "
         "endianness = (int) { BIG_ENDIAN, LITTLE_ENDIAN }, "
-        "signed = (boolean) { true, false }, " COMMON_AUDIO_CAPS)
+        "signed = (boolean) { true, false }, "
+        COMMON_AUDIO_CAPS ";"
+        "audio/x-tta, "
+        "width = (int) { 8, 16, 24 }, "
+        "channels = (int) { 1, 2 }, " "rate = (int) [ 8000, 96000 ]")
     );
 
 static GstStaticPadTemplate subtitlesink_templ =
@@ -110,6 +114,8 @@ GST_STATIC_PAD_TEMPLATE ("subtitle_%d",
     GST_PAD_REQUEST,
     GST_STATIC_CAPS_ANY);
 
+static GArray *used_uids;
+
 /* gobject magic foo */
 static void gst_matroska_mux_base_init (GstMatroskaMuxClass * klass);
 static void gst_matroska_mux_class_init (GstMatroskaMuxClass * klass);
@@ -135,6 +141,9 @@ static void gst_matroska_mux_get_property (GObject * object,
 /* reset muxer */
 static void gst_matroska_mux_reset (GstElement * element);
 
+/* uid generation */
+static guint32 gst_matroska_mux_create_uid ();
+
 static GstEbmlWriteClass *parent_class = NULL;
 
 /*static guint gst_matroska_mux_signals[LAST_SIGNAL] = { 0 };*/
@@ -230,6 +239,7 @@ gst_matroska_mux_init (GstMatroskaMux * mux)
   for (i = 0; i < GST_MATROSKA_MUX_MAX_STREAMS; i++) {
     mux->sink[i].buffer = NULL;
     mux->sink[i].track = NULL;
+    mux->sink[i].duration = 0;
   }
   mux->index = NULL;
 
@@ -237,6 +247,28 @@ gst_matroska_mux_init (GstMatroskaMux * mux)
   gst_matroska_mux_reset (GST_ELEMENT (mux));
 }
 
+static guint32
+gst_matroska_mux_create_uid ()
+{
+  guint32 uid = 0;
+  GRand *rand = g_rand_new ();
+
+  while (!uid) {
+    guint i;
+
+    uid = g_rand_int (rand);
+    for (i = 0; i < used_uids->len; i++) {
+      if (g_array_index (used_uids, guint32, i) == uid) {
+        uid = 0;
+        break;
+      }
+    }
+    g_array_append_val (used_uids, uid);
+  }
+  g_free (rand);
+  return uid;
+}
+
 static void
 gst_matroska_mux_reset (GstElement * element)
 {
@@ -284,6 +316,23 @@ gst_matroska_mux_reset (GstElement * element)
   /* reset timers */
   mux->time_scale = 1000000;
   mux->duration = 0;
+
+  /* reset uid array */
+  if (used_uids) {
+    g_free (used_uids);
+  }
+  /* arbitrary size, 10 should be enough in most cases */
+  used_uids = g_array_sized_new (FALSE, FALSE, sizeof (guint32), 10);
+
+  /* reset cluster */
+  mux->cluster = 0;
+  mux->cluster_time = 0;
+  mux->cluster_pos = 0;
+
+  /* reset meta-seek index */
+  mux->num_meta_indexes = 0;
+  g_free (mux->meta_index);
+  mux->meta_index = NULL;
 }
 
 static GstPadLinkReturn
@@ -358,23 +407,51 @@ gst_matroska_mux_video_pad_link (GstPad * pad, const GstCaps * caps)
     return GST_PAD_LINK_OK;
   } else if (!strcmp (mimetype, "video/x-divx")) {
     gint divxversion;
+    BITMAPINFOHEADER *bih;
+
+    bih = (BITMAPINFOHEADER *) g_malloc0 (sizeof (BITMAPINFOHEADER));
+    GST_WRITE_UINT32_LE (&bih->bi_size, sizeof (BITMAPINFOHEADER));
+    GST_WRITE_UINT32_LE (&bih->bi_width, videocontext->pixel_width);
+    GST_WRITE_UINT32_LE (&bih->bi_height, videocontext->pixel_height);
+    GST_WRITE_UINT16_LE (&bih->bi_planes, (guint16) 1);
+    GST_WRITE_UINT16_LE (&bih->bi_bit_count, (guint16) 24);
+    GST_WRITE_UINT32_LE (&bih->bi_size_image, videocontext->pixel_width *
+        videocontext->pixel_height * 3);
 
     gst_structure_get_int (structure, "divxversion", &divxversion);
     switch (divxversion) {
       case 3:
-        context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_MSMPEG4V3);
+        GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("DIV3"));
         break;
       case 4:
-        context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_MPEG4_SP);
+        GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("DIVX"));
         break;
       case 5:
-        context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_MPEG4_ASP);
+        GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("DX50"));
         break;
     }
 
+    context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_VFW_FOURCC);
+    context->codec_priv = (gpointer) bih;
+    context->codec_priv_size = sizeof (BITMAPINFOHEADER);
+
     return GST_PAD_LINK_OK;
   } else if (!strcmp (mimetype, "video/x-xvid")) {
-    context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_MPEG4_ASP);
+    BITMAPINFOHEADER *bih;
+
+    bih = (BITMAPINFOHEADER *) g_malloc0 (sizeof (BITMAPINFOHEADER));
+    GST_WRITE_UINT32_LE (&bih->bi_size, sizeof (BITMAPINFOHEADER));
+    GST_WRITE_UINT32_LE (&bih->bi_width, videocontext->pixel_width);
+    GST_WRITE_UINT32_LE (&bih->bi_height, videocontext->pixel_height);
+    GST_WRITE_UINT16_LE (&bih->bi_planes, (guint16) 1);
+    GST_WRITE_UINT16_LE (&bih->bi_bit_count, (guint16) 24);
+    GST_WRITE_UINT32_LE (&bih->bi_compression, GST_STR_FOURCC ("XVID"));
+    GST_WRITE_UINT32_LE (&bih->bi_size_image, videocontext->pixel_width *
+        videocontext->pixel_height * 3);
+
+    context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_VIDEO_VFW_FOURCC);
+    context->codec_priv = (gpointer) bih;
+    context->codec_priv_size = sizeof (BITMAPINFOHEADER);
 
     return GST_PAD_LINK_OK;
   } else if (!strcmp (mimetype, "video/mpeg")) {
@@ -435,6 +512,7 @@ gst_matroska_mux_audio_pad_link (GstPad * pad, const GstCaps * caps)
   audiocontext->samplerate = samplerate;
   audiocontext->channels = channels;
   audiocontext->bitdepth = 0;
+  context->default_duration = 0;
 
   if (!strcmp (mimetype, "audio/mpeg")) {
     gint mpegversion = 0;
@@ -493,6 +571,17 @@ gst_matroska_mux_audio_pad_link (GstPad * pad, const GstCaps * caps)
   } else if (!strcmp (mimetype, "audio/x-ac3")) {
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_AC3);
 
+    return GST_PAD_LINK_OK;
+  } else if (!strcmp (mimetype, "audio/x-tta")) {
+    gint width;
+
+    /* TTA frame duration */
+    context->default_duration = 1.04489795918367346939 * GST_SECOND;
+
+    gst_structure_get_int (structure, "width", &width);
+    audiocontext->bitdepth = width;
+    context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_TTA);
+
     return GST_PAD_LINK_OK;
   }
 
@@ -569,16 +658,19 @@ gst_matroska_mux_track_header (GstMatroskaMux * mux,
   gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKNUMBER, context->num);
   gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKTYPE, context->type);
 
+  gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKUID,
+      gst_matroska_mux_create_uid ());
+  if (context->default_duration) {
+    gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKDEFAULTDURATION,
+        context->default_duration);
+  }
+
   /* type-specific stuff */
   switch (context->type) {
     case GST_MATROSKA_TRACK_TYPE_VIDEO:{
       GstMatroskaTrackVideoContext *videocontext =
           (GstMatroskaTrackVideoContext *) context;
 
-      /* framerate, but not in the video part */
-      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TRACKDEFAULTDURATION,
-          context->default_duration);
-
       master = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_TRACKVIDEO);
       gst_ebml_write_uint (ebml, GST_MATROSKA_ID_VIDEOPIXELWIDTH,
           videocontext->pixel_width);
@@ -647,6 +739,7 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
   guint32 seekhead_id[] = { GST_MATROSKA_ID_INFO,
     GST_MATROSKA_ID_TRACKS,
     GST_MATROSKA_ID_CUES,
+    GST_MATROSKA_ID_SEEKHEAD,
 #if 0
     GST_MATROSKA_ID_TAGS,
 #endif
@@ -655,6 +748,10 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
   guint64 master, child;
   gint i;
   guint tracknum = 1;
+  gdouble duration = 0;
+  guint32 *segment_uid = (guint32 *) g_malloc (16);
+  GRand *rand = g_rand_new ();
+  GTimeVal time = { 0, 0 };
 
   /* we start with a EBML header */
   gst_ebml_write_header (ebml, "matroska", 1);
@@ -681,12 +778,32 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
   /* segment info */
   mux->info_pos = ebml->pos;
   master = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_INFO);
+  for (i = 0; i < 4; i++) {
+    segment_uid[i] = g_rand_int (rand);
+  }
+  g_free (rand);
+  gst_ebml_write_binary (ebml, GST_MATROSKA_ID_SEGMENTUID,
+      (guint8 *) segment_uid, 16);
   gst_ebml_write_uint (ebml, GST_MATROSKA_ID_TIMECODESCALE, mux->time_scale);
   mux->duration_pos = ebml->pos;
-  gst_ebml_write_float (ebml, GST_MATROSKA_ID_DURATION, 0);
-  gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_MUXINGAPP, "GStreamer");
-  if (mux->metadata &&
-      gst_structure_has_field (gst_caps_get_structure (mux->metadata, 0),
+  /* get duration */
+  for (i = 0; i < mux->num_streams; i++) {
+    gint64 trackduration;
+    GstFormat format = GST_FORMAT_TIME;
+
+    if (gst_pad_query (GST_PAD_PEER (mux->sink[i].track->pad), GST_QUERY_TOTAL,
+            &format, &trackduration)) {
+      if ((gdouble) trackduration > duration) {
+        duration = (gdouble) trackduration;
+      }
+    }
+  }
+  gst_ebml_write_float (ebml, GST_MATROSKA_ID_DURATION,
+      duration / mux->time_scale);
+  gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_MUXINGAPP,
+      "GStreamer plugin version " GST_PLUGINS_VERSION);
+  if (mux->metadata
+      && gst_structure_has_field (gst_caps_get_structure (mux->metadata, 0),
           "application")) {
     const gchar *app;
 
@@ -696,8 +813,8 @@ gst_matroska_mux_start (GstMatroskaMux * mux)
       gst_ebml_write_utf8 (ebml, GST_MATROSKA_ID_WRITINGAPP, app);
     }
   }
-  /* FIXME: how do I get this? Automatic? Via tags? */
-  /*gst_ebml_write_date (ebml, GST_MATROSKA_ID_DATEUTC, 0); */
+  g_get_current_time (&time);
+  gst_ebml_write_date (ebml, GST_MATROSKA_ID_DATEUTC, time.tv_sec);
   gst_ebml_write_master_finish (ebml, master);
 
   /* tracks */
@@ -722,6 +839,13 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
 {
   GstEbmlWrite *ebml = GST_EBML_WRITE (mux);
   guint64 pos;
+  guint64 duration = 0;
+  gint i;
+
+  /* finish last cluster */
+  if (mux->cluster) {
+    gst_ebml_write_master_finish (ebml, mux->cluster);
+  }
 
   /* cues */
   if (mux->index != NULL) {
@@ -737,7 +861,7 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
 
       pointentry_master = gst_ebml_write_master_start (ebml,
           GST_MATROSKA_ID_POINTENTRY);
-      gst_ebml_write_date (ebml, GST_MATROSKA_ID_CUETIME,
+      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CUETIME,
           idx->time / mux->time_scale);
       trackpos_master = gst_ebml_write_master_start (ebml,
           GST_MATROSKA_ID_CUETRACKPOSITION);
@@ -752,6 +876,28 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
     gst_ebml_write_flush_cache (ebml);
   }
 
+  if (mux->meta_index != NULL) {
+    guint n;
+    guint64 master, seekentry_master;
+
+    mux->meta_pos = ebml->pos;
+    gst_ebml_write_set_cache (ebml, 12 + 28 * mux->num_meta_indexes);
+    master = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_SEEKHEAD);
+
+    for (n = 0; n < mux->num_meta_indexes; n++) {
+      GstMatroskaMetaSeekIndex *idx = &mux->meta_index[n];
+
+      seekentry_master = gst_ebml_write_master_start (ebml,
+          GST_MATROSKA_ID_SEEKENTRY);
+      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_SEEKID, idx->id);
+      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_SEEKPOSITION,
+          idx->pos - mux->segment_master);
+      gst_ebml_write_master_finish (ebml, seekentry_master);
+    }
+    gst_ebml_write_master_finish (ebml, master);
+  }
+  gst_ebml_write_flush_cache (ebml);
+
   /* FIXME: tags */
 
   /* update seekhead. We know that:
@@ -779,17 +925,35 @@ gst_matroska_mux_finish (GstMatroskaMux * mux)
     gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 26);
     gst_ebml_write_seek (ebml, my_pos);
   }
+  if (mux->meta_index != NULL) {
+    gst_ebml_replace_uint (ebml, mux->seekhead_pos + 116,
+        mux->meta_pos - mux->segment_master);
+  } else {
+    /* void'ify */
+    guint64 my_pos = ebml->pos;
+
+    gst_ebml_write_seek (ebml, mux->seekhead_pos + 96);
+    gst_ebml_write_buffer_header (ebml, GST_EBML_ID_VOID, 26);
+    gst_ebml_write_seek (ebml, my_pos);
+  }
 #if 0
   gst_ebml_replace_uint (ebml, mux->seekhead_pos + 116,
       mux->tags_pos - mux->segment_master);
 #endif
 
   /* update duration */
-  pos = GST_EBML_WRITE (mux)->pos;
-  gst_ebml_write_seek (ebml, mux->duration_pos);
-  gst_ebml_write_float (ebml, GST_MATROSKA_ID_DURATION,
-      mux->duration / mux->time_scale);
-  gst_ebml_write_seek (ebml, pos);
+  /* first get the overall duration */
+  for (i = 0; i < mux->num_streams; i++) {
+    if (mux->sink[i].duration > duration)
+      duration = mux->sink[i].duration;
+  }
+  if (duration != 0) {
+    pos = GST_EBML_WRITE (mux)->pos;
+    gst_ebml_write_seek (ebml, mux->duration_pos);
+    gst_ebml_write_float (ebml, GST_MATROSKA_ID_DURATION,
+        (gdouble) duration / mux->time_scale);
+    gst_ebml_write_seek (ebml, pos);
+  }
 
   /* finish segment - this also writes element length */
   gst_ebml_write_master_finish (ebml, mux->segment_pos);
@@ -849,11 +1013,56 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux)
   buf = mux->sink[i].buffer;
   mux->sink[i].buffer = NULL;
 
+  if (mux->cluster) {
+    /* start a new cluster every two seconds */
+    if (mux->cluster_time + GST_SECOND * 2 < GST_BUFFER_TIMESTAMP (buf)) {
+      GstMatroskaMetaSeekIndex *idx;
+
+      gst_ebml_write_master_finish (ebml, mux->cluster);
+      mux->cluster_pos = ebml->pos;
+      mux->cluster =
+          gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_CLUSTER);
+      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CLUSTERTIMECODE,
+          GST_BUFFER_TIMESTAMP (buf) / mux->time_scale);
+      mux->cluster_time = GST_BUFFER_TIMESTAMP (buf);
+
+      if (mux->num_meta_indexes % 32 == 0) {
+        mux->meta_index = g_renew (GstMatroskaMetaSeekIndex, mux->meta_index,
+            mux->num_meta_indexes + 32);
+      }
+      idx = &mux->meta_index[mux->num_meta_indexes++];
+      idx->id = GST_MATROSKA_ID_CLUSTER;
+      idx->pos = mux->cluster_pos;
+    }
+  } else {
+    /* first cluster */
+    GstMatroskaMetaSeekIndex *idx;
+
+    mux->cluster_pos = ebml->pos;
+    mux->cluster = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_CLUSTER);
+    gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CLUSTERTIMECODE,
+        GST_BUFFER_TIMESTAMP (buf) / mux->time_scale);
+    mux->cluster_time = GST_BUFFER_TIMESTAMP (buf);
+
+    if (mux->num_meta_indexes % 32 == 0) {
+      mux->meta_index = g_renew (GstMatroskaMetaSeekIndex, mux->meta_index,
+          mux->num_meta_indexes + 32);
+    }
+    idx = &mux->meta_index[mux->num_meta_indexes++];
+    idx->id = GST_MATROSKA_ID_CLUSTER;
+    idx->pos = mux->cluster_pos;
+  }
+  cluster = mux->cluster;
+
+  /* update duration of this track */
+  if (GST_BUFFER_DURATION_IS_VALID (buf))
+    mux->sink[i].duration += GST_BUFFER_DURATION (buf);
+
   /* We currently write an index entry for each keyframe in a
-   * video track. This can be largely improved, such as doing
+   * video track or one entry for each cluster in an audio track
+   * for audio only files. This can be largely improved, such as doing
    * one for each keyframe or each second (for all-keyframe
-   * streams), only the *first* video track or the audio track
-   * if we have no video tracks. But that'll come later... */
+   * streams), only the *first* video track. But that'll come later... */
   if (mux->sink[i].track->type == GST_MATROSKA_TRACK_TYPE_VIDEO &&
       GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_KEY_UNIT)) {
     GstMatroskaIndex *idx;
@@ -864,40 +1073,68 @@ gst_matroska_mux_write_data (GstMatroskaMux * mux)
     }
     idx = &mux->index[mux->num_indexes++];
 
-    idx->pos = ebml->pos;
+    idx->pos = mux->cluster_pos;
+    idx->time = GST_BUFFER_TIMESTAMP (buf);
+    idx->track = mux->sink[i].track->num;
+  } else if ((mux->sink[i].track->type == GST_MATROSKA_TRACK_TYPE_AUDIO) &&
+      (mux->num_streams == 1)) {
+    GstMatroskaIndex *idx;
+
+    if (mux->num_indexes % 32 == 0) {
+      mux->index = g_renew (GstMatroskaIndex, mux->index,
+          mux->num_indexes + 32);
+    }
+    idx = &mux->index[mux->num_indexes++];
+
+    idx->pos = mux->cluster_pos;
     idx->time = GST_BUFFER_TIMESTAMP (buf);
     idx->track = mux->sink[i].track->num;
   }
 
-  /* write one cluster with one blockgroup with one block with
+  /* write one blockgroup with one block with
    * one slice (*breath*).
-   * FIXME: lacing, multiple frames/cluster, etc. */
-  cluster = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_CLUSTER);
-  gst_ebml_write_uint (ebml, GST_MATROSKA_ID_CLUSTERTIMECODE,
-      GST_BUFFER_TIMESTAMP (buf) / mux->time_scale);
+   * FIXME: lacing, etc. */
   blockgroup = gst_ebml_write_master_start (ebml, GST_MATROSKA_ID_BLOCKGROUP);
   gst_ebml_write_buffer_header (ebml, GST_MATROSKA_ID_BLOCK,
       GST_BUFFER_SIZE (buf) + 4);
   hdr = gst_buffer_new_and_alloc (4);
   /* track num - FIXME: what if num >= 0x80 (unlikely)? */
   GST_BUFFER_DATA (hdr)[0] = mux->sink[i].track->num | 0x80;
-  /* time relative to clustertime - we don't use this yet */
-  *(guint16 *) & GST_BUFFER_DATA (hdr)[1] = GUINT16_TO_BE (0);
+  /* time relative to clustertime */
+  *(guint16 *) & GST_BUFFER_DATA (hdr)[1] = GUINT16_TO_BE (
+      (GST_BUFFER_TIMESTAMP (buf) - mux->cluster_time) / mux->time_scale);
   /* flags - no lacing (yet) */
   GST_BUFFER_DATA (hdr)[3] = 0;
   gst_ebml_write_buffer (ebml, hdr);
   gst_ebml_write_buffer (ebml, buf);
+  if (GST_BUFFER_DURATION_IS_VALID (buf)) {
+    guint64 block_duration = GST_BUFFER_DURATION (buf);
+
+    if (block_duration != mux->sink[i].track->default_duration) {
+      gst_ebml_write_uint (ebml, GST_MATROSKA_ID_BLOCKDURATION,
+          block_duration / mux->time_scale);
+    }
+  }
   gst_ebml_write_master_finish (ebml, blockgroup);
-  gst_ebml_write_master_finish (ebml, cluster);
 }
 
 static void
 gst_matroska_mux_loop (GstElement * element)
 {
   GstMatroskaMux *mux = GST_MATROSKA_MUX (element);
+  guint i;
 
   /* start with a header */
   if (mux->state == GST_MATROSKA_MUX_STATE_START) {
+    if (mux->num_streams == 0) {
+      return;
+    }
+    for (i = 0; i < mux->num_streams; i++) {
+      if (!gst_pad_is_negotiated (mux->sink[i].track->pad)) {
+        return;
+      } else {
+      }
+    }
     mux->state = GST_MATROSKA_MUX_STATE_HEADER;
     gst_matroska_mux_start (mux);
     mux->state = GST_MATROSKA_MUX_STATE_DATA;
