@@ -4,6 +4,10 @@
  * Authors: Olivier Lapicque <olivierl@jps.net>
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "stdafx.h"
 #include "sndfile.h"
 
