@@ -21,7 +21,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <gstmpegdemux.h>
+#include "gstmpegdemux.h"
 
 
 GST_DEBUG_CATEGORY_STATIC (gstmpegdemux_debug);
@@ -175,9 +175,6 @@ gst_mpeg_demux_base_init (GstMPEGDemuxClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
-
   klass->video_template = gst_static_pad_template_get (&video_template);
   klass->audio_template = gst_static_pad_template_get (&audio_template);
   klass->private_template = gst_static_pad_template_get (&private_template);
@@ -218,20 +215,16 @@ gst_mpeg_demux_class_init (GstMPEGDemuxClass * klass)
   klass->get_private_stream = gst_mpeg_demux_get_private_stream;
   klass->send_subbuffer = gst_mpeg_demux_send_subbuffer;
   klass->process_private = gst_mpeg_demux_process_private;
+
+  /* we have our own sink pad template, but don't use it in subclasses */
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_template));
 }
 
 static void
 gst_mpeg_demux_init (GstMPEGDemux * mpeg_demux)
 {
   gint i;
-  GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (mpeg_demux);
-
-  gst_element_remove_pad (GST_ELEMENT (mpeg_parse), mpeg_parse->sinkpad);
-  mpeg_parse->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&sink_template),
-      "sink");
-  gst_element_add_pad (GST_ELEMENT (mpeg_parse), mpeg_parse->sinkpad);
-  gst_element_remove_pad (GST_ELEMENT (mpeg_parse), mpeg_parse->srcpad);
 
   /* i think everything is already zero'd, but oh well */
   for (i = 0; i < GST_MPEG_DEMUX_NUM_VIDEO_STREAMS; i++) {
@@ -331,7 +324,6 @@ gst_mpeg_demux_new_output_pad (GstMPEGDemux * mpeg_demux,
   GstPad *pad;
 
   pad = gst_pad_new_from_template (temp, name);
-  gst_element_add_pad (GST_ELEMENT (mpeg_demux), pad);
 
   gst_pad_set_formats_function (pad, gst_mpeg_demux_get_src_formats);
   gst_pad_set_convert_function (pad, gst_mpeg_parse_convert_src);
@@ -369,6 +361,7 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
   GstMPEGVideoStream *video_str;
   gchar *name;
   GstCaps *caps;
+  gboolean set_caps = FALSE;
 
   g_return_val_if_fail (stream_nr < GST_MPEG_DEMUX_NUM_VIDEO_STREAMS, NULL);
   g_return_val_if_fail (type > GST_MPEG_DEMUX_VIDEO_UNKNOWN &&
@@ -379,7 +372,6 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
   if (str == NULL) {
     video_str = g_new0 (GstMPEGVideoStream, 1);
     str = (GstMPEGStream *) video_str;
-    str->type = GST_MPEG_DEMUX_VIDEO_UNKNOWN;
 
     name = g_strdup_printf ("video_%02d", stream_nr);
     CLASS (mpeg_demux)->init_stream (mpeg_demux, type, str, stream_nr, name,
@@ -387,6 +379,8 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
     g_free (name);
 
     mpeg_demux->video_stream[stream_nr] = str;
+
+    set_caps = TRUE;
   } else {
     /* This stream may have been created by a derived class, reset the
        size. */
@@ -394,16 +388,22 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
     mpeg_demux->video_stream[stream_nr] = str = (GstMPEGStream *) video_str;
   }
 
-  if (str->type != GST_MPEG_DEMUX_VIDEO_MPEG ||
-      video_str->mpeg_version != mpeg_version) {
+  if (set_caps || video_str->mpeg_version != mpeg_version) {
     /* We need to set new caps for this pad. */
     caps = gst_caps_new_simple ("video/mpeg",
         "mpegversion", G_TYPE_INT, mpeg_version,
         "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
-    gst_pad_set_explicit_caps (str->pad, caps);
+    if (!gst_pad_set_explicit_caps (str->pad, caps)) {
+      GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_demux),
+          CORE, NEGOTIATION, (NULL), ("failed to set caps"));
+      gst_caps_free (caps);
+      gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
+      return str;
+    }
+    gst_caps_free (caps);
+    gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
 
     /* Store the current values. */
-    str->type = GST_MPEG_DEMUX_VIDEO_MPEG;
     video_str->mpeg_version = mpeg_version;
   }
 
@@ -417,6 +417,7 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
   GstMPEGStream *str;
   gchar *name;
   GstCaps *caps;
+  gboolean set_caps = FALSE;
 
   g_return_val_if_fail (stream_nr < GST_MPEG_DEMUX_NUM_AUDIO_STREAMS, NULL);
   g_return_val_if_fail (type > GST_MPEG_DEMUX_AUDIO_UNKNOWN &&
@@ -426,7 +427,6 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
 
   if (str == NULL) {
     str = g_new0 (GstMPEGStream, 1);
-    str->type = GST_MPEG_DEMUX_AUDIO_MPEG;
 
     name = g_strdup_printf ("audio_%02d", stream_nr);
     CLASS (mpeg_demux)->init_stream (mpeg_demux, type, str, stream_nr, name,
@@ -434,19 +434,28 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
     g_free (name);
 
     mpeg_demux->audio_stream[stream_nr] = str;
+
+    /* new pad, set caps */
+    set_caps = TRUE;
   } else {
     /* This stream may have been created by a derived class, reset the
        size. */
     str = g_renew (GstMPEGStream, str, 1);
   }
 
-  if (str->type != GST_MPEG_DEMUX_AUDIO_MPEG) {
+  if (set_caps) {
     /* We need to set new caps for this pad. */
     caps = gst_caps_new_simple ("audio/mpeg",
         "mpegversion", G_TYPE_INT, 1, NULL);
-    gst_pad_set_explicit_caps (str->pad, caps);
-
-    str->type = GST_MPEG_DEMUX_AUDIO_MPEG;
+    if (!gst_pad_set_explicit_caps (str->pad, caps)) {
+      GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_demux),
+          CORE, NEGOTIATION, (NULL), ("failed to set caps"));
+      gst_caps_free (caps);
+      gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
+      return str;
+    }
+    gst_caps_free (caps);
+    gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
   }
 
   return str;
@@ -469,6 +478,7 @@ gst_mpeg_demux_get_private_stream (GstMPEGDemux * mpeg_demux,
     CLASS (mpeg_demux)->init_stream (mpeg_demux, type, str, stream_nr, name,
         CLASS (mpeg_demux)->private_template);
     g_free (name);
+    gst_element_add_pad (GST_ELEMENT (mpeg_demux), str->pad);
 
     mpeg_demux->private_stream[stream_nr] = str;
   }
@@ -666,9 +676,9 @@ gst_mpeg_demux_parse_packet (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
           case 0x20:
             /* pts:3 ! 1 ! pts:15 ! 1 | pts:15 ! 1 */
             pts = ((guint64) (bits & 0x0E)) << 29;
-            pts |= ((guint64) * buf++) << 22;
+            pts |= ((guint64) (*buf++)) << 22;
             pts |= ((guint64) (*buf++ & 0xFE)) << 14;
-            pts |= ((guint64) * buf++) << 7;
+            pts |= ((guint64) (*buf++)) << 7;
             pts |= ((guint64) (*buf++ & 0xFE)) >> 1;
 
             GST_DEBUG_OBJECT (mpeg_demux, "PTS = %" G_GUINT64_FORMAT, pts);
@@ -677,9 +687,9 @@ gst_mpeg_demux_parse_packet (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
           case 0x30:
             /* pts:3 ! 1 ! pts:15 ! 1 | pts:15 ! 1 */
             pts = ((guint64) (bits & 0x0E)) << 29;
-            pts |= ((guint64) * buf++) << 22;
+            pts |= ((guint64) (*buf++)) << 22;
             pts |= ((guint64) (*buf++ & 0xFE)) << 14;
-            pts |= ((guint64) * buf++) << 7;
+            pts |= ((guint64) (*buf++)) << 7;
             pts |= ((guint64) (*buf++ & 0xFE)) >> 1;
 
             /* sync:4 ! pts:3 ! 1 ! pts:15 ! 1 | pts:15 ! 1 */
@@ -714,6 +724,7 @@ done:
 
   /* calculate the amount of real data in this packet */
   datalen = packet_length - headerlen + 2;
+
   GST_DEBUG_OBJECT (mpeg_demux, "headerlen is %d, datalen is %d",
       headerlen, datalen);
 
@@ -769,7 +780,7 @@ gst_mpeg_demux_parse_pes (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
 
   guint16 datalen;
   guint16 headerlen;
-  GstClockTime timestamp = GST_CLOCK_TIME_NONE;
+  GstClockTime timestamp;
 
   GstMPEGStream *outstream = NULL;
   guint8 *buf;
@@ -809,11 +820,11 @@ gst_mpeg_demux_parse_pes (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
       pts |= ((guint64) * buf++) << 7;
       pts |= ((guint64) (*buf++ & 0xFE)) >> 1;
 
-      GST_DEBUG_OBJECT (mpeg_demux, "0x%02x PTS = %" G_GUINT64_FORMAT,
-          id, MPEGTIME_TO_GSTTIME (pts));
+      GST_DEBUG_OBJECT (mpeg_demux, "0x%02x (%lld) PTS = %" G_GUINT64_FORMAT,
+          id, pts, MPEGTIME_TO_GSTTIME (pts));
 
       pts += mpeg_parse->adjust;
-      timestamp = MPEGTIME_TO_GSTTIME (pts) + mpeg_demux->adjust;;
+      timestamp = MPEGTIME_TO_GSTTIME (pts) + mpeg_demux->adjust;
     } else {
       timestamp = GST_CLOCK_TIME_NONE;
     }
@@ -848,6 +859,8 @@ gst_mpeg_demux_parse_pes (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
     /* constant corresponds to the 2 bytes of the packet length. */
     headerlen = 2;
     datalen = packet_length;
+
+    timestamp = GST_CLOCK_TIME_NONE;
   }
 
   GST_DEBUG_OBJECT (mpeg_demux, "headerlen is %d, datalen is %d",
@@ -1081,5 +1094,5 @@ gboolean
 gst_mpeg_demux_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "mpegdemux",
-      GST_RANK_PRIMARY, GST_TYPE_MPEG_DEMUX);
+      GST_RANK_SECONDARY, GST_TYPE_MPEG_DEMUX);
 }
