@@ -47,7 +47,7 @@ enum {
   GST_MPEG_DEMUX_STREAM_VIDEO = 1,
   GST_MPEG_DEMUX_STREAM_AUDIO,
   GST_MPEG_DEMUX_STREAM_PRIVATE,
-  GST_MPEG_DEMUX_STREAM_LAST,
+  GST_MPEG_DEMUX_STREAM_LAST
 };
 
 /* Supported number of streams. */
@@ -67,7 +67,7 @@ enum {
   GST_MPEG_DEMUX_VIDEO_UNKNOWN =
     GST_MPEG_DEMUX_STREAM_TYPE (GST_MPEG_DEMUX_STREAM_VIDEO, 1),
   GST_MPEG_DEMUX_VIDEO_MPEG,
-  GST_MPEG_DEMUX_VIDEO_LAST,
+  GST_MPEG_DEMUX_VIDEO_LAST
 };
 
 /* The recognized audio types. */
@@ -75,14 +75,14 @@ enum {
   GST_MPEG_DEMUX_AUDIO_UNKNOWN =
     GST_MPEG_DEMUX_STREAM_TYPE (GST_MPEG_DEMUX_STREAM_AUDIO, 1),
   GST_MPEG_DEMUX_AUDIO_MPEG,
-  GST_MPEG_DEMUX_AUDIO_LAST,
+  GST_MPEG_DEMUX_AUDIO_LAST
 };
 
 /* The recognized private stream types. */
 enum {
   GST_MPEG_DEMUX_PRIVATE_UNKNOWN =
     GST_MPEG_DEMUX_STREAM_TYPE (GST_MPEG_DEMUX_STREAM_PRIVATE, 1),
-  GST_MPEG_DEMUX_PRIVATE_LAST,
+  GST_MPEG_DEMUX_PRIVATE_LAST
 };
 
 typedef struct _GstMPEGStream GstMPEGStream;
