@@ -69,7 +69,7 @@ enum
 
 enum
 {
-  ARG_0,
+  ARG_0
 };
 
 
@@ -105,6 +105,7 @@ gst_overlay_get_type (void)
       0,
       (GInstanceInitFunc) gst_overlay_init,
     };
+
     overlay_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstOverlay", &overlay_info,
         0);
