@@ -67,6 +67,11 @@ static void gst_play_base_bin_remove_element (GstBin * bin,
 extern GstElementStateReturn gst_element_set_state_func (GstElement * element,
     GstElementState state);
 
+static void gst_play_base_bin_error (GstElement * element,
+    GstElement * source, GError * error, gchar * debug, gpointer data);
+static void gst_play_base_bin_found_tag (GstElement * element,
+    GstElement * source, const GstTagList * taglist, gpointer data);
+
 static GstElementClass *element_class;
 static GstElementClass *parent_class;
 static guint gst_play_base_bin_signals[LAST_SIGNAL] = { 0 };
@@ -186,6 +191,7 @@ gst_play_base_bin_dispose (GObject * object)
 
   play_base_bin = GST_PLAY_BASE_BIN (object);
   g_free (play_base_bin->uri);
+  play_base_bin->uri = NULL;
 
   if (G_OBJECT_CLASS (parent_class)->dispose) {
     G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -195,8 +201,11 @@ gst_play_base_bin_dispose (GObject * object)
 static void
 queue_overrun (GstElement * element, GstPlayBaseBin * play_base_bin)
 {
+  GST_DEBUG ("queue %s overrun", gst_element_get_name (element));
   g_mutex_lock (play_base_bin->preroll_lock);
+  GST_DEBUG ("signal preroll done");
   g_cond_signal (play_base_bin->preroll_cond);
+  GST_DEBUG ("signaled preroll done");
   g_mutex_unlock (play_base_bin->preroll_lock);
 }
 
@@ -241,6 +250,9 @@ remove_prerolls (GstPlayBaseBin * play_base_bin)
   g_list_free (play_base_bin->streaminfo);
   play_base_bin->streaminfo = NULL;
   play_base_bin->nstreams = 0;
+  play_base_bin->naudiopads = 0;
+  play_base_bin->nvideopads = 0;
+  play_base_bin->nunknownpads = 0;
 }
 
 static void
@@ -250,6 +262,7 @@ unknown_type (GstElement * element, GstCaps * caps,
   gchar *capsstr = gst_caps_to_string (caps);
 
   g_warning ("don't know how to handle %s", capsstr);
+
   g_free (capsstr);
 }
 
@@ -258,7 +271,9 @@ no_more_pads (GstElement * element, GstPlayBaseBin * play_base_bin)
 {
   GST_DEBUG ("no more pads");
   g_mutex_lock (play_base_bin->preroll_lock);
+  GST_DEBUG ("signal preroll done");
   g_cond_signal (play_base_bin->preroll_cond);
+  GST_DEBUG ("signaled preroll done");
   g_mutex_unlock (play_base_bin->preroll_lock);
 }
 
@@ -273,8 +288,9 @@ new_decoded_pad (GstElement * element, GstPad * pad, gboolean last,
   GstStreamInfo *info;
   GstStreamType type;
   GstPad *srcpad;
+  gboolean need_preroll;
 
-  GST_DEBUG ("play base: new decoded pad");
+  GST_DEBUG ("play base: new decoded pad %d", last);
 
   caps = gst_pad_get_caps (pad);
 
@@ -288,18 +304,33 @@ new_decoded_pad (GstElement * element, GstPad * pad, gboolean last,
 
   play_base_bin->nstreams++;
 
+  need_preroll = FALSE;
+
   if (g_str_has_prefix (mimetype, "audio/")) {
     type = GST_STREAM_TYPE_AUDIO;
+    play_base_bin->naudiopads++;
+    /* first audio pad gets a preroll element */
+    if (play_base_bin->naudiopads == 1) {
+      need_preroll = TRUE;
+    }
   } else if (g_str_has_prefix (mimetype, "video/")) {
     type = GST_STREAM_TYPE_VIDEO;
+    play_base_bin->nvideopads++;
+    /* first video pad gets a preroll element */
+    if (play_base_bin->nvideopads == 1) {
+      need_preroll = TRUE;
+    }
   } else {
     type = GST_STREAM_TYPE_UNKNOWN;
+    play_base_bin->nunknownpads++;
   }
 
-  if (last) {
+  if (last || !need_preroll) {
+    GST_DEBUG ("play base: pad does not need preroll");
     srcpad = pad;
-    no_more_pads (NULL, play_base_bin);
   } else {
+    GST_DEBUG ("play base: pad needs preroll");
+
     new_element = gen_preroll_element (play_base_bin, pad);
     srcpad = gst_element_get_pad (new_element, "src");
     gst_bin_add (GST_BIN (play_base_bin->thread), new_element);
@@ -308,13 +339,37 @@ new_decoded_pad (GstElement * element, GstPad * pad, gboolean last,
     play_base_bin->preroll_elems =
         g_list_prepend (play_base_bin->preroll_elems, new_element);
 
+    gst_element_set_state (new_element, GST_STATE_READY);
+
     gst_pad_link (pad, gst_element_get_pad (new_element, "sink"));
+
     gst_element_set_state (new_element, GST_STATE_PAUSED);
   }
 
+  /* add the stream to the list */
   info = gst_stream_info_new (srcpad, type, NULL);
-
   play_base_bin->streaminfo = g_list_append (play_base_bin->streaminfo, info);
+
+  /* signal the no more pads after adding the stream */
+  if (last)
+    no_more_pads (NULL, play_base_bin);
+}
+
+static void
+state_change (GstElement * element,
+    GstElementState old_state, GstElementState new_state, gpointer data)
+{
+  GstPlayBaseBin *play_base_bin = GST_PLAY_BASE_BIN (data);
+
+  if (old_state > new_state) {
+    /* EOS or error occurred */
+    GST_DEBUG ("state changed downwards");
+    g_mutex_lock (play_base_bin->preroll_lock);
+    GST_DEBUG ("signal preroll done");
+    g_cond_signal (play_base_bin->preroll_cond);
+    GST_DEBUG ("signaled preroll done");
+    g_mutex_unlock (play_base_bin->preroll_lock);
+  }
 }
 
 static gboolean
@@ -343,7 +398,7 @@ setup_source (GstPlayBaseBin * play_base_bin)
 
   {
     gboolean res;
-    gint sig1, sig2, sig3;
+    gint sig1, sig2, sig3, sig4;
     GstElement *old_dec;
 
     old_dec = play_base_bin->decoder;
@@ -382,10 +437,21 @@ setup_source (GstPlayBaseBin * play_base_bin)
      * dynamic streams, the cond is unlocked. We can remove the signal handlers then
      */
     g_mutex_lock (play_base_bin->preroll_lock);
-    gst_element_set_state (play_base_bin->thread, GST_STATE_PLAYING);
-    g_cond_wait (play_base_bin->preroll_cond, play_base_bin->preroll_lock);
+    if (gst_element_set_state (play_base_bin->thread, GST_STATE_PLAYING) ==
+        GST_STATE_SUCCESS) {
+      GST_DEBUG ("waiting for preroll...");
+      sig4 = g_signal_connect (G_OBJECT (play_base_bin->thread),
+          "state-change", G_CALLBACK (state_change), play_base_bin);
+      g_cond_wait (play_base_bin->preroll_cond, play_base_bin->preroll_lock);
+      GST_DEBUG ("preroll done !");
+    } else {
+      GST_DEBUG ("state change failed, media cannot be loaded");
+      sig4 = 0;
+    }
     g_mutex_unlock (play_base_bin->preroll_lock);
 
+    if (sig4 != 0)
+      g_signal_handler_disconnect (G_OBJECT (play_base_bin->thread), sig4);
     g_signal_handler_disconnect (G_OBJECT (play_base_bin->decoder), sig3);
     g_signal_handler_disconnect (G_OBJECT (play_base_bin->decoder), sig2);
     g_signal_handler_disconnect (G_OBJECT (play_base_bin->decoder), sig1);
@@ -415,10 +481,14 @@ gst_play_base_bin_set_property (GObject * object, guint prop_id,
         g_warning ("cannot set NULL uri");
         return;
       }
-      if (!play_base_bin->uri || !strcmp (play_base_bin->uri, uri)) {
+      /* if we have no previous uri, or the new uri is different from the
+       * old one, replug */
+      if (play_base_bin->uri == NULL || strcmp (play_base_bin->uri, uri) != 0) {
         g_free (play_base_bin->uri);
         play_base_bin->uri = g_strdup (uri);
 
+        GST_DEBUG ("setting new uri to %s", uri);
+
         play_base_bin->need_rebuild = TRUE;
       }
       break;
@@ -493,6 +563,8 @@ gst_play_base_bin_change_state (GstElement * element)
 
         g_signal_connect (G_OBJECT (play_base_bin->thread), "eos",
             G_CALLBACK (play_base_eos), play_base_bin);
+        g_signal_connect (play_base_bin->thread, "found_tag",
+            G_CALLBACK (gst_play_base_bin_found_tag), play_base_bin);
       } else {
         g_warning ("could not get 'opt' scheduler");
         gst_object_unref (GST_OBJECT (play_base_bin->thread));
@@ -505,12 +577,27 @@ gst_play_base_bin_change_state (GstElement * element)
     case GST_STATE_READY_TO_PAUSED:
     {
       if (!setup_source (play_base_bin)) {
-        GST_ELEMENT_ERROR (GST_ELEMENT (play_base_bin), LIBRARY, TOO_LAZY,
-            (NULL), ("cannot handle uri \"%s\"", play_base_bin->uri));
+        GST_ELEMENT_ERROR (GST_ELEMENT (play_base_bin), STREAM,
+            CODEC_NOT_FOUND,
+            ("cannot open file \"%s\"", play_base_bin->uri), (NULL));
+        ret = GST_STATE_FAILURE;
+      } else if (!play_base_bin->streaminfo) {
+        GST_ELEMENT_ERROR (GST_ELEMENT (play_base_bin), STREAM,
+            CODEC_NOT_FOUND,
+            ("Failed to find any supported stream in file \"%s\"",
+                play_base_bin->uri), (NULL));
         ret = GST_STATE_FAILURE;
       } else {
         ret = gst_element_set_state (play_base_bin->thread, GST_STATE_PAUSED);
       }
+      if (ret == GST_STATE_SUCCESS) {
+        /* error forwarding:
+         * we only connect after the stream has been set up. If that failed,
+         * we simply emit our own error. This also prevents us from failing
+         * because one stream was unrecognized. */
+        g_signal_connect (play_base_bin->thread, "error",
+            G_CALLBACK (gst_play_base_bin_error), play_base_bin);
+      }
       break;
     }
     case GST_STATE_PAUSED_TO_PLAYING:
@@ -520,6 +607,8 @@ gst_play_base_bin_change_state (GstElement * element)
       ret = gst_element_set_state (play_base_bin->thread, GST_STATE_PAUSED);
       break;
     case GST_STATE_PAUSED_TO_READY:
+      g_signal_handlers_disconnect_by_func (play_base_bin->thread,
+          G_CALLBACK (gst_play_base_bin_error), play_base_bin);
       ret = gst_element_set_state (play_base_bin->thread, GST_STATE_READY);
       play_base_bin->need_rebuild = TRUE;
       break;
@@ -561,11 +650,15 @@ gst_play_base_bin_add_element (GstBin * bin, GstElement * element)
     }
     gst_bin_add (GST_BIN (play_base_bin->thread), element);
 
+    /* hack */
     sched = gst_element_get_scheduler (GST_ELEMENT (play_base_bin->thread));
     clock = gst_scheduler_get_clock (sched);
     gst_scheduler_set_clock (sched, clock);
 
-    //gst_element_sync_state_with_parent (element);
+    /* FIXME set element to READY so that negotiation can happen. This
+     * currently fails because of weird negotiation problems. */
+    /* gst_element_set_state (element, GST_STATE_READY); */
+
   } else {
     g_warning ("adding elements is not allowed in NULL");
   }
@@ -600,6 +693,52 @@ gst_play_base_bin_remove_element (GstBin * bin, GstElement * element)
   }
 }
 
+static void
+gst_play_base_bin_error (GstElement * element,
+    GstElement * _source, GError * error, gchar * debug, gpointer data)
+{
+  GstObject *source, *parent;
+
+  source = GST_OBJECT (_source);
+  parent = GST_OBJECT (data);
+
+  /* tell ourselves */
+  gst_object_ref (source);
+  gst_object_ref (parent);
+  GST_DEBUG ("forwarding error \"%s\" from %s to %s", error->message,
+      GST_ELEMENT_NAME (source), GST_OBJECT_NAME (parent));
+
+  g_signal_emit_by_name (G_OBJECT (parent), "error", source, error, debug);
+
+  GST_DEBUG ("forwarded error \"%s\" from %s to %s", error->message,
+      GST_ELEMENT_NAME (source), GST_OBJECT_NAME (parent));
+  gst_object_unref (source);
+  gst_object_unref (parent);
+}
+
+static void
+gst_play_base_bin_found_tag (GstElement * element,
+    GstElement * _source, const GstTagList * taglist, gpointer data)
+{
+  GstObject *source, *parent;
+
+  source = GST_OBJECT (_source);
+  parent = GST_OBJECT (data);
+
+  /* tell ourselves */
+  gst_object_ref (source);
+  gst_object_ref (parent);
+  GST_DEBUG ("forwarding taglist %p from %s to %s", taglist,
+      GST_ELEMENT_NAME (source), GST_OBJECT_NAME (parent));
+
+  g_signal_emit_by_name (G_OBJECT (parent), "found-tag", source, taglist);
+
+  GST_DEBUG ("forwarded taglist %p from %s to %s", taglist,
+      GST_ELEMENT_NAME (source), GST_OBJECT_NAME (parent));
+  gst_object_unref (source);
+  gst_object_unref (parent);
+}
+
 void
 gst_play_base_bin_mute_stream (GstPlayBaseBin * play_base_bin,
     GstStreamInfo * info, gboolean mute)
