@@ -109,6 +109,7 @@ gst_play_on_demand_get_type (void)
       0,
       (GInstanceInitFunc) play_on_demand_init,
     };
+
     play_on_demand_type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstPlayOnDemand", &play_on_demand_info, 0);
   }
@@ -136,7 +137,7 @@ enum
   PROP_MAX_PLAYS,
   PROP_TICK_RATE,
   PROP_TOTAL_TICKS,
-  PROP_TICKS,
+  PROP_TICKS
 };
 
 static guint gst_pod_filter_signals[LAST_SIGNAL] = { 0 };
