@@ -46,7 +46,7 @@ static GstElementDetails gst_silence_details = {
   "silence source",
   "Source/Audio",
   "Generates silence",
-  "Zaheer Merali <zaheer@grid9.net>"
+  "Zaheer Abbas Merali <zaheerabbas at merali dot org>"
 };
 
 static GstStaticPadTemplate gst_silence_src_template =
@@ -174,7 +174,7 @@ gst_silence_init (GstSilence * silence)
   silence->channels = 2;
   silence->frequency = 44100;
   silence->bytes_per_read = 4096;
-
+  silence->width = 2;           /* 2 bytes */
   silence->timestamp = 0;
   silence->offset = 0;
   silence->samples = 0;
@@ -198,8 +198,11 @@ gst_silence_link (GstPad * pad, const GstCaps * caps)
 
   if (!strcmp (gst_structure_get_name (structure), "audio/x-raw-int")) {
     silence->law = 0;
+    gst_structure_get_int (structure, "width", &silence->width);
+    silence->width /= 8;
   } else {
     silence->law = 1;
+    silence->width = 1;
   }
 
   return GST_PAD_LINK_OK;
@@ -257,6 +260,7 @@ gst_silence_get (GstPad * pad)
   GstSilence *src;
   GstBuffer *buf;
   guint tdiff, samples;
+  gint i;
 
   g_return_val_if_fail (pad != NULL, NULL);
   src = GST_SILENCE (gst_pad_get_parent (pad));
@@ -269,7 +273,7 @@ gst_silence_get (GstPad * pad)
     }
   }
 
-  samples = src->bytes_per_read / (src->channels * (src->law ? 1 : 2));
+  samples = src->bytes_per_read / (src->channels * src->width);
   tdiff = samples * GST_SECOND / src->frequency;
 
   buf = gst_buffer_new_and_alloc (src->bytes_per_read);
@@ -281,7 +285,16 @@ gst_silence_get (GstPad * pad)
   GST_BUFFER_DURATION (buf) = tdiff;
   GST_BUFFER_SIZE (buf) = src->bytes_per_read;
 
-  memset (GST_BUFFER_DATA (buf), src->law ? 0 : 128, src->bytes_per_read);
+  if (src->law || src->width == 1)
+    memset (GST_BUFFER_DATA (buf), src->law ? 0 : 128, src->bytes_per_read);
+  else {
+    /* ok src->width = 2 */
+    gint16 *tmp = (gint16 *) GST_BUFFER_DATA (buf);
+
+    for (i = 0; i < src->bytes_per_read / 2; i++) {
+      tmp[i] = (gint16) 0;
+    }
+  }
 
   src->timestamp += tdiff;
   src->samples += samples;
