@@ -121,9 +121,9 @@ speed_parse_caps (GstSpeed * filter, const GstCaps * caps)
   ret &= gst_structure_get_int (structure, "channels", &filter->channels);
   ret &= gst_structure_get_int (structure, "width", &filter->width);
   ret &= gst_structure_get_int (structure, "endianness", &filter->endianness);
-  ret &=
-      gst_structure_get_int (structure, "buffer-frames",
-      &filter->buffer_frames);
+
+  filter->buffer_frames = 0;
+  gst_structure_get_int (structure, "buffer-frames", &filter->buffer_frames);
 
   mimetype = gst_structure_get_name (structure);
 
@@ -157,6 +157,7 @@ gst_speed_get_type (void)
       0,
       (GInstanceInitFunc) speed_init,
     };
+
     speed_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstSpeed", &speed_info, 0);
   }
