@@ -301,18 +301,16 @@ static void
 gst_videomixer_pad_link (GstPad * pad, GstPad * peer, gpointer data)
 {
   //GstVideoMixer *videomixer = GST_VIDEO_MIXER (data);
-  const gchar *padname = gst_pad_get_name (pad);
 
-  GST_DEBUG ("pad '%s' connected", padname);
+  GST_DEBUG ("pad '%s' connected", gst_pad_get_name (pad));
 }
 
 static void
 gst_videomixer_pad_unlink (GstPad * pad, GstPad * peer, gpointer data)
 {
   //GstVideoMixer *videomixer = GST_VIDEO_MIXER (data);
-  const gchar *padname = gst_pad_get_name (pad);
 
-  GST_DEBUG ("pad '%s' unlinked", padname);
+  GST_DEBUG ("pad '%s' unlinked", gst_pad_get_name (pad));
 }
 
 static void
@@ -993,6 +991,10 @@ gst_videomixer_blend_buffers (GstVideoMixer * mix, GstBuffer * outbuf)
           pad->in_width, pad->in_height,
           pad->alpha,
           GST_BUFFER_DATA (outbuf), mix->out_width, mix->out_height);
+      if (pad == mix->master) {
+        GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (pad->buffer);
+        GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (pad->buffer);
+      }
     }
   }
 }
