@@ -25,7 +25,7 @@
 #include <gst/gst.h>
 #include <stdlib.h>
 #include <math.h>
-#include <videoscale.h>
+#include "videoscale.h"
 #include <string.h>
 
 #include "gstvideoscale.h"
@@ -34,9 +34,11 @@
 #include "videoscale_x86.h"
 #endif
 
+#define ROUND_UP_2(x) (((x) + 1) & ~1)
+#define ROUND_UP_4(x) (((x) + 3) & ~3)
+#define ROUND_UP_8(x) (((x) + 7) & ~7)
+
 /* scalers */
-static void gst_videoscale_scale_nearest (GstVideoscale * scale,
-    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh);
 #if 0
 static void gst_videoscale_scale_plane_slow (GstVideoscale * scale,
     unsigned char *src, unsigned char *dest, int sw, int sh, int dw, int dh);
@@ -49,6 +51,9 @@ static unsigned char gst_videoscale_bilinear (unsigned char *src, double x,
 static unsigned char gst_videoscale_bicubic (unsigned char *src, double x,
     double y, int sw, int sh);
 #endif
+static int
+gst_videoscale_get_size (struct videoscale_format_struct *format, int width,
+    int height);
 
 static void gst_videoscale_planar411 (GstVideoscale * scale,
     unsigned char *dest, unsigned char *src);
@@ -65,15 +70,20 @@ static void gst_videoscale_24bit (GstVideoscale * scale, unsigned char *dest,
 static void gst_videoscale_16bit (GstVideoscale * scale, unsigned char *dest,
     unsigned char *src);
 
+static void gst_videoscale_scale_nearest_str1 (GstVideoscale * scale,
+    unsigned char *dest, unsigned char *src, int sw, int sh, int ss, int dw,
+    int dh, int ds);
 static void gst_videoscale_scale_nearest_str2 (GstVideoscale * scale,
-    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh);
+    unsigned char *dest, unsigned char *src, int sw, int sh, int ss, int dw,
+    int dh, int ds);
 static void gst_videoscale_scale_nearest_str4 (GstVideoscale * scale,
-    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh);
-static void gst_videoscale_scale_nearest_32bit (GstVideoscale * scale,
+    unsigned char *dest, unsigned char *src, int sw, int sh, int ss, int dw,
+    int dh, int ds);
+static void gst_videoscale_scale_nearest_16bit (GstVideoscale * scale,
     unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh);
 static void gst_videoscale_scale_nearest_24bit (GstVideoscale * scale,
     unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh);
-static void gst_videoscale_scale_nearest_16bit (GstVideoscale * scale,
+static void gst_videoscale_scale_nearest_32bit (GstVideoscale * scale,
     unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh);
 
 #define fourcc_YUY2 GST_MAKE_FOURCC('Y','U','Y','2')
@@ -141,8 +151,8 @@ videoscale_get_structure (struct videoscale_format_struct *format)
   }
 
   gst_structure_set (structure,
-      "width", GST_TYPE_INT_RANGE, 100, G_MAXINT,
-      "height", GST_TYPE_INT_RANGE, 100, G_MAXINT,
+      "width", GST_TYPE_INT_RANGE, 16, G_MAXINT,
+      "height", GST_TYPE_INT_RANGE, 16, G_MAXINT,
       "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
 
   return structure;
@@ -207,11 +217,11 @@ gst_videoscale_setup (GstVideoscale * videoscale)
   g_return_if_fail (videoscale->format != NULL);
 
   GST_DEBUG_OBJECT (videoscale, "format=%p " GST_FOURCC_FORMAT
-      " from %dx%d to %dx%d",
+      " from %dx%d to %dx%d, %d bpp",
       videoscale->format,
       GST_FOURCC_ARGS (videoscale->format->fourcc),
       videoscale->from_width, videoscale->from_height,
-      videoscale->to_width, videoscale->to_height);
+      videoscale->to_width, videoscale->to_height, videoscale->format->bpp);
 
   if (videoscale->to_width == 0 || videoscale->to_height == 0 ||
       videoscale->from_width == 0 || videoscale->from_height == 0) {
@@ -231,10 +241,10 @@ gst_videoscale_setup (GstVideoscale * videoscale)
 
   GST_DEBUG_OBJECT (videoscale, "scaling method POINT_SAMPLE");
 
-  videoscale->from_buf_size = (videoscale->from_width * videoscale->from_height
-      * videoscale->format->bpp) / 8;
-  videoscale->to_buf_size = (videoscale->to_width * videoscale->to_height
-      * videoscale->format->bpp) / 8;
+  videoscale->from_buf_size = gst_videoscale_get_size (videoscale->format,
+      videoscale->from_width, videoscale->from_height);
+  videoscale->to_buf_size = gst_videoscale_get_size (videoscale->format,
+      videoscale->to_width, videoscale->to_height);
 
   videoscale->passthru = FALSE;
   videoscale->inited = TRUE;
@@ -250,7 +260,7 @@ gst_videoscale_scale_rgb (GstVideoscale * scale, unsigned char *dest,
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG_OBJECT (scale, "scaling RGB %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_LOG_OBJECT (scale, "scaling RGB %dx%d to %dx%d", sw, sh, dw, dh);
 
   switch (scale->scale_bytes) {
     case 2:
@@ -265,10 +275,56 @@ gst_videoscale_scale_rgb (GstVideoscale * scale, unsigned char *dest,
       break;
   }
 
-  GST_DEBUG_OBJECT (scale, "%p %p", src, dest);
+  GST_LOG_OBJECT (scale, "%p %p", src, dest);
   //scale->scaler(scale, src, dest, sw, sh, dw, dh);
 }
 #endif
+/* calculate the total size needed for an image in the given format
+ * of the given width and height, taking stride into account */
+static int
+gst_videoscale_get_size (struct videoscale_format_struct *format, int width,
+    int height)
+{
+  int stride = 0;
+  int ustride = 0, vstride = 0;
+  int size = 0;
+
+  /* FIXME: we should get from and to strides from caps.  For now we conform
+   * to videotestsrc's idea of it, which is to round w * bytespp to nearest
+   * multiple of 4 */
+
+  switch (format->fourcc) {
+    case fourcc_RGB_:
+      stride = ROUND_UP_4 (width * ROUND_UP_4 (format->bpp)) / 8;
+      size = stride * height;
+      break;
+    case fourcc_YUY2:
+    case fourcc_UYVY:
+    case fourcc_YVYU:
+      stride = ROUND_UP_2 (width) * 2;
+      size = stride * height;
+      break;
+    case fourcc_I420:
+    case fourcc_YV12:
+      stride = ROUND_UP_4 (width);
+      ustride = ROUND_UP_8 (width) / 2;
+      vstride = ROUND_UP_8 (stride) / 2;
+      size = stride * ROUND_UP_2 (height) + ustride * ROUND_UP_2 (height) / 2 +
+          vstride * ROUND_UP_2 (height) / 2;
+      break;
+    case fourcc_Y422:
+    case fourcc_UYNV:
+    case fourcc_Y800:
+      g_warning ("unhandled known YUV fourcc");
+      break;
+    default:
+      g_warning ("unhandled unknown fourcc");
+      break;
+  }
+  GST_LOG ("image size for %dx%d in " GST_FOURCC_FORMAT " is %d bytes",
+      width, height, GST_FOURCC_ARGS (format->fourcc), size);
+  return size;
+}
 
 static void
 gst_videoscale_planar411 (GstVideoscale * scale, unsigned char *dest,
@@ -278,26 +334,37 @@ gst_videoscale_planar411 (GstVideoscale * scale, unsigned char *dest,
   int sh = scale->from_height;
   int dw = scale->to_width;
   int dh = scale->to_height;
+  int ss, ds;                   /* strides */
 
-  GST_DEBUG_OBJECT (scale, "scaling planar 4:1:1 %dx%d to %dx%d", sw, sh, dw,
-      dh);
+  GST_LOG_OBJECT (scale, "scaling planar 4:1:1 %dx%d to %dx%d", sw, sh, dw, dh);
 
-  gst_videoscale_scale_nearest (scale, dest, src, sw, sh, dw, dh);
+  /* Y */
 
-  src += sw * sh;
-  dest += dw * dh;
+  ss = ROUND_UP_4 (sw);
+  ds = ROUND_UP_4 (dw);
 
-  dh = dh >> 1;
-  dw = dw >> 1;
-  sh = sh >> 1;
-  sw = sw >> 1;
+  gst_videoscale_scale_nearest_str1 (scale, dest, src, sw, sh, ss, dw, dh, ds);
 
-  gst_videoscale_scale_nearest (scale, dest, src, sw, sh, dw, dh);
+  src += ss * sh;
+  dest += ds * dh;
 
-  src += sw * sh;
-  dest += dw * dh;
+  /* Cb and Cr are 2x2, so divide widths and heights in half */
 
-  gst_videoscale_scale_nearest (scale, dest, src, sw, sh, dw, dh);
+  /* we want to round up, so we add 1 before shifting */
+  dh = (dh + 1) >> 1;
+  dw = (dw + 1) >> 1;
+  sh = (sh + 1) >> 1;
+  sw = (sw + 1) >> 1;
+
+  ss = ROUND_UP_4 (sw);
+  ds = ROUND_UP_4 (dw);
+
+  gst_videoscale_scale_nearest_str1 (scale, dest, src, sw, sh, ss, dw, dh, ds);
+
+  src += ss * sh;
+  dest += ds * dh;
+
+  gst_videoscale_scale_nearest_str1 (scale, dest, src, sw, sh, ss, dw, dh, ds);
 }
 
 static void
@@ -308,10 +375,13 @@ gst_videoscale_planar400 (GstVideoscale * scale, unsigned char *dest,
   int sh = scale->from_height;
   int dw = scale->to_width;
   int dh = scale->to_height;
+  int ss, ds;                   /* strides */
 
-  GST_DEBUG_OBJECT (scale, "scaling Y-only %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_LOG_OBJECT (scale, "scaling Y-only %dx%d to %dx%d", sw, sh, dw, dh);
 
-  gst_videoscale_scale_nearest (scale, dest, src, sw, sh, dw, dh);
+  ss = ROUND_UP_4 (sw);
+  ds = ROUND_UP_4 (dw);
+  gst_videoscale_scale_nearest_str1 (scale, dest, src, sw, sh, ss, dw, dh, ds);
 }
 
 static void
@@ -322,15 +392,20 @@ gst_videoscale_packed422 (GstVideoscale * scale, unsigned char *dest,
   int sh = scale->from_height;
   int dw = scale->to_width;
   int dh = scale->to_height;
+  int ss, ds;                   /* strides */
 
-  GST_DEBUG_OBJECT (scale, "scaling 4:2:2 %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_LOG_OBJECT (scale, "scaling 4:2:2 %dx%d to %dx%d", sw, sh, dw, dh);
 
-  gst_videoscale_scale_nearest_str2 (scale, dest, src, sw, sh, dw, dh);
-  gst_videoscale_scale_nearest_str4 (scale, dest + 1, src + 1, sw / 2, sh,
-      dw / 2, dh);
-  gst_videoscale_scale_nearest_str4 (scale, dest + 3, src + 3, sw / 2, sh,
-      dw / 2, dh);
+  /* Y */
+  ss = ROUND_UP_4 (sw * 2);
+  ds = ROUND_UP_4 (dw * 2);
+  gst_videoscale_scale_nearest_str2 (scale, dest, src, sw, sh, ss, dw, dh, ds);
 
+  /* Cb and Cr */
+  gst_videoscale_scale_nearest_str4 (scale, dest + 1, src + 1, sw / 2, sh, ss,
+      dw / 2, dh, ds);
+  gst_videoscale_scale_nearest_str4 (scale, dest + 3, src + 3, sw / 2, sh, ss,
+      dw / 2, dh, ds);
 }
 
 static void
@@ -341,15 +416,21 @@ gst_videoscale_packed422rev (GstVideoscale * scale, unsigned char *dest,
   int sh = scale->from_height;
   int dw = scale->to_width;
   int dh = scale->to_height;
+  int ss, ds;                   /* strides */
 
-  GST_DEBUG_OBJECT (scale, "scaling 4:2:2 %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_LOG_OBJECT (scale, "scaling 4:2:2 %dx%d to %dx%d", sw, sh, dw, dh);
 
-  gst_videoscale_scale_nearest_str2 (scale, dest + 1, src, sw, sh, dw, dh);
-  gst_videoscale_scale_nearest_str4 (scale, dest, src + 1, sw / 2, sh, dw / 2,
-      dh);
-  gst_videoscale_scale_nearest_str4 (scale, dest + 2, src + 3, sw / 2, sh,
-      dw / 2, dh);
+  /* Y */
+  ss = ROUND_UP_4 (sw * 2);
+  ds = ROUND_UP_4 (dw * 2);
+  gst_videoscale_scale_nearest_str2 (scale, dest + 1, src + 1, sw, sh, ss, dw,
+      dh, ds);
 
+  /* Cb and Cr */
+  gst_videoscale_scale_nearest_str4 (scale, dest, src, sw / 2, sh, ss,
+      dw / 2, dh, ds);
+  gst_videoscale_scale_nearest_str4 (scale, dest + 2, src + 2, sw / 2, sh, ss,
+      dw / 2, dh, ds);
 }
 
 static void
@@ -361,7 +442,7 @@ gst_videoscale_32bit (GstVideoscale * scale, unsigned char *dest,
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG_OBJECT (scale, "scaling 32bit %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_LOG_OBJECT (scale, "scaling 32bit %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest_32bit (scale, dest, src, sw, sh, dw, dh);
 
@@ -376,7 +457,7 @@ gst_videoscale_24bit (GstVideoscale * scale, unsigned char *dest,
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG_OBJECT (scale, "scaling 24bit %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_LOG_OBJECT (scale, "scaling 24bit %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest_24bit (scale, dest, src, sw, sh, dw, dh);
 
@@ -391,7 +472,7 @@ gst_videoscale_16bit (GstVideoscale * scale, unsigned char *dest,
   int dw = scale->to_width;
   int dh = scale->to_height;
 
-  GST_DEBUG_OBJECT (scale, "scaling 16bit %dx%d to %dx%d", sw, sh, dw, dh);
+  GST_LOG_OBJECT (scale, "scaling 16bit %dx%d to %dx%d", sw, sh, dw, dh);
 
   gst_videoscale_scale_nearest_16bit (scale, dest, src, sw, sh, dw, dh);
 
@@ -410,7 +491,7 @@ gst_videoscale_bilinear (unsigned char *src, double x, double y, int sw, int sh)
   double dest;
   int color;
 
-  GST_DEBUG_OBJECT (scale, "scaling bilinear %f %f %dx%d", x, y, sw, sh);
+  GST_LOG_OBJECT (scale, "scaling bilinear %f %f %dx%d", x, y, sw, sh);
 
   dest = (1 - a) * (1 - b) * RC (j, k) + a * (1 - b) * RC (j + 1, k);
 
@@ -439,7 +520,7 @@ gst_videoscale_bicubic (unsigned char *src, double x, double y, int sw, int sh)
   double t1, t2, t3, t4;
   double a1, a2, a3, a4;
 
-  GST_DEBUG_OBJECT (scale, "scaling bicubic %dx%d", sw, sh);
+  GST_LOG_OBJECT (scale, "scaling bicubic %dx%d", sw, sh);
 
   a1 = -a * (1 - a) * (1 - a);
   a2 = (1 - 2 * a * a + a * a * a);
@@ -480,7 +561,7 @@ gst_videoscale_scale_plane_slow (GstVideoscale * scale, unsigned char *src,
   double xr, yr;
   int x, y;
 
-  GST_DEBUG_OBJECT (scale, "scale plane slow %dx%d %dx%d %g %g %p %p", sw, sh,
+  GST_LOG_OBJECT (scale, "scale plane slow %dx%d %dx%d %g %g %p %p", sw, sh,
       dw, dh, zoomx, zoomy, src, dest);
 
   for (y = 0; y < dh; y++) {
@@ -488,11 +569,11 @@ gst_videoscale_scale_plane_slow (GstVideoscale * scale, unsigned char *src,
     for (x = 0; x < dw; x++) {
       xr = ((double) x) / zoomx;
 
-      GST_DEBUG_OBJECT (scale, "scale plane slow %g %g %p", xr, yr,
+      GST_LOG_OBJECT (scale, "scale plane slow %g %g %p", xr, yr,
           (src + (int) (x) + (int) ((y) * sw)));
 
       if (floor (xr) == xr && floor (yr) == yr) {
-        GST_DEBUG_OBJECT (scale, "scale plane %g %g %p %p", xr, yr,
+        GST_LOG_OBJECT (scale, "scale plane %g %g %p %p", xr, yr,
             (src + (int) (x) + (int) ((y) * sw)), dest);
         *dest++ = RC (xr, yr);
       } else {
@@ -514,7 +595,7 @@ gst_videoscale_scale_point_sample (GstVideoscale * scale, unsigned char *src,
   int sum, xcount, ycount, loop;
   unsigned char *srcp, *srcp2;
 
-  GST_DEBUG_OBJECT (scale, "scaling nearest point sample %p %p %d", src, dest,
+  GST_LOG_OBJECT (scale, "scaling nearest point sample %p %p %d", src, dest,
       dw);
 
   ypos = 0x10000;
@@ -556,42 +637,42 @@ gst_videoscale_scale_point_sample (GstVideoscale * scale, unsigned char *src,
 #endif
 
 static void
-gst_videoscale_scale_nearest (GstVideoscale * scale,
-    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh)
+gst_videoscale_scale_nearest_str1 (GstVideoscale * scale,
+    unsigned char *dest, unsigned char *src, int sw, int sh, int ss,
+    int dw, int dh, int ds)
 {
   int ypos, yinc, y;
   int xpos, xinc, x;
-  guchar *destp = dest;
-  guchar *srcp = src;
+  guchar *destp;
+  guchar *srcp;
 
-  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
+  GST_LOG_OBJECT (scale, "scaling nearest from %p to %p with dest width %d",
+      src, dest, dw);
 
-
-  ypos = 0x10000;
+  ypos = 0;
   yinc = (sh << 16) / dh;
   xinc = (sw << 16) / dw;
 
   for (y = dh; y; y--) {
-
-    while (ypos > 0x10000) {
-      ypos -= 0x10000;
-      src += sw;
+    if (ypos >= 0x10000) {
+      src += (ypos >> 16) * ss;
+      ypos &= 0xffff;
     }
 
-    xpos = 0x10000;
+    xpos = 0;
 
     srcp = src;
     destp = dest;
 
     for (x = dw; x; x--) {
-      while (xpos >= 0x10000L) {
-        srcp++;
-        xpos -= 0x10000L;
+      if (xpos >= 0x10000) {
+        srcp += (xpos >> 16);
+        xpos &= 0xffff;
       }
       *destp++ = *srcp;
       xpos += xinc;
     }
-    dest += dw;
+    dest += ds;
 
     ypos += yinc;
   }
@@ -599,42 +680,43 @@ gst_videoscale_scale_nearest (GstVideoscale * scale,
 
 static void
 gst_videoscale_scale_nearest_str2 (GstVideoscale * scale,
-    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh)
+    unsigned char *dest, unsigned char *src, int sw, int sh, int ss,
+    int dw, int dh, int ds)
 {
   int ypos, yinc, y;
   int xpos, xinc, x;
-  guchar *destp = dest;
-  guchar *srcp = src;
+  guchar *destp;
+  guchar *srcp;
 
-  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
+  GST_LOG_OBJECT (scale, "scaling nearest from %p to %p with dest width %d",
+      src, dest, dw);
 
-
-  ypos = 0x10000;
+  ypos = 0;
   yinc = (sh << 16) / dh;
   xinc = (sw << 16) / dw;
 
   for (y = dh; y; y--) {
 
-    while (ypos > 0x10000) {
-      ypos -= 0x10000;
-      src += sw * 2;
+    if (ypos >= 0x10000) {
+      src += (ypos >> 16) * ss;
+      ypos &= 0xffff;
     }
 
-    xpos = 0x10000;
+    xpos = 0;
 
     srcp = src;
     destp = dest;
 
     for (x = dw; x; x--) {
-      while (xpos >= 0x10000L) {
-        srcp += 2;
-        xpos -= 0x10000L;
+      if (xpos >= 0x10000) {
+        srcp += (xpos >> 16) * 2;
+        xpos &= 0xffff;
       }
       *destp = *srcp;
       destp += 2;
       xpos += xinc;
     }
-    dest += dw * 2;
+    dest += ds;
 
     ypos += yinc;
   }
@@ -642,130 +724,43 @@ gst_videoscale_scale_nearest_str2 (GstVideoscale * scale,
 
 static void
 gst_videoscale_scale_nearest_str4 (GstVideoscale * scale,
-    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh)
+    unsigned char *dest, unsigned char *src, int sw, int sh, int ss,
+    int dw, int dh, int ds)
 {
   int ypos, yinc, y;
   int xpos, xinc, x;
-  guchar *destp = dest;
-  guchar *srcp = src;
+  guchar *destp;
+  guchar *srcp;
 
-  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
+  GST_LOG_OBJECT (scale, "scaling nearest from %p to %p with dest width %d",
+      src, dest, dw);
 
-
-  ypos = 0x10000;
+  ypos = 0;
   yinc = (sh << 16) / dh;
   xinc = (sw << 16) / dw;
 
   for (y = dh; y; y--) {
 
-    while (ypos > 0x10000) {
-      ypos -= 0x10000;
-      src += sw * 4;
+    if (ypos >= 0x10000) {
+      src += (ypos >> 16) * ss;
+      ypos &= 0xffff;
     }
 
-    xpos = 0x10000;
+    xpos = 0;
 
     srcp = src;
     destp = dest;
 
     for (x = dw; x; x--) {
-      while (xpos >= 0x10000L) {
-        srcp += 4;
-        xpos -= 0x10000L;
+      if (xpos >= 0x10000) {
+        srcp += (xpos >> 16) * 4;
+        xpos &= 0xffff;
       }
       *destp = *srcp;
       destp += 4;
       xpos += xinc;
     }
-    dest += dw * 4;
-
-    ypos += yinc;
-  }
-}
-
-static void
-gst_videoscale_scale_nearest_32bit (GstVideoscale * scale,
-    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh)
-{
-  int ypos, yinc, y;
-  int xpos, xinc, x;
-  guchar *destp = dest;
-  guchar *srcp = src;
-
-  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
-
-
-  ypos = 0x10000;
-  yinc = (sh << 16) / dh;
-  xinc = (sw << 16) / dw;
-
-  for (y = dh; y; y--) {
-
-    while (ypos > 0x10000) {
-      ypos -= 0x10000;
-      src += sw * 4;
-    }
-
-    xpos = 0x10000;
-
-    srcp = src;
-    destp = dest;
-
-    for (x = dw; x; x--) {
-      while (xpos >= 0x10000L) {
-        srcp += 4;
-        xpos -= 0x10000L;
-      }
-      *(guint32 *) destp = *(guint32 *) srcp;
-      destp += 4;
-      xpos += xinc;
-    }
-    dest += dw * 4;
-
-    ypos += yinc;
-  }
-}
-
-static void
-gst_videoscale_scale_nearest_24bit (GstVideoscale * scale,
-    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh)
-{
-  int ypos, yinc, y;
-  int xpos, xinc, x;
-  guchar *destp = dest;
-  guchar *srcp = src;
-
-  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
-
-
-  ypos = 0x10000;
-  yinc = (sh << 16) / dh;
-  xinc = (sw << 16) / dw;
-
-  for (y = dh; y; y--) {
-
-    while (ypos > 0x10000) {
-      ypos -= 0x10000;
-      src += sw * 3;
-    }
-
-    xpos = 0x10000;
-
-    srcp = src;
-    destp = dest;
-
-    for (x = dw; x; x--) {
-      while (xpos >= 0x10000L) {
-        srcp += 3;
-        xpos -= 0x10000L;
-      }
-      destp[0] = srcp[0];
-      destp[1] = srcp[1];
-      destp[2] = srcp[2];
-      destp += 3;
-      xpos += xinc;
-    }
-    dest += dw * 3;
+    dest += ds;
 
     ypos += yinc;
   }
@@ -777,39 +772,145 @@ gst_videoscale_scale_nearest_16bit (GstVideoscale * scale,
 {
   int ypos, yinc, y;
   int xpos, xinc, x;
-  guchar *destp = dest;
-  guchar *srcp = src;
+  guchar *destp;
+  guchar *srcp;
+  int ss, ds;                   /* row strides in bytes */
 
-  GST_DEBUG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
+  GST_LOG_OBJECT (scale, "scaling nearest from %p to %p, destination width %d",
+      src, dest, dw);
 
+  /* FIXME: strides should be gotten from caps; for now we do it Just Like
+     videotestsrc, which means round off to next multiple of 4 bytes */
+  ss = sw * 2;
+  if (sw % 2 == 1)
+    ss += 2;
+  ds = dw * 2;
+  if (dw % 2 == 1)
+    ds += 2;
 
-  ypos = 0x10000;
+  ypos = 0;
+  yinc = (sh << 16) / dh;       /* 16 bit fixed point arithmetic */
+  xinc = (sw << 16) / dw;
+
+  /* go over all destination lines */
+  for (y = dh; y; y--) {        /* faster than 0 .. dh */
+
+    if (ypos >= 0x10000) {      /* ypos >= 1 ? */
+      src += (ypos >> 16) * ss; /* go down round(ypos) src lines */
+      ypos &= 0xffff;           /* ypos %= 1 */
+    }
+
+    xpos = 0;
+
+    srcp = src;
+    destp = dest;
+
+    /* go over all destination pixels for each line */
+    for (x = dw; x; x--) {
+      if (xpos >= 0x10000) {    /* xpos >= 1 ? */
+        srcp += (xpos >> 16) * 2;       /* go right round(xpos) src pixels */
+        xpos &= 0xffff;         /* xpos %= 1 */
+      }
+      destp[0] = srcp[0];
+      destp[1] = srcp[1];
+      destp += 2;               /* go right one destination pixel */
+      xpos += xinc;
+    }
+    dest += ds;                 /* go down one destination line */
+
+    ypos += yinc;
+  }
+}
+
+static void
+gst_videoscale_scale_nearest_24bit (GstVideoscale * scale,
+    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh)
+{
+  int ypos, yinc, y;
+  int xpos, xinc, x;
+  guchar *destp;
+  guchar *srcp;
+  int ss, ds;                   /* row strides in bytes */
+
+  GST_LOG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
+
+  /* FIXME: strides should be gotten from caps; for now we do it Just Like
+     videotestsrc, which means round off to next multiple of 4 bytes */
+  ss = ROUND_UP_4 (sw * 3);
+  ds = ROUND_UP_4 (dw * 3);
+
+  ypos = 0;
   yinc = (sh << 16) / dh;
   xinc = (sw << 16) / dw;
 
   for (y = dh; y; y--) {
 
-    while (ypos > 0x10000) {
-      ypos -= 0x10000;
-      src += sw * 2;
+    if (ypos >= 0x10000) {
+      src += (ypos >> 16) * ss;
+      ypos &= 0xffff;
     }
 
-    xpos = 0x10000;
+    xpos = 0;
 
     srcp = src;
     destp = dest;
 
     for (x = dw; x; x--) {
-      while (xpos >= 0x10000L) {
-        srcp += 2;
-        xpos -= 0x10000L;
+      if (xpos >= 0x10000) {
+        srcp += (xpos >> 16) * 3;
+        xpos &= 0xffff;
       }
       destp[0] = srcp[0];
       destp[1] = srcp[1];
-      destp += 2;
+      destp[2] = srcp[2];
+      destp += 3;
       xpos += xinc;
     }
-    dest += dw * 2;
+    dest += ds;
+
+    ypos += yinc;
+  }
+}
+
+static void
+gst_videoscale_scale_nearest_32bit (GstVideoscale * scale,
+    unsigned char *dest, unsigned char *src, int sw, int sh, int dw, int dh)
+{
+  int ypos, yinc, y;
+  int xpos, xinc, x;
+  guchar *destp;
+  guchar *srcp;
+
+  GST_LOG_OBJECT (scale, "scaling nearest %p %p %d", src, dest, dw);
+
+  /* given how videoscale rounds off stride to nearest multiple of 4,
+   * we don't have stride issues for 32 bit */
+  ypos = 0;
+  yinc = (sh << 16) / dh;
+  xinc = (sw << 16) / dw;
+
+  for (y = dh; y; y--) {
+
+    if (ypos >= 0x10000) {
+      src += (ypos >> 16) * sw * 4;
+      ypos &= 0xffff;
+    }
+
+    xpos = 0;
+
+    srcp = src;
+    destp = dest;
+
+    for (x = dw; x; x--) {
+      if (xpos >= 0x10000) {
+        srcp += (xpos >> 16) * 4;
+        xpos &= 0xffff;
+      }
+      *(guint32 *) destp = *(guint32 *) srcp;
+      destp += 4;
+      xpos += xinc;
+    }
+    dest += dw * 4;
 
     ypos += yinc;
   }
