@@ -259,7 +259,7 @@ gst_wavenc_stop_file (GstWavEnc * wavenc)
   GstEvent *event;
   GstBuffer *outbuf;
 
-  event = gst_event_new_seek (GST_FORMAT_BYTES | GST_SEEK_METHOD_SET, 0);
+  event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, 0);
   gst_pad_push (wavenc->srcpad, GST_DATA (event));
 
   outbuf = gst_buffer_new_and_alloc (WAV_HEADER_LEN);
@@ -294,7 +294,7 @@ gst_wavenc_init (GstWavEnc * wavenc)
 
   wavenc->setup = FALSE;
   wavenc->flush_header = TRUE;
-
+  wavenc->newmediacount = 0;
   GST_FLAG_SET (wavenc, GST_ELEMENT_EVENT_AWARE);
 }
 
@@ -303,8 +303,7 @@ struct _maps
   guint32 id;
   char *name;
 }
-maps[] =
-{
+maps[] = {
   {
   GST_RIFF_INFO_IARL, "Location"}
   , {
@@ -618,10 +617,21 @@ gst_wavenc_chain (GstPad * pad, GstData * _data)
 #endif
 
       gst_wavenc_stop_file (wavenc);
-      gst_pad_push (wavenc->srcpad, GST_DATA (gst_event_new (GST_EVENT_EOS)));
+      gst_pad_push (wavenc->srcpad, _data);
       gst_element_set_eos (GST_ELEMENT (wavenc));
+    } else if (GST_EVENT_TYPE (buf) == GST_EVENT_DISCONTINUOUS) {
+      if (GST_EVENT_DISCONT_NEW_MEDIA (buf)) {
+        /* new media */
+        if (wavenc->newmediacount++ > 0) {
+          gst_wavenc_stop_file (wavenc);
+          wavenc->setup = FALSE;
+          wavenc->flush_header = TRUE;
+          gst_wavenc_setup (wavenc);
+        }
+        gst_pad_event_default (wavenc->sinkpad, GST_EVENT (buf));
+      }
     } else {
-      gst_pad_event_default (wavenc->srcpad, GST_EVENT (buf));
+      gst_pad_event_default (wavenc->sinkpad, GST_EVENT (buf));
     }
     return;
   }
