@@ -24,7 +24,16 @@
 #endif
 #include <string.h>
 
-#include <gstwavparse.h>
+#include "gstwavparse.h"
+#include "gst/riff/riff-ids.h"
+#include "gst/riff/riff-media.h"
+
+#ifndef G_MAXUINT32
+#define G_MAXUINT32 0xffffffff
+#endif
+
+GST_DEBUG_CATEGORY (wavparse_debug);
+#define GST_CAT_DEFAULT (wavparse_debug)
 
 static void gst_wavparse_base_init (gpointer g_class);
 static void gst_wavparse_class_init (GstWavParseClass * klass);
@@ -46,13 +55,6 @@ static gboolean gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event);
 static void gst_wavparse_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-/* elementfactory information */
-static GstElementDetails gst_wavparse_details =
-GST_ELEMENT_DETAILS (".wav demuxer",
-    "Codec/Demuxer",
-    "Parse a .wav file into raw audio",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
 static GstStaticPadTemplate sink_template_factory =
 GST_STATIC_PAD_TEMPLATE ("wavparse_sink",
     GST_PAD_SINK,
@@ -63,7 +65,7 @@ GST_STATIC_PAD_TEMPLATE ("wavparse_sink",
 static GstStaticPadTemplate src_template_factory =
     GST_STATIC_PAD_TEMPLATE ("wavparse_src",
     GST_PAD_SRC,
-    GST_PAD_ALWAYS,
+    GST_PAD_SOMETIMES,          /* FIXME: spider */
     GST_STATIC_CAPS ("audio/x-raw-int, "
         "endianness = (int) little_endian, "
         "signed = (boolean) { true, false }, "
@@ -80,7 +82,17 @@ static GstStaticPadTemplate src_template_factory =
         "rate = (int) [ 8000, 48000 ], "
         "channels = (int) [ 1, 2 ]; "
         "audio/x-mulaw, "
-        "rate = (int) [ 8000, 48000 ], " "channels = (int) [ 1, 2 ]")
+        "rate = (int) [ 8000, 48000 ], " "channels = (int) [ 1, 2 ];"
+        "audio/x-adpcm, "
+        "layout = (string) microsoft, "
+        "block_align = (int) [ 1, 8192 ], "
+        "rate = (int) [ 8000, 48000 ], "
+        "channels = (int) [ 1, 2 ]; "
+        "audio/x-adpcm, "
+        "layout = (string) dvi, "
+        "block_align = (int) [ 1, 8192 ], "
+        "rate = (int) [ 8000, 48000 ], " "channels = (int) [ 1, 2 ];"
+        "audio/x-vnd.sony.atrac3")
     );
 
 /* WavParse signals and args */
@@ -92,7 +104,7 @@ enum
 
 enum
 {
-  PROP_0,
+  PROP_0
 };
 
 static GstElementClass *parent_class = NULL;
@@ -118,8 +130,8 @@ gst_wavparse_get_type (void)
     };
 
     wavparse_type =
-        g_type_register_static (GST_TYPE_ELEMENT, "GstWavParse", &wavparse_info,
-        0);
+        g_type_register_static (GST_TYPE_RIFF_READ, "GstWavParse",
+        &wavparse_info, 0);
   }
   return wavparse_type;
 }
@@ -129,6 +141,11 @@ static void
 gst_wavparse_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  static GstElementDetails gst_wavparse_details =
+      GST_ELEMENT_DETAILS (".wav demuxer",
+      "Codec/Demuxer/Audio",
+      "Parse a .wav file into raw audio",
+      "Erik Walthinsen <omega@cse.ogi.edu>");
 
   gst_element_class_set_details (element_class, &gst_wavparse_details);
 
@@ -138,6 +155,7 @@ gst_wavparse_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template_factory));
 }
+
 static void
 gst_wavparse_class_init (GstWavParseClass * klass)
 {
@@ -147,10 +165,12 @@ gst_wavparse_class_init (GstWavParseClass * klass)
   gstelement_class = (GstElementClass *) klass;
   object_class = (GObjectClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_ref (GST_TYPE_RIFF_READ);
 
   object_class->get_property = gst_wavparse_get_property;
   gstelement_class->change_state = gst_wavparse_change_state;
+
+  GST_DEBUG_CATEGORY_INIT (wavparse_debug, "wavparse", 0, "WAV parser");
 }
 
 static void
@@ -161,6 +181,7 @@ gst_wavparse_init (GstWavParse * wavparse)
       gst_pad_new_from_template (gst_static_pad_template_get
       (&sink_template_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (wavparse), wavparse->sinkpad);
+  GST_RIFF_READ (wavparse)->sinkpad = wavparse->sinkpad;
 
   gst_pad_set_formats_function (wavparse->sinkpad, gst_wavparse_get_formats);
   gst_pad_set_convert_function (wavparse->sinkpad, gst_wavparse_pad_convert);
@@ -168,6 +189,7 @@ gst_wavparse_init (GstWavParse * wavparse)
       gst_wavparse_get_query_types);
   gst_pad_set_query_function (wavparse->sinkpad, gst_wavparse_pad_query);
 
+#if 0
   /* source */
   wavparse->srcpad =
       gst_pad_new_from_template (gst_static_pad_template_get
@@ -182,15 +204,51 @@ gst_wavparse_init (GstWavParse * wavparse)
   gst_pad_set_event_function (wavparse->srcpad, gst_wavparse_srcpad_event);
   gst_pad_set_event_mask_function (wavparse->srcpad,
       gst_wavparse_get_event_masks);
+#endif
 
   gst_element_set_loop_function (GST_ELEMENT (wavparse), gst_wavparse_loop);
 
-  wavparse->state = GST_WAVPARSE_UNKNOWN;
-  wavparse->bps = 0;
+  wavparse->state = GST_WAVPARSE_START;
+
+  /* These will all be set correctly in the fmt chunk */
+  wavparse->depth = 0;
+  wavparse->rate = 0;
+  wavparse->width = 0;
+  wavparse->channels = 0;
+
   wavparse->seek_pending = FALSE;
   wavparse->seek_offset = 0;
 }
 
+static void
+gst_wavparse_destroy_sourcepad (GstWavParse * wavparse)
+{
+  if (wavparse->srcpad) {
+    gst_element_remove_pad (GST_ELEMENT (wavparse), wavparse->srcpad);
+    wavparse->srcpad = NULL;
+  }
+}
+
+static void
+gst_wavparse_create_sourcepad (GstWavParse * wavparse)
+{
+  gst_wavparse_destroy_sourcepad (wavparse);
+
+  /* source */
+  wavparse->srcpad =
+      gst_pad_new_from_template (gst_static_pad_template_get
+      (&src_template_factory), "src");
+  gst_pad_use_explicit_caps (wavparse->srcpad);
+  gst_pad_set_formats_function (wavparse->srcpad, gst_wavparse_get_formats);
+  gst_pad_set_convert_function (wavparse->srcpad, gst_wavparse_pad_convert);
+  gst_pad_set_query_type_function (wavparse->srcpad,
+      gst_wavparse_get_query_types);
+  gst_pad_set_query_function (wavparse->srcpad, gst_wavparse_pad_query);
+  gst_pad_set_event_function (wavparse->srcpad, gst_wavparse_srcpad_event);
+  gst_pad_set_event_mask_function (wavparse->srcpad,
+      gst_wavparse_get_event_masks);
+}
+
 static void
 gst_wavparse_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec)
@@ -390,123 +448,6 @@ gst_wavparse_parse_adtl (GstWavParse * wavparse, int len)
 }
 #endif
 
-#if 0
-static void
-gst_wavparse_parse_info (GstWavParse * wavparse, int len)
-{
-  gst_riff_chunk *temp_chunk, chunk;
-  GstByteStream *bs = wavparse->bs;
-  guint8 *tempdata;
-  guint32 got_bytes;
-  char *name, *type;
-
-  while (len > 0) {
-    got_bytes =
-        gst_bytestream_peek_bytes (bs, &tempdata, sizeof (gst_riff_chunk));
-    temp_chunk = (gst_riff_chunk *) tempdata;
-
-    chunk.id = GUINT32_FROM_LE (temp_chunk->id);
-    chunk.size = GUINT32_FROM_LE (temp_chunk->size);
-
-    gst_bytestream_flush (bs, sizeof (gst_riff_chunk));
-    if (got_bytes != sizeof (gst_riff_chunk)) {
-      return;
-    }
-
-    /* move our pointer on past the header */
-    len -= sizeof (gst_riff_chunk);
-
-    if (chunk.size == 0) {
-      continue;
-    }
-
-    got_bytes = gst_bytestream_peek_bytes (bs, &tempdata, chunk.size);
-    name = (char *) tempdata;
-    if (got_bytes != chunk.size) {
-      return;
-    }
-
-    /* move our pointer on past the data ... on an even boundary */
-    gst_bytestream_flush (bs, (chunk.size + 1) & ~1);
-    len -= ((chunk.size + 1) & ~1);
-
-    /* We now have an info string in 'name' of type chunk.id
-       - find type */
-    switch (chunk.id) {
-      case GST_RIFF_INFO_IARL:
-        type = "Location";
-        break;
-
-      case GST_RIFF_INFO_IART:
-        type = "Artist";
-        break;
-
-      case GST_RIFF_INFO_ICMS:
-        type = "Commissioner";
-        break;
-
-      case GST_RIFF_INFO_ICMT:
-        type = "Comment";
-        break;
-
-      case GST_RIFF_INFO_ICOP:
-        type = "Copyright";
-        break;
-
-      case GST_RIFF_INFO_ICRD:
-        type = "Creation Date";
-        break;
-
-      case GST_RIFF_INFO_IENG:
-        type = "Engineer";
-        break;
-
-      case GST_RIFF_INFO_IGNR:
-        type = "Genre";
-        break;
-
-      case GST_RIFF_INFO_IKEY:
-        type = "Keywords";
-        break;
-
-      case GST_RIFF_INFO_INAM:
-        type = "Title";         /* name */
-        break;
-
-      case GST_RIFF_INFO_IPRD:
-        type = "Product";
-        break;
-
-      case GST_RIFF_INFO_ISBJ:
-        type = "Subject";
-        break;
-
-      case GST_RIFF_INFO_ISFT:
-        type = "Software";
-        break;
-
-      case GST_RIFF_INFO_ITCH:
-        type = "Technician";
-        break;
-
-      default:
-        g_print ("Unknown: %4.4s\n", (char *) &chunk.id);
-        type = NULL;
-        break;
-    }
-
-    if (type) {
-      GstPropsEntry *entry;
-
-      entry = gst_props_entry_new (type, G_TYPE_STRING (name));
-      gst_props_add_entry (wavparse->metadata->properties, entry);
-    }
-  }
-
-  g_object_notify (G_OBJECT (wavparse), "metadata");
-}
-#endif
-
 #if 0
 static void
 gst_wavparse_parse_cues (GstWavParse * wavparse, int len)
@@ -572,322 +513,288 @@ gst_wavparse_parse_cues (GstWavParse * wavparse, int len)
 }
 #endif
 
-static void
-gst_wavparse_parse_fmt (GstWavParse * wavparse, guint size)
+static gboolean
+gst_wavparse_stream_init (GstWavParse * wav)
 {
-  GstWavParseFormat *format;
-  GstCaps *caps = NULL;
-  guint8 *fmtdata;
-  GstByteStream *bs = wavparse->bs;
-  guint32 got_bytes;
+  GstRiffRead *riff = GST_RIFF_READ (wav);
+  guint32 doctype;
 
-  got_bytes =
-      gst_bytestream_peek_bytes (bs, &fmtdata, sizeof (GstWavParseFormat));
-  format = (GstWavParseFormat *) fmtdata;
-
-  if (got_bytes == sizeof (GstWavParseFormat)) {
-    gst_bytestream_flush (bs, size);
-    wavparse->bps = GUINT16_FROM_LE (format->wBlockAlign);
-    wavparse->rate = GUINT32_FROM_LE (format->dwSamplesPerSec);
-    wavparse->channels = GUINT16_FROM_LE (format->wChannels);
-    wavparse->width = GUINT16_FROM_LE (format->wBitsPerSample);
-    wavparse->format = GINT16_FROM_LE (format->wFormatTag);
-
-    /* set the caps on the src pad */
-    /* FIXME: handle all of the other formats as well */
-    switch (wavparse->format) {
-      case GST_RIFF_WAVE_FORMAT_ALAW:
-      case GST_RIFF_WAVE_FORMAT_MULAW:{
-        char *mime = (wavparse->format == GST_RIFF_WAVE_FORMAT_ALAW) ?
-            "audio/x-alaw" : "audio/x-mulaw";
-        if (wavparse->width != 8) {
-          g_warning ("Ignoring invalid width %d", wavparse->width);
-          return;
-        }
-
-        caps = gst_caps_new_simple (mime,
-            "rate", G_TYPE_INT, wavparse->rate,
-            "channels", G_TYPE_INT, wavparse->channels, NULL);
-      }
-        break;
-
-      case GST_RIFF_WAVE_FORMAT_PCM:
-        caps = gst_caps_new_simple ("audio/x-raw-int",
-            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
-            "signed", G_TYPE_BOOLEAN, (wavparse->width > 8) ? TRUE : FALSE,
-            "width", G_TYPE_INT, wavparse->width,
-            "depth", G_TYPE_INT, wavparse->width,
-            "rate", G_TYPE_INT, wavparse->rate,
-            "channels", G_TYPE_INT, wavparse->channels, NULL);
-        break;
-
-      case GST_RIFF_WAVE_FORMAT_MPEGL12:
-      case GST_RIFF_WAVE_FORMAT_MPEGL3:{
-        int layer = (wavparse->format == GST_RIFF_WAVE_FORMAT_MPEGL12) ? 2 : 3;
-
-        caps = gst_caps_new_simple ("audio/mpeg",
-            "mpegversion", G_TYPE_INT, 1,
-            "layer", G_TYPE_INT, layer,
-            "rate", G_TYPE_INT, wavparse->rate,
-            "channels", G_TYPE_INT, wavparse->channels, NULL);
-      }
-        break;
-
-      default:
-        GST_ELEMENT_ERROR (wavparse, STREAM, NOT_IMPLEMENTED, (NULL),
-            ("format %d not handled", wavparse->format));
-        return;
-    }
-
-    if (caps) {
-      gst_pad_set_explicit_caps (wavparse->srcpad, caps);
-      gst_caps_free (caps);
-    }
-
-    GST_DEBUG ("frequency %d, channels %d", wavparse->rate, wavparse->channels);
+  if (!gst_riff_read_header (riff, &doctype)) {
+    GST_WARNING_OBJECT (wav, "could not read header");
+    return FALSE;
   }
+
+  if (doctype != GST_RIFF_RIFF_WAVE) {
+    GST_ELEMENT_ERROR (wav, STREAM, WRONG_TYPE, (NULL), (NULL));
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+/* Read 'fmt ' header */
+static gboolean
+gst_wavparse_fmt (GstWavParse * wav)
+{
+  GstRiffRead *riff = GST_RIFF_READ (wav);
+  gst_riff_strf_auds *header = NULL;
+  GstCaps *caps;
+
+  if (!gst_riff_read_strf_auds (riff, &header)) {
+    g_warning ("Not fmt");
+    return FALSE;
+  }
+
+  wav->format = header->format;
+  wav->rate = header->rate;
+  wav->channels = header->channels;
+  wav->blockalign = header->blockalign;
+  wav->width = (header->blockalign * 8) / header->channels;
+  wav->depth = header->size;
+  wav->bps = header->av_bps;
+
+  caps = gst_riff_create_audio_caps (header->format, NULL, header, NULL);
+
+  if (caps) {
+    gst_wavparse_create_sourcepad (wav);
+    gst_pad_set_explicit_caps (wav->srcpad, caps);
+    gst_caps_free (caps);
+    gst_element_add_pad (GST_ELEMENT (wav), wav->srcpad);
+    GST_DEBUG ("frequency %d, channels %d", wav->rate, wav->channels);
+  } else {
+    GST_ELEMENT_ERROR (wav, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
+    return FALSE;
+  }
+
+
+
+  g_free (header);
+
+  return TRUE;
 }
 
 static gboolean
-gst_wavparse_handle_sink_event (GstWavParse * wavparse)
+gst_wavparse_other (GstWavParse * wav)
 {
-  guint32 remaining;
-  GstEvent *event;
-  GstEventType type;
-  gboolean res = TRUE;
+  GstRiffRead *riff = GST_RIFF_READ (wav);
+  guint32 tag, length;
 
-  gst_bytestream_get_status (wavparse->bs, &remaining, &event);
+  if (!gst_riff_peek_head (riff, &tag, &length, NULL)) {
+    GST_WARNING_OBJECT (wav, "could not peek head");
+    return FALSE;
+  }
+  GST_DEBUG_OBJECT (wav, "got tag (%08x) %4.4s, length %d", tag,
+      (gchar *) & tag, length);
 
-  type = event ? GST_EVENT_TYPE (event) : GST_EVENT_UNKNOWN;
-  GST_DEBUG ("wavparse: event %p %d", event, type);
+  switch (tag) {
+    case GST_RIFF_TAG_LIST:
+      if (!(tag = gst_riff_peek_list (riff))) {
+        GST_WARNING_OBJECT (wav, "could not peek list");
+        return FALSE;
+      }
 
-  switch (type) {
-    case GST_EVENT_EOS:
-      gst_bytestream_flush (wavparse->bs, remaining);
-      gst_pad_event_default (wavparse->sinkpad, event);
-      res = FALSE;
-      goto done;
+      switch (tag) {
+        case GST_RIFF_LIST_INFO:
+          if (!gst_riff_read_list (riff, &tag) || !gst_riff_read_info (riff)) {
+            GST_WARNING_OBJECT (wav, "could not read list");
+            return FALSE;
+          }
+          break;
 
-    case GST_EVENT_FLUSH:
-      g_warning ("Wavparse: Flush event");
+        case GST_RIFF_LIST_adtl:
+          if (!gst_riff_read_skip (riff)) {
+            GST_WARNING_OBJECT (wav, "could not read skip");
+            return FALSE;
+          }
+          break;
+
+        default:
+          GST_DEBUG_OBJECT (wav, "skipping tag (%08x) %4.4s", tag,
+              (gchar *) & tag);
+          if (!gst_riff_read_skip (riff)) {
+            GST_WARNING_OBJECT (wav, "could not read skip");
+            return FALSE;
+          }
+          break;
+      }
+
+      break;
+
+    case GST_RIFF_TAG_data:
+      if (!gst_bytestream_flush (riff->bs, 8)) {
+        GST_WARNING_OBJECT (wav, "could not flush 8 bytes");
+        return FALSE;
+      }
+
+      GST_DEBUG_OBJECT (wav, "switching to data mode");
+      wav->state = GST_WAVPARSE_DATA;
+      wav->datastart = gst_bytestream_tell (riff->bs);
+      if (length == 0) {
+        guint64 file_length;
+
+        /* length is 0, data probably stretches to the end
+         * of file */
+        GST_DEBUG_OBJECT (wav, "length is 0 trying to find length");
+        /* get length of file */
+        file_length = gst_bytestream_length (riff->bs);
+        if (file_length == -1) {
+          GST_DEBUG_OBJECT (wav,
+              "could not get file length, assuming data to eof");
+          /* could not get length, assuming till eof */
+          length = G_MAXUINT32;
+        }
+        if (file_length > G_MAXUINT32) {
+          GST_DEBUG_OBJECT (wav, "file length %lld, clipping to 32 bits");
+          /* could not get length, assuming till eof */
+          length = G_MAXUINT32;
+        } else {
+          GST_DEBUG_OBJECT (wav, "file length %lld, datalength", file_length,
+              length);
+          /* substract offset of datastart from length */
+          length = file_length - wav->datastart;
+          GST_DEBUG_OBJECT (wav, "datalength %lld", length);
+        }
+      }
+      wav->dataleft = wav->datasize = (guint64) length;
+      break;
+
+    case GST_RIFF_TAG_cue:
+      if (!gst_riff_read_skip (riff)) {
+        GST_WARNING_OBJECT (wav, "could not read skip");
+        return FALSE;
+      }
       break;
 
     default:
-      g_warning ("Wavparse: Unhandled event %d", type);
+      GST_DEBUG_OBJECT (wav, "skipping tag (%08x) %4.4s", tag, (gchar *) & tag);
+      if (!gst_riff_read_skip (riff))
+        return FALSE;
       break;
   }
 
-  gst_event_unref (event);
-
-done:
-  return res;
+  return TRUE;
 }
 
+static gboolean
+gst_wavparse_handle_seek (GstWavParse * wav)
+{
+  GstRiffRead *riff = GST_RIFF_READ (wav);
+  GstEvent *event = NULL;
+  guint32 remaining;
+  guint8 *data;
+
+  if (!gst_bytestream_seek (riff->bs, wav->seek_offset + wav->datastart,
+          GST_SEEK_METHOD_SET))
+    return FALSE;
+
+  /* wait for discont */
+  while (!event) {
+    if (gst_bytestream_peek_bytes (riff->bs, &data, 1)) {
+      GST_WARNING ("Unexpected data after seek - this means seek failed");
+      return FALSE;
+    }
+
+    /* get the discont event and return */
+    gst_bytestream_get_status (riff->bs, &remaining, &event);
+    if (!event) {
+      GST_WARNING ("No discontinuity event after seek - seek failed");
+      return FALSE;
+    } else if (GST_EVENT_TYPE (event) != GST_EVENT_DISCONTINUOUS) {
+      GstEventType type = GST_EVENT_TYPE (event);
+
+      gst_pad_event_default (riff->sinkpad, event);
+      if (type == GST_EVENT_EOS)
+        return FALSE;
+
+      event = NULL;
+    }
+  }
+
+  wav->dataleft = wav->datasize - wav->seek_offset;
+
+  gst_event_unref (event);
+  event = gst_event_new_discontinuous (FALSE,
+      GST_FORMAT_BYTES, wav->seek_offset,
+      GST_FORMAT_TIME, GST_SECOND * wav->seek_offset / wav->bps,
+      GST_FORMAT_UNDEFINED);
+  gst_pad_event_default (wav->sinkpad, event);
+
+  return TRUE;
+}
+
+#define MAX_BUFFER_SIZE 4096
+
 static void
 gst_wavparse_loop (GstElement * element)
 {
-  GstWavParse *wavparse;
-  gst_riff_riff chunk;
-  guint32 flush = 0;
-  guint32 got_bytes;
-  GstByteStream *bs;
+  GstWavParse *wav = GST_WAVPARSE (element);
+  GstRiffRead *riff = GST_RIFF_READ (wav);
 
-  wavparse = GST_WAVPARSE (element);
-
-  bs = wavparse->bs;
-
-  if (wavparse->seek_pending) {
-    GST_DEBUG ("wavparse: seek pending to %" G_GINT64_FORMAT " %08llx",
-        wavparse->seek_offset, (unsigned long long) wavparse->seek_offset);
-
-    if (!gst_bytestream_seek (bs, wavparse->seek_offset, GST_SEEK_METHOD_SET)) {
-      GST_INFO ("wavparse: Could not seek");
+  if (wav->state == GST_WAVPARSE_DATA) {
+    /* seek handling */
+    if (wav->seek_pending) {
+      gst_wavparse_handle_seek (wav);
+      wav->seek_pending = FALSE;
     }
 
-    wavparse->seek_pending = FALSE;
-  }
+    if (wav->dataleft > 0) {
+      guint32 got_bytes, desired;
+      GstBuffer *buf = NULL;
 
-  if (wavparse->state == GST_WAVPARSE_DATA) {
-    GstBuffer *buf;
-    int desired;
-
-    /* This seems to want the whole chunk,
-       Will this screw up streaming?
-       Does anyone care about streaming wavs?
-       FIXME: Should we have a decent buffer size? */
-
-#define MAX_BUFFER_SIZE 1024
-
-    if (wavparse->dataleft > 0) {
-      desired = MIN (wavparse->dataleft, MAX_BUFFER_SIZE);
-      got_bytes = gst_bytestream_peek (bs, &buf, desired);
-
-      if (got_bytes != desired) {
-        /* EOS? */
-        GstEvent *event;
-        guint32 remaining;
-
-        gst_bytestream_get_status (bs, &remaining, &event);
-        if (event && GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
-          gst_pad_event_default (wavparse->sinkpad, event);
-        } else {
-          GST_ELEMENT_ERROR (element, RESOURCE, READ, (NULL), (NULL));
-        }
+      desired = MIN (wav->dataleft, MAX_BUFFER_SIZE);
+      if (!(buf = gst_riff_read_element_data (riff, desired, &got_bytes))) {
+        GST_WARNING_OBJECT (wav, "trying to read %d bytes failed", desired);
         return;
       }
+      GST_DEBUG_OBJECT (wav, "read %d bytes, got %d bytes", desired, got_bytes);
 
-      wavparse->dataleft -= got_bytes;
-      wavparse->byteoffset += got_bytes;
+      GST_BUFFER_TIMESTAMP (buf) = GST_SECOND *
+          (wav->datasize - wav->dataleft) / wav->bps;
+      GST_BUFFER_DURATION (buf) = GST_SECOND * got_bytes / wav->bps;
 
-      gst_bytestream_flush (bs, got_bytes);
+      gst_pad_push (wav->srcpad, GST_DATA (buf));
 
-      gst_pad_push (wavparse->srcpad, GST_DATA (buf));
-      return;
-    } else {
-      wavparse->state = GST_WAVPARSE_OTHER;
-    }
-  }
-
-  do {
-    gst_riff_riff *temp_chunk;
-    guint8 *tempdata;
-    guint32 skipsize;
-
-    /* read first two dwords to get chunktype and size */
-    while (TRUE) {
-      got_bytes =
-          gst_bytestream_peek_bytes (bs, &tempdata, sizeof (gst_riff_chunk));
-      temp_chunk = (gst_riff_riff *) tempdata;
-
-      if (got_bytes < sizeof (gst_riff_chunk)) {
-        if (!gst_wavparse_handle_sink_event (wavparse)) {
-          return;
-        }
+      wav->byteoffset += got_bytes;
+      if (got_bytes < wav->dataleft) {
+        wav->dataleft -= got_bytes;
+        return;
       } else {
-        break;
+        wav->dataleft = 0;
+        wav->state = GST_WAVPARSE_OTHER;
       }
+    } else {
+      wav->state = GST_WAVPARSE_OTHER;
     }
+  }
 
-    chunk.id = GUINT32_FROM_LE (temp_chunk->id);
-    chunk.size = GUINT32_FROM_LE (temp_chunk->size);
-
-    switch (chunk.id) {
-      case GST_RIFF_TAG_RIFF:
-      case GST_RIFF_TAG_LIST:
-        /* Read complete list chunk */
-        while (TRUE) {
-          got_bytes =
-              gst_bytestream_peek_bytes (bs, &tempdata, sizeof (gst_riff_list));
-          temp_chunk = (gst_riff_riff *) tempdata;
-          if (got_bytes < sizeof (gst_riff_list)) {
-            if (!gst_wavparse_handle_sink_event (wavparse)) {
-              return;
-            }
-          } else {
-            break;
-          }
-        }
-
-        chunk.type = GUINT32_FROM_LE (temp_chunk->type);
-        skipsize = sizeof (gst_riff_list);
-        break;
-
-      case GST_RIFF_TAG_cue:
-        skipsize = 0;
-        break;
-
-      default:
-        skipsize = sizeof (gst_riff_chunk);
-        break;
-    }
-    gst_bytestream_flush (bs, skipsize);
-  } while (FALSE);
-
-  /* need to flush an even number of bytes at the end */
-  flush = (chunk.size + 1) & ~1;
-
-  switch (wavparse->state) {
+  switch (wav->state) {
     case GST_WAVPARSE_START:
-      if (chunk.id != GST_RIFF_TAG_RIFF && chunk.type != GST_RIFF_RIFF_WAVE) {
-        GST_ELEMENT_ERROR (element, STREAM, WRONG_TYPE, (NULL),
-            ("chunk.id %08x chunk.type %08x", chunk.id, chunk.type));
+      if (!gst_wavparse_stream_init (wav)) {
         return;
       }
 
-      wavparse->state = GST_WAVPARSE_OTHER;
-      /* We are not going to flush lists */
-      flush = 0;
-      break;
+      wav->state = GST_WAVPARSE_FMT;
+      /* fall-through */
+
+    case GST_WAVPARSE_FMT:
+      if (!gst_wavparse_fmt (wav)) {
+        return;
+      }
+
+      wav->state = GST_WAVPARSE_OTHER;
+      /* fall-through */
 
     case GST_WAVPARSE_OTHER:
-      GST_DEBUG ("riff tag: %4.4s %08x", (char *) &chunk.id, chunk.size);
-
-      switch (chunk.id) {
-        case GST_RIFF_TAG_data:
-          wavparse->state = GST_WAVPARSE_DATA;
-          wavparse->dataleft = chunk.size;
-          wavparse->byteoffset = 0;
-          flush = 0;
-          break;
-
-        case GST_RIFF_TAG_fmt:
-          gst_wavparse_parse_fmt (wavparse, chunk.size);
-          flush = 0;
-          break;
-
-        case GST_RIFF_TAG_cue:
-          //gst_wavparse_parse_cues (wavparse, chunk.size);
-          break;
-
-        case GST_RIFF_TAG_LIST:
-          GST_DEBUG ("list type: %4.4s", (char *) &chunk.type);
-          switch (chunk.type) {
-            case GST_RIFF_LIST_INFO:
-              //gst_wavparse_parse_info (wavparse, chunk.size - 4);
-              //flush = 0;
-
-              break;
-
-            case GST_RIFF_LIST_adtl:
-              //gst_wavparse_parse_adtl (wavparse, chunk.size - 4);
-              //flush = 0;
-              break;
-
-            default:
-              //flush = 0;
-              break;
-          }
-
-        default:
-          GST_DEBUG ("  *****  unknown chunkid %08x", chunk.id);
-          //flush = 0;
-          break;
+      if (!gst_wavparse_other (wav)) {
+        return;
       }
+
       break;
 
     case GST_WAVPARSE_DATA:
-      /* Should have been handled up there ^^^^ */
-      flush = 0;
-      break;
 
     default:
-      /* Unknown */
-      g_warning ("Unknown state %d\n", wavparse->state);
-      //GST_DEBUG ("  *****  unknown chunkid %08x", chunk.id);
-      break;
-  }
-
-  if (flush > 0) {
-    gboolean res;
-
-    res = gst_bytestream_flush (bs, flush);
-    if (!res) {
-      guint32 remaining;
-      GstEvent *event;
-
-      gst_bytestream_get_status (bs, &remaining, &event);
-      gst_event_unref (event);
-    }
+      g_assert_not_reached ();
   }
 }
 
@@ -910,12 +817,8 @@ gst_wavparse_pad_convert (GstPad * pad,
     GstFormat src_format, gint64 src_value,
     GstFormat * dest_format, gint64 * dest_value)
 {
-  gint bytes_per_sample;
-  glong byterate;
+  guint bytes_per_sample, byterate;
   GstWavParse *wavparse;
-  const GstFormat *formats;
-  gboolean src_format_ok = FALSE;
-  gboolean dest_format_ok = FALSE;
 
   wavparse = GST_WAVPARSE (gst_pad_get_parent (pad));
 
@@ -925,68 +828,60 @@ gst_wavparse_pad_convert (GstPad * pad,
         wavparse->channels, wavparse->width);
     return FALSE;
   }
-  byterate = (glong) (bytes_per_sample * wavparse->rate);
+  byterate = wavparse->bps;
   if (byterate == 0) {
     g_warning ("byterate is 0, internal error\n");
     return FALSE;
   }
   GST_DEBUG ("bytes per sample: %d", bytes_per_sample);
-  /* check if both src_format and sink_format are in the supported formats */
-  formats = gst_pad_get_formats (pad);
-
-  while (formats && *formats) {
-    if (src_format == *formats) {
-      src_format_ok = TRUE;
-    }
-    if (*dest_format == *formats) {
-      dest_format_ok = TRUE;
-    }
-    formats++;
-  }
-  if (!src_format_ok || !dest_format_ok) {
-    GST_DEBUG ("src or dest format not supported");
-    return FALSE;
-  }
-
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
-      if (*dest_format == GST_FORMAT_DEFAULT)
-        *dest_value = src_value / bytes_per_sample;
-      else if (*dest_format == GST_FORMAT_TIME)
-        *dest_value = src_value * GST_SECOND / byterate;
-      else {
-        GST_DEBUG ("can't convert from bytes to other than units/time");
-        return FALSE;
+      switch (*dest_format) {
+        case GST_FORMAT_DEFAULT:
+          *dest_value = src_value / bytes_per_sample;
+          break;
+        case GST_FORMAT_TIME:
+          *dest_value = src_value * GST_SECOND / byterate;
+          break;
+        default:
+          return FALSE;
       }
-
+      *dest_value -= *dest_value % bytes_per_sample;
       break;
+
     case GST_FORMAT_DEFAULT:
-      if (*dest_format == GST_FORMAT_BYTES)
-        *dest_value = src_value * bytes_per_sample;
-      else if (*dest_format == GST_FORMAT_TIME)
-        *dest_value = src_value * GST_SECOND / wavparse->rate;
-      else {
-        GST_DEBUG ("can't convert from units to other than bytes/time");
-        return FALSE;
+      switch (*dest_format) {
+        case GST_FORMAT_BYTES:
+          *dest_value = src_value * bytes_per_sample;
+          break;
+        case GST_FORMAT_TIME:
+          *dest_value = src_value * GST_SECOND / wavparse->rate;
+          break;
+        default:
+          return FALSE;
       }
       break;
-    case GST_FORMAT_TIME:
-      if (*dest_format == GST_FORMAT_BYTES)
-        *dest_value = src_value * byterate / GST_SECOND;
-      else if (*dest_format == GST_FORMAT_DEFAULT)
-        *dest_value = src_value * wavparse->rate / GST_SECOND;
-      else {
-        GST_DEBUG ("can't convert from time to other than bytes/units");
-        return FALSE;
-      }
 
-      *dest_value = *dest_value & ~(bytes_per_sample - 1);
+    case GST_FORMAT_TIME:
+      switch (*dest_format) {
+        case GST_FORMAT_BYTES:
+          /* make sure we end up on a sample boundary */
+          *dest_value =
+              (src_value * wavparse->rate / GST_SECOND) * wavparse->blockalign;
+          break;
+        case GST_FORMAT_DEFAULT:
+          *dest_value = src_value * wavparse->rate / GST_SECOND;
+          break;
+        default:
+          return FALSE;
+      }
       break;
+
     default:
-      g_warning ("unhandled format for wavparse\n");
       return FALSE;
   }
+
   return TRUE;
 }
 
@@ -1007,28 +902,31 @@ static gboolean
 gst_wavparse_pad_query (GstPad * pad, GstQueryType type,
     GstFormat * format, gint64 * value)
 {
-  GstFormat peer_format = GST_FORMAT_BYTES;
-  gint64 peer_value;
-  GstWavParse *wavparse;
+  gint64 bytevalue;
+  GstWavParse *wav = GST_WAVPARSE (gst_pad_get_parent (pad));
 
-  /* probe sink's peer pad, convert value, and that's it :) */
-  /* FIXME: ideally we'd loop over possible formats of peer instead
-   * of only using BYTE */
+  /* only if we know */
+  if (wav->state != GST_WAVPARSE_DATA)
+    return FALSE;
 
-  /* only support byte, time and unit queries */
-  wavparse = GST_WAVPARSE (gst_pad_get_parent (pad));
-  if (!gst_pad_query (GST_PAD_PEER (wavparse->sinkpad), type,
-          &peer_format, &peer_value)) {
-    GST_DEBUG ("Could not query sink pad's peer");
-    return FALSE;
+  switch (type) {
+    case GST_QUERY_POSITION:
+      bytevalue = wav->datasize - wav->dataleft;
+      break;
+    case GST_QUERY_TOTAL:
+      bytevalue = wav->datasize;
+      break;
+    default:
+      return FALSE;
   }
-  if (!gst_pad_convert (wavparse->sinkpad, peer_format, peer_value,
-          format, value)) {
-    GST_DEBUG ("Could not convert sink pad's peer");
-    return FALSE;
+
+  if (*format == GST_FORMAT_BYTES) {
+    *value = bytevalue;
+    return TRUE;
   }
-  GST_DEBUG ("pad_query done, value %" G_GINT64_FORMAT "\n", *value);
-  return TRUE;
+
+  return gst_pad_convert (wav->sinkpad, GST_FORMAT_BYTES,
+      bytevalue, format, value);
 }
 
 static const GstEventMask *
@@ -1045,7 +943,6 @@ gst_wavparse_get_event_masks (GstPad * pad)
 static gboolean
 gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event)
 {
-#if 0
   GstWavParse *wavparse = GST_WAVPARSE (GST_PAD_PARENT (pad));
   gboolean res = FALSE;
 
@@ -1057,15 +954,8 @@ gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event)
       gint64 byteoffset;
       GstFormat format;
 
-      /* we can only seek when in the DATA state */
-      if (wavparse->state != GST_WAVPARSE_DATA) {
-        return FALSE;
-      }
-
+      /* bring format to samples for the peer element, */
       format = GST_FORMAT_BYTES;
-
-      /* bring format to bytes for the peer element, 
-       * FIXME be smarter here */
       res = gst_pad_convert (pad,
           GST_EVENT_SEEK_FORMAT (event),
           GST_EVENT_SEEK_OFFSET (event), &format, &byteoffset);
@@ -1074,7 +964,6 @@ gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event)
         /* ok, seek worked, update our state */
         wavparse->seek_offset = byteoffset;
         wavparse->seek_pending = TRUE;
-        wavparse->need_discont = TRUE;
       }
       break;
     }
@@ -1083,35 +972,42 @@ gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event)
   }
 
   gst_event_unref (event);
+
   return res;
-#else
-  return FALSE;
-#endif
 }
 
 static GstElementStateReturn
 gst_wavparse_change_state (GstElement * element)
 {
-  GstWavParse *wavparse = GST_WAVPARSE (element);
+  GstWavParse *wav = GST_WAVPARSE (element);
 
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_NULL_TO_READY:
       break;
+
     case GST_STATE_READY_TO_PAUSED:
-      wavparse->bs = gst_bytestream_new (wavparse->sinkpad);
-      wavparse->state = GST_WAVPARSE_START;
+      wav->state = GST_WAVPARSE_START;
       break;
+
     case GST_STATE_PAUSED_TO_PLAYING:
       break;
+
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
+
     case GST_STATE_PAUSED_TO_READY:
-      gst_bytestream_destroy (wavparse->bs);
-      wavparse->state = GST_WAVPARSE_UNKNOWN;
-      wavparse->bps = 0;
-      wavparse->seek_pending = FALSE;
-      wavparse->seek_offset = 0;
+      gst_wavparse_destroy_sourcepad (wav);
+      wav->state = GST_WAVPARSE_START;
+
+      wav->width = 0;
+      wav->depth = 0;
+      wav->rate = 0;
+      wav->channels = 0;
+
+      wav->seek_pending = FALSE;
+      wav->seek_offset = 0;
       break;
+
     case GST_STATE_READY_TO_NULL:
       break;
   }
@@ -1125,11 +1021,11 @@ gst_wavparse_change_state (GstElement * element)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_library_load ("gstbytestream")) {
+  if (!gst_library_load ("riff")) {
     return FALSE;
   }
 
-  return gst_element_register (plugin, "wavparse", GST_RANK_SECONDARY,
+  return gst_element_register (plugin, "wavparse", GST_RANK_PRIMARY,
       GST_TYPE_WAVPARSE);
 }
 
