@@ -456,6 +456,7 @@ gst_osselement_reset (GstOssElement * oss)
   oss->rate = 44100;
   oss->fragment = 0;
   oss->bps = 0;
+  oss->sample_width = 0;
 
 /* AFMT_*_BE not available on all OSS includes (e.g. FBSD) */
 #ifdef WORDS_BIGENDIAN
@@ -545,6 +546,7 @@ gst_osselement_parse_caps (GstOssElement * oss, const GstCaps * caps)
   gst_structure_get_int (structure, "channels", &oss->channels);
   gst_structure_get_int (structure, "rate", &oss->rate);
 
+  oss->sample_width = bps * oss->channels;
   oss->bps = bps * oss->channels * oss->rate;
   oss->format = format;
 
@@ -1126,7 +1128,7 @@ gst_osselement_probe_caps (GstOssElement * oss)
 
   if (gst_caps_is_empty (caps)) {
     GST_ELEMENT_ERROR (oss, RESOURCE, SETTINGS,
-        (_("Your oss device could not be probed correctly")), (NULL));
+        (_("Your OSS device could not be probed correctly")), (NULL));
     return;
   }
   GST_DEBUG ("probed caps: %" GST_PTR_FORMAT, caps);
@@ -1217,6 +1219,14 @@ gst_osselement_rate_probe_check (GstOssProbe * probe)
     }
   }
   n_checks++;
+  if (probe->min == -1 || probe->max == -1) {
+    /* This is a workaround for drivers that return -EINVAL (or another
+     * error) for rates outside of [8000,48000].  If this fails, the
+     * driver is seriously buggy, and probably doesn't work with other
+     * media libraries/apps.  */
+    probe->min = gst_osselement_rate_check_rate (probe, 8000);
+    probe->max = gst_osselement_rate_check_rate (probe, 48000);
+  }
   if (probe->min == -1 || probe->max == -1) {
     GST_DEBUG ("unexpected check_rate error");
     return FALSE;
