@@ -26,12 +26,25 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <sys/soundcard.h>
 #include <sys/ioctl.h>
 #include <errno.h>
 #include <unistd.h>
 #include <string.h>
 
+#ifdef HAVE_OSS_INCLUDE_IN_SYS
+#include <sys/soundcard.h>
+#else
+
+#ifdef HAVE_OSS_INCLUDE_IN_ROOT
+#include <soundcard.h>
+#else
+
+#include <machine/soundcard.h>
+
+#endif /* HAVE_OSS_INCLUDE_IN_ROOT */
+
+#endif /* HAVE_OSS_INCLUDE_IN_SYS */
+
 #include <gstosssrc.h>
 #include <gstosselement.h>
 #include <gst/audio/audioclock.h>
@@ -55,17 +68,22 @@ enum
 {
   ARG_0,
   ARG_BUFFERSIZE,
-  ARG_FRAGMENT,
+  ARG_FRAGMENT
 };
 
 static GstStaticPadTemplate osssrc_src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
-        "endianness = (int) BYTE_ORDER, "
+        "endianness = (int) { LITTLE_ENDIAN, BIG_ENDIAN }, "
         "signed = (boolean) { TRUE, FALSE }, "
-        "width = (int) { 8, 16 }, "
+        "width = (int) 16, "
         "depth = (int) { 8, 16 }, "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]; "
+        "audio/x-raw-int, "
+        "signed = (boolean) { TRUE, FALSE }, "
+        "width = (int) 8, "
+        "depth = (int) 8, "
         "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 2 ]")
     );
 
@@ -317,6 +335,7 @@ gst_osssrc_get (GstPad * pad)
   GstOssSrc *src;
   GstBuffer *buf;
   glong readbytes;
+  glong readsamples;
 
   src = GST_OSSSRC (gst_pad_get_parent (pad));
 
@@ -358,17 +377,34 @@ gst_osssrc_get (GstPad * pad)
     return GST_DATA (gst_event_new (GST_EVENT_INTERRUPT));
   }
 
+  readsamples = readbytes * GST_OSSELEMENT (src)->rate /
+      GST_OSSELEMENT (src)->bps;
+
   GST_BUFFER_SIZE (buf) = readbytes;
   GST_BUFFER_OFFSET (buf) = src->curoffset;
-
-  /* FIXME: we are falsely assuming that we are the master clock here */
-  GST_BUFFER_TIMESTAMP (buf) =
-      src->curoffset * GST_SECOND / GST_OSSELEMENT (src)->bps;
+  GST_BUFFER_OFFSET_END (buf) = src->curoffset + readsamples;
   GST_BUFFER_DURATION (buf) =
-      (GST_SECOND * GST_BUFFER_SIZE (buf)) / (GST_OSSELEMENT (src)->bps *
-      GST_OSSELEMENT (src)->rate);
+      readsamples * GST_SECOND / GST_OSSELEMENT (src)->rate;
 
-  src->curoffset += readbytes;
+  /* if we have a clock */
+  if (src->clock) {
+    if (src->clock == src->provided_clock) {
+      /* if it's our own clock, we can be very accurate */
+      GST_BUFFER_TIMESTAMP (buf) =
+          src->curoffset * GST_SECOND / GST_OSSELEMENT (src)->rate;
+    } else {
+      /* somebody elses clock, timestamp with that clock, no discontinuity in
+       * the stream since the OFFSET is updated correctly. Elements can stretch
+       * to match timestamps */
+      GST_BUFFER_TIMESTAMP (buf) =
+          gst_element_get_time (GST_ELEMENT (src)) - GST_BUFFER_DURATION (buf);
+    }
+  } else {
+    /* no clock, no timestamp */
+    GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
+  }
+
+  src->curoffset += readsamples;
 
   GST_DEBUG ("pushed buffer from soundcard of %ld bytes, timestamp %"
       G_GINT64_FORMAT, readbytes, GST_BUFFER_TIMESTAMP (buf));
