@@ -67,13 +67,11 @@ struct _GstOssSrc {
 };
 
 struct _GstOssSrcClass {
-  GstElementClass parent_class;
+  GstOssElementClass parent_class;
 };
 
 GType gst_osssrc_get_type(void);
 
-gboolean gst_osssrc_factory_init (GstPlugin *plugin);
-
 G_END_DECLS
 
 #endif /* __GST_OSSSRC_H__ */
