@@ -1,4 +1,7 @@
-/* G-Streamer hardware MJPEG video source plugin
+/* GStreamer
+ *
+ * gstv4lmjpegsrc.c: hardware MJPEG video source plugin
+ *
  * Copyright (C) 2001-2002 Ronald Bultje <rbultje@ronald.bitfreak.net>
  *
  * This library is free software; you can redistribute it and/or
@@ -24,6 +27,9 @@
 #include <string.h>
 #include "v4lmjpegsrc_calls.h"
 
+GST_DEBUG_CATEGORY (v4lmjpegsrc_debug);
+#define GST_CAT_DEFAULT v4lmjpegsrc_debug
+
 /* elementfactory information */
 static GstElementDetails gst_v4lmjpegsrc_details = {
   "Video (video4linux/MJPEG) Source",
@@ -121,6 +127,7 @@ gst_v4lmjpegsrc_get_type (void)
       (GInstanceInitFunc) gst_v4lmjpegsrc_init,
       NULL
     };
+
     v4lmjpegsrc_type =
         g_type_register_static (GST_TYPE_V4LELEMENT, "GstV4lMjpegSrc",
         &v4lmjpegsrc_info, 0);
@@ -186,7 +193,7 @@ gst_v4lmjpegsrc_class_init (GstV4lMjpegSrcClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_USE_FIXED_FPS,
       g_param_spec_boolean ("use_fixed_fps", "Use Fixed FPS",
           "Drop/Insert frames to reach a certain FPS (TRUE) "
-          "or adapt FPS to suit the number of frabbed frames",
+          "or adapt FPS to suit the number of grabbed frames",
           TRUE, G_PARAM_READWRITE));
 
   /* signals */
@@ -207,6 +214,8 @@ gst_v4lmjpegsrc_class_init (GstV4lMjpegSrcClass * klass)
       G_STRUCT_OFFSET (GstV4lMjpegSrcClass, frame_lost), NULL, NULL,
       g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
 
+  GST_DEBUG_CATEGORY_INIT (v4lmjpegsrc_debug, "v4lmjpegsrc", 0,
+      "V4L MJPEG source element");
   gobject_class->set_property = gst_v4lmjpegsrc_set_property;
   gobject_class->get_property = gst_v4lmjpegsrc_get_property;
 
@@ -565,7 +574,8 @@ gst_v4lmjpegsrc_get (GstPad * pad)
   GST_BUFFER_DATA (buf) = gst_v4lmjpegsrc_get_buffer (v4lmjpegsrc, num);
   GST_BUFFER_SIZE (buf) = v4lmjpegsrc->last_size;
   GST_BUFFER_MAXSIZE (buf) = v4lmjpegsrc->breq.size;
-  GST_BUFFER_FLAG_SET (buf, GST_BUFFER_READONLY | GST_BUFFER_DONTFREE);
+  GST_BUFFER_FLAG_SET (buf, GST_BUFFER_READONLY);
+  GST_BUFFER_FLAG_SET (buf, GST_BUFFER_DONTFREE);
   if (v4lmjpegsrc->use_fixed_fps)
     GST_BUFFER_TIMESTAMP (buf) = v4lmjpegsrc->handled * GST_SECOND / fps;
   else                          /* calculate time based on our own clock */
