@@ -1,4 +1,7 @@
-/* G-Streamer generic V4L element - generic V4L overlay handling
+/* GStreamer
+ *
+ * v4l-overlay_calls.c: calls for generic V4L overlay handling
+ *
  * Copyright (C) 2001-2002 Ronald Bultje <rbultje@ronald.bitfreak.net>
  *
  * This library is free software; you can redistribute it and/or
@@ -24,6 +27,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
@@ -31,11 +35,8 @@
 #include <errno.h>
 #include "v4l_calls.h"
 
-#define DEBUG(format, args...) \
-	GST_DEBUG_OBJECT (\
-		GST_ELEMENT(v4lelement), \
-		"V4L-overlay: " format, ##args)
-
+GST_DEBUG_CATEGORY (v4loverlay_debug);
+#define GST_CAT_DEFAULT v4loverlay_debug
 
 /******************************************************
  * gst_v4l_set_overlay():
@@ -47,22 +48,34 @@ gboolean
 gst_v4l_set_overlay (GstV4lElement * v4lelement)
 {
   gchar *buff;
+  gchar *path;
+  gint ret;
 
   if (v4lelement->display)
     g_free (v4lelement->display);
   v4lelement->display = g_strdup (g_getenv ("DISPLAY"));
 
-  DEBUG ("setting display to '%s'", v4lelement->display);
+  GST_DEBUG_OBJECT (v4lelement, "setting display to '%s'", v4lelement->display);
   GST_V4L_CHECK_NOT_OPEN (v4lelement);
 
   if (!v4lelement->display || v4lelement->display[0] != ':')
     return FALSE;
 
   /* start v4l-conf */
+  path = g_find_program_in_path ("v4l-conf");
+  if (!path) {
+    GST_ELEMENT_ERROR (v4lelement, RESOURCE, FAILED,
+        (_("Program 'v4l-conf' missing from path.")),
+        ("Cannot set XVideo overlay mode."));
+    return FALSE;
+  }
+  g_free (path);
+
   buff = g_strdup_printf ("v4l-conf -q -c %s -d %s",
       v4lelement->videodev, v4lelement->display);
 
-  switch (system (buff)) {
+  ret = system (buff);
+  switch (ret) {
     case -1:
       GST_ELEMENT_ERROR (v4lelement, RESOURCE, FAILED,
           (_("Could not start v4l-conf.")), GST_ERROR_SYSTEM);
@@ -71,10 +84,16 @@ gst_v4l_set_overlay (GstV4lElement * v4lelement)
     case 0:
       break;
     default:
-      GST_ELEMENT_ERROR (v4lelement, RESOURCE, FAILED,
-          (_("Executing v4l-conf failed.")), GST_ERROR_SYSTEM);
+    {
+      /* if we get here, the system command did not fail but v4l-conf
+       * returned an error code, we just warn for now because it is not
+       * always fatal (like not having overlay support) */
+      gint status = WEXITSTATUS (ret);
+
+      g_warning ("v4l-conf returned %d.", status);
       g_free (buff);
-      return FALSE;
+      return TRUE;
+    }
   }
 
   g_free (buff);
@@ -95,8 +114,8 @@ gst_v4l_set_window (GstElement * element,
   GstV4lElement *v4lelement = GST_V4LELEMENT (element);
   struct video_window vwin;
 
-  DEBUG ("setting video window to position (x,y/wxh) = %d,%d/%dx%d",
-      x, y, w, h);
+  GST_DEBUG_OBJECT (v4lelement,
+      "setting video window to position (x,y/wxh) = %d,%d/%dx%d", x, y, w, h);
   GST_V4L_CHECK_OPEN (v4lelement);
   GST_V4L_CHECK_OVERLAY (v4lelement);
 
@@ -107,7 +126,7 @@ gst_v4l_set_window (GstElement * element,
   vwin.flags = 0;
 
   if (clips && !(v4lelement->vcap.type & VID_TYPE_CLIPPING)) {
-    DEBUG ("Device \'%s\' doesn't do clipping",
+    GST_DEBUG_OBJECT (v4lelement, "Device \'%s\' doesn't do clipping",
         v4lelement->videodev ? v4lelement->videodev : "/dev/video");
     vwin.clips = 0;
   } else {
@@ -136,7 +155,8 @@ gst_v4l_enable_overlay (GstV4lElement * v4lelement, gboolean enable)
 {
   gint doit = enable ? 1 : 0;
 
-  DEBUG ("%s overlay", enable ? "enabling" : "disabling");
+  GST_DEBUG_OBJECT (v4lelement, "%s overlay",
+      enable ? "enabling" : "disabling");
   GST_V4L_CHECK_OPEN (v4lelement);
   GST_V4L_CHECK_OVERLAY (v4lelement);
 
