@@ -47,6 +47,7 @@ GST_DEBUG_CATEGORY_EXTERN (v4l_debug);
 
 #define GST_CAT_DEFAULT v4l_debug
 
+#ifndef GST_DISABLE_GST_DEBUG
 /* palette names */
 static const char *palette_name[] = {
   "",                           /* 0 */
@@ -67,6 +68,7 @@ static const char *palette_name[] = {
   "YUV-4:2:0 (planar)",         /* VIDEO_PALETTE_YUV420P */
   "YUV-4:1:0 (planar)"          /* VIDEO_PALETTE_YUV410P */
 };
+#endif
 
 /******************************************************
  * gst_v4lsrc_queue_frame():
@@ -152,11 +154,7 @@ gst_v4lsrc_set_capture (GstV4lSrc * v4lsrc,
     gint width, gint height, gint palette)
 {
   GST_DEBUG_OBJECT (v4lsrc,
-      "capture properties set to width = %d, height = %d, palette = %d",
-      width, height, palette);
-
-  /*GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lsrc)); */
-  /*GST_V4L_CHECK_NOT_ACTIVE(GST_V4LELEMENT(v4lsrc)); */
+      "capture properties set to %dx%d, palette %d", width, height, palette);
 
   v4lsrc->mmap.width = width;
   v4lsrc->mmap.height = height;
@@ -496,8 +494,8 @@ gst_v4lsrc_try_capture (GstV4lSrc * v4lsrc, gint width, gint height,
   vmmap.frame = frame;
   if (ioctl (GST_V4LELEMENT (v4lsrc)->video_fd, VIDIOCMCAPTURE, &vmmap) < 0) {
     if (errno != EINVAL)        /* our format failed! */
-      GST_ELEMENT_ERROR (v4lsrc, RESOURCE, OPEN_READ_WRITE, (NULL),
-          ("Error queueing our try-out buffer: %s", g_strerror (errno)));
+      GST_ERROR_OBJECT (v4lsrc,
+          "Error queueing our try-out buffer: %s", g_strerror (errno));
     munmap (buffer, vmbuf.size);
     return FALSE;
   }
