@@ -147,7 +147,7 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   }
 
   xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, 0, 0);
-  if ((int) xvimage->SHMInfo.shmaddr == -1) {
+  if (xvimage->SHMInfo.shmaddr == ((void *) -1)) {
     GST_WARNING ("Failed to shmat: %s", g_strerror (errno));
     goto beach;
   }
@@ -163,9 +163,14 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   XSync (xcontext->disp, 0);
 
   XShmDetach (xcontext->disp, &xvimage->SHMInfo);
+  XSync (xcontext->disp, FALSE);
+
   shmdt (xvimage->SHMInfo.shmaddr);
   shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
 
+  /* To be sure, reset the SHMInfo entry */
+  xvimage->SHMInfo.shmaddr = ((void *) -1);
+
   /* store whether we succeeded in result and reset error_caught */
   result = !error_caught;
   error_caught = FALSE;
@@ -225,7 +230,7 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink,
     }
 
     xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, 0, 0);
-    if ((int) xvimage->SHMInfo.shmaddr == -1) {
+    if (xvimage->SHMInfo.shmaddr == ((void *) -1)) {
       GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
           ("Failed to shmat: %s", g_strerror (errno)));
       goto beach;
@@ -287,8 +292,9 @@ gst_xvimagesink_xvimage_destroy (GstXvImageSink * xvimagesink,
 
 #ifdef HAVE_XSHM
   if (xvimagesink->xcontext->use_xshm) {
-    if ((int) xvimage->SHMInfo.shmaddr != -1) {
+    if (xvimage->SHMInfo.shmaddr != ((void *) -1)) {
       XShmDetach (xvimagesink->xcontext->disp, &xvimage->SHMInfo);
+      XSync (xvimagesink->xcontext->disp, FALSE);
       shmdt (xvimage->SHMInfo.shmaddr);
     }
     if (xvimage->SHMInfo.shmid > 0)
